FUNCTION ECKV2Side_cos2S_k_phi, k,phirad,U10,Sparam

; Given a positive angular wavenumber k = |kvector| in rad/m, angle phi in radians, -pi <= phi <= pi, 
; and wind speed U10 in m/s, this routine returns the value of a TWO-SIDED directional gravity-capillary 
; energy spectrum Psi(k,phi).
   
; For use in rectangular (kx,ky) grids, E2S(kx,ky) = Psi(k=sqrt(kx^2+ky^2), phi=atan(ky/kx))

; The one-sided omnidirectional spectrun is that of 
; Elfouhaily, T., B. Chapron, K. Katsaros, and D. Vandmark, 1997. A unified directional spectrum for 
; long and short wind-driven waves. J Geophys Res, vol 102(C7), 15781-15796.
; Psi(k,phi) is given by ECKV Eq. (67) and related equations.

; Either the original ECKV spreading function or a Longuet-Higgins cos^2S(phi/2) spreading function
; can be used.

; This returns a TWO-sided energy spectrum, asymmetric in upwind and downwind directions,
; valid for wavenumbers in the full gravity-capillary range.  The magnitude of the 2-sided
; spectrum equals the ECKV omnidirectional spectrum, since the total energy is mostly
; in the downwind half of the 2-sided spectrum.

; Input:
;   k = the magnitude of the angular wavenumber k, in rad/m
;   phirad = the angle in rad, -pi to pi; phi = 0 is downwind, phi = +/-pi is upwind 
;   U10 = the wind speed at 10 m above sea level
;   Sparam = the spreading function parameter
;       a negative Sparam gives the symmetric ECKV spreading function
;       a positive Sparam give the S value of the cos-2S family

; Output:
;   E2S = Psi(k,phi) = the two-sided energy spectrum for the given (k,phi) values and U10, 
;         in units of m^2/(rad/m)^2 (shown as m^4/rad^2 in ECKV Table A1)
 
;------ parameters defining the sea state

g = 9.81

; Omegac = 0.84 for a "fully developed" sea (corresponds to Pierson-Moskowicz)
;        = 1 for a "mature" sea (used in ECKV Fig 8a)
;        = 2 to 5 for a "young" sea; max allowed value is 5
Omegac = 0.84

;  convert U10 = wind at 10 m to the friction velocity u* using ECKV Eq. 61
Cd10N = 0.00144           ; value deduced from ECKV Fig 11
ustar = sqrt(Cd10N)*U10

; values from ECKV Eq. 59
ao = 0.1733
ap = 4.0
cm = 0.23
am = 0.13*ustar/cm
km = 370.0     ; corresponds to lambda = 1.7 cm for gravity-capillary wave boundary

if (Omegac le 1.0) then begin
   gammac = 1.7
endif else begin
   gammac = 1.7 + 6.0*alog10(Omegac)
endelse

sigma = 0.08*(1.0 + 4.0*Omegac^(-3))
twosig2 = 1.0/(2.0*sigma*sigma)
alphap = 0.006*Omegac^(0.55) ; ECKV Eq. 34

; ECKV Eq. 44:
if(ustar le cm) then begin
   alpham = 0.01*(1.0 + alog(ustar/cm))
endif else begin
   alpham = 0.01*(1.0 + 3.0*alog(ustar/cm))
endelse

;  get the wavelength corresponding to the max of the wavenumber spectrum
ko = g/(U10*U10)
kp = ko*Omegac*Omegac
wavemax = 2.0*!pi/kp

cp = sqrt(g/kp)
Capomega = Omegac   ; = U10/cp for thetabar = 0

;------ evaluate the ECKV omnidirectional S(k) spectrum for the given k value

c = sqrt((g/k)*(1.0 + (k/km)^2)) ; = sqrt(g/k + Tw*k/rhow)
sqrtkkp = sqrt(k/kp)

;  the low-frequency (long-wave) part  B_l of Eq. (31)
Capgamma = exp(-twosig2*(sqrtkkp - 1.0)^2) ; below Eq. (3)
fJp = gammac^Capgamma ; Eq. (3)
fLpm = exp(-1.25*(kp/k)^2) ; Eq. (2)
Fp = fLpm*fJp*exp(-0.3162*Capomega*(sqrtkkp - 1.0)) ; Eq. (32)
Bl = 0.5*alphap*(cp/c)*Fp    ; sqrt(k/kp) = cp/c
;Slow = Bl/k^3

; the high-frequency (short-wave) part B_h of Eq. (40)
; NB: A typo in Eq. (41) of the Elfouhaily paper omits the Lpm*Jp factor
; from the definition of Fm; compare with Fp in Eq. (32)
Fm = fLpm*fJp*exp(-0.25*(k/km - 1.0)^2) ; Eq. (41) with typo fixed
Bh = 0.5*alpham*(cm/c)*Fm ; Eq. (40)
;Shigh = Bh/k^3

;----- Choose and evaluate an angular spreading function:

IF Sparam lt 0.0 THEN BEGIN
; the original ECKV spreading function:  
Deltak = tanh( ao + ap*(c/cp)^(2.5) + am*(cm/c)^(2.5) ) ; Eq. (57)
Spread = 0.5*(1.0 + Deltak*cos(2.0*phirad))/(2.0*!pi) ; 0.5 for 2 sided 
 
ENDIF ELSE BEGIN
 
; a cos2s spreading function

Option = 1
; Option 1: use a fixed value for S
CASE Option OF
1: BEGIN
;S = 0.68 ; min value for capillary waves
S = Sparam ;10.0 ; typical value for long gravity waves
Cnorm = (0.5/sqrt(!pi))*GAMMA(S + 1.0)/GAMMA(S + 0.5)
  ;Cnorm = 4.0/(3.0*!pi) ; the value for S = 2.0
Spread = Cnorm*(cos(0.5*phirad))^(2.0*S)
 if abs(abs(phirad) - !pi) lt 1.0e-6 then Spread = 0.0 ; where cos(pi/2) = 0.0 
   END
   
2: BEGIN 
; WARNING: This S(k) function often gives S so big that E2S is NaN.  Needs fixing.  
; Option 2: Use the k-dependent spreading parameter S from Heron et al. Eq. (11) and the ECKV Deltak function
; c = inf if k = 0, so S = inf also.  But E2D = undefined for k = 0, so set Spread = 0 also.
IF (k gt 1.0e-6) THEN BEGIN
  S = 2.0/ALOG(2.0)*(ao + ap*(c/cp)^(2.5) + am*(cm/c)^(2.5))
  Cnorm = (0.5/sqrt(!pi))*GAMMA(S + 1.0)/GAMMA(S + 0.5)
  ;print, 'k,phirad,S, Cnorm = ',k,phirad,S, Cnorm,Spread
  ;Cnorm = 4.0/(3.0*!pi) ; the value for S = 2.0
  Spread = Cnorm*(cos(0.5*phirad))^(2.0*S)
  if abs(abs(phirad) - !pi) lt 1.0e-6 then Spread = 0.0 ; where cos(pi/2) = 0.0
  print, 'k,phirad,S, Cnorm,Spread = ',k,phirad,S, Cnorm,Spread
ENDIF ELSE BEGIN
  Spread = 0.0
  print,'k = 0 found'
ENDELSE    
   END
   
3: BEGIN

   if (k lt 1.0e-6) then begin
     Spread = 0.0
     goto, kzero
   endif
     
   ; an ad hoc function based on Mitsuyasu and Mizuno with min and max limits added
   S = 11.5*(g*k)^(-1.25)
   IF S gt 20 then S = 20
   If S lt 0.68 then S = 0.68
   Cnorm = (0.5/sqrt(!pi))*GAMMA(S + 1.0)/GAMMA(S + 0.5)
   Spread = Cnorm*(cos(0.5*phirad))^(2.0*S)
   if abs(abs(phirad) - !pi) lt 1.0e-6 then Spread = 0.0 ; where cos(pi/2) = 0.0
   ;print, 'opt 3: k,phirad,S, Cnorm,Spread = ',k,phirad,S, Cnorm,Spread
   if (k lt 1.0e-6) then Spread = 0.0

   END
ENDCASE
   kzero: dummy = 0 ; dummy statement for the go to statement

ENDELSE
         
; combine as in ECDV Eq. (67) to get E2S(k,phi) 
; (Note: this is Psi(k,phi) = (1/k)S(k)Phi(k,phi) of ECKV, so 1/k^4)
  
E2D = ( (Bl + Bh)/k^4 ) * Spread
  
return, E2D

END
