FUNCTION ECKVomni_k,k,U10,Omegac

; Given the angular wavenumber k in rad/m and wind speed U10 in m/s, this routine returns the 
; omnidirectional gravity-capillary energy spectrum E(k) = the S(k) of Elfouhaily, T., B. Chapron, K. Katsaros, 
; and D. Vandmark, 1997. A unified directional spectrum for long and short wind-driven waves.
; J Geophys Res, vol 102(C7), 15781-15796.
; S(k) is given by ECKV Eq. (30) and related equations.
; This is a one-sided energy spectrum valid for positive wavenumbers in the full gravity-capillary range.

; Input:
;   k = the angular wavenumber k in rad/m
;       k can be a single wavenumber or an array of positive wavenumbers
;   U10 = the wind speed at 10 m above sea level
;         Note: U10 must be passed via a common block if the function is called by a numerical
;             integration routine such as qromb, which requires as input a function routine 
;             with just one argument for the abscissa
;   Omegac = the age parameter:
;          = 0.84 for a "fully developed" sea (corresponds to Pierson-Moskowicz)
;          = 1 for a "mature" sea (used in ECKV Fig 8a)
;          = 2 to 5 for a "young" sea; max allowed value is 5

; Output:
;   E(k) = the one-sided energy spectrum for the given k and U10, units of m^2/(rad/m) = m^3/rad in ECKV Table A1
;          This E(k) is the S(k) of ECKV Fig 8(a), which appears to be for Omegac = 1.0,
;          and of ECKV Fig 11(a), which is for Omegac = 0.84.
 
; optional common block (if used, comment out ",U10" in the function call above
; common common_ECKV, U10 
                    
; use this common block if the high and slow wavenumber components are separately needed
;common common_ECKV, U10,Slow,Shigh

g = 9.82

; Omegac = 0.84 for a "fully developed" sea (corresponds to Pierson-Moskowicz)
;        = 1 for a "mature" sea (used in ECKV Fig 8a)
;        = 2 to 5 for a "young" sea; max allowed value is 5
IF(Omegac lt 0.84 or Omegac gt 5.0) then begin
   print,"ERROR: ECKVomni_k routine: Omegac =',e10.3, ' is outside the allowed range of 0.84 to 5.0; results may be in error'
ENDIF

;  convert U10 = wind at 10 m to the friction velocity u* using ECKV Eq. 61
Cd10N = 0.00144           ; value deduced from ECKV Fig 11
ustar = sqrt(Cd10N)*U10

; values from ECKV Eq. 59
ao = 0.1733
ap = 4.0
cm = 0.23
am = 0.13*ustar/cm
km = 370.0     ; corresponds to lambda = 1.7 cm for gravity-capillary wave boundary

if (Omegac le 1.0) then begin
   gamma = 1.7
endif else begin
   gamma = 1.7 + 6.0*alog10(Omegac)
endelse

sigma = 0.08*(1.0 + 4.0*Omegac^(-3))
twosig2 = 1.0/(2.0*sigma*sigma)
alphap = 0.006*Omegac^(0.55) ; ECKV Eq. 34

; ECKV Eq. 44:
if(ustar le cm) then begin
   alpham = 0.01*(1.0 + alog(ustar/cm))
endif else begin
   alpham = 0.01*(1.0 + 3.0*alog(ustar/cm))
endelse

;  get the wavelength corresponding to the max of the wavenumber spectrum
ko = g/(U10*U10)
kp = ko*Omegac*Omegac
wavemax = 2.0*!pi/kp

cp = sqrt(g/kp)
Capomega = Omegac   ; = U10/cp for thetabar = 0

;  define the S(k) spectrum for the given k value

c = sqrt((g/k)*(1.0 + (k/km)^2)) ; = sqrt(g/k + Tw*k/rhow)
sqrtkkp = sqrt(k/kp)

;  the low-frequency (long-wave) part
Capgamma = exp(-twosig2*(sqrtkkp - 1.0)^2)
fJp = gamma^Capgamma
fLpm = exp(-1.25*(kp/k)^2)
Fp = fLpm*fJp*exp(-0.3162*Capomega*(sqrtkkp - 1.0))
Bl = 0.5*alphap*(cp/c)*Fp    ; sqrt(k/kp) = cp/c
;Slow = Bl/k^3

; the high-frequency (short-wave) part
; NB: A typo in Eq. (41) of the Elfouhaily paper omits the Lpm*Jp factor
; from the definition of Fm; compare with Fp in Eq. (32)
Fm = fLpm*fJp*exp(-0.25*(k/km - 1.0)^2)
Bh = 0.5*alpham*(cm/c)*Fm
;Shigh = Bh/k^3

; return the omnidirectional elevation spectrum from ECKV (30):

return, (Bl + Bh)/k^3

END
