FUNCTION Horoshenkov_S2snu, nu, sigmaw,Lo,var

; Given the spatial frequency nu and spectrum parameters sigmaw, Lo, and var, this routine returns 
; the two-sided elevation variance spectrum corresponding to W(rho) (times the variance) in 
; Horoshenkov, et al, 2013. The pattern of surface waves in a shallow free surface flow. 
; JGR Earth Surf, 118, 1864-1876.

; NOTE: Horoshenkov has two versions of the W(rho) equation: The abstract and Section 4 versions have a 1/2 in
; the exponential term; the Eq. (4) version does not.  I don't know which is correct.  This routine uses
; the version with the 1/2 in the exponent.

; Input:
;   nu = the spatial wavenumber in 1/m for which S2s(nu) is to be returned
;       nu can be a single number or a 1D array with pos or neg values
;   sigmaw = the correlation radius, in m, e.g. from Horoshenkov Table 3
;   Lo = the characteristic length, in m, e.g. from Horoshenkov Table 3
;   var = the variance of the water surface, in m^2, which converts the H autocorrelation to an autocovariance
;   Note: sigmaw, Lo, and var must be passed via a common block if the function is called by a numerical
;         integration routine such as qromb, which requires as input a function routine
;         with just one argument for the abscissa

; Output:
;   S2s(nu) = the two-sided energy spectrum for the given inputs, units of m^2/(1/m)

; Written 07 April 2016 by Curtis Mobley

;**********************

A = sqrt(!pi/2.0)*sigmaw*var
B = 0.5*(2.0*!pi*sigmaw)^2 

; Define S2s(nu).  This is the analytic Fourier transform of the Horoshenkov Eq(4) autocovariance

S2snu = A*( exp(-B*(nu + 1.0/Lo)^2) + exp(-B*(nu - 1.0/Lo)^2) )

RETURN, S2snu

END