FUNCTION Pierson_Moskowitz_k,k,U10

; Given the angular wavenumber k [rad/m] and wind speed U10 [m/s], this routine returns the one-sided 
; Pierson-Moskowitz ; gravity-wave energy spectrum E(k) = S(omega(k)) of Pierson, W. J. and 
; L. Moskowitz, 1964. A proposed spectral form for fully developed wind seas ; based on the 
; similarity theory of S. A. Kitaigorodskii ; J Geophys Res, vol 69, 5181-5190.

; This is a one-sided energy spectrum valid for positive wavenumbers in the gravity-wave range.
; It should not be used for short-wavelength gravity waves or for capillary waves.

; The formula for S(omega) in temporal angular freq omega = 2pi/time, in rad/sec or Hz, given by P-M Eq. 12 
; was used with the dispersion relation omega^2 = g k to convert to radial angular wavenumber 
; k = 2 pi/wavelength, in rad/m

; Input:
;   k = the angular spatial wavenumber in rad/m
;      k can be a single wavenumber or an array of positive wavenumbers
;   U10 = the wind speed at 10 m above sea level (U was at 19.5 m in PM64)
;         Note: U10 must be passed via a common block if the function is called by a numerical
;             integration routine such as qromb, which requires as input a function routine 
;             with just one argument for the abscissa

; Output:
;   E(k) = the one-sided energy spectrum for the given k and U10, units of m^2/m^(-1)
 
; optional common block (if used, comment out ",U10" in the function call above
; common common_PM, U10 
                       
alpha = 0.0081 ; Phillips const
beta = 0.74
g = 9.82 ; accel of gravity, m/s^2

C1 = alpha/2.0
C2 = beta*g*g
U19 = 1.026*U10 ; convert U at 10 m to 19.5 m as used in P-M formulas (Stewart, Ocean Waves, Eq. 16.31)

; E(omega) from Eq. (12) of P-M 1964 written in terms of wind speed
; (see Stewart, Intro to Phys Ocean, Eq. (16.30):
;EPMomega = (alpha*g*g/omega^5)*exp(-beta*(g/(omega*U19))^4) 

; E(k) after change of variables from omega to k:
EPMk = C1*k^(-3)*exp(-C2*(k^(-2))*U19^(-4)) ; values for positive wavenumbers

RETURN, EPMk

END
