@Horoshenkov_S2snu
@Horoshenkov_Czz
@LogLabels

pro WK_H_nu ; Wiener-Khinchin, Horoshenkov, nu as spatial frequency variable

; This routine is based on WK_PM_nu.pro, with the Pierson-Moskowitz S2s(nu) 
; spectrum replaced by the Horoshenkov spectrum

; This routine created Fig 5.8 of the Tutorial

; ***************************************************************************************************
; * This code is copyright (c) 2016 by Curtis D. Mobley.                                            *
; * Permission is hereby given to reproduce and use this code for non-commercial academic research, *
; * provided that the user suitably acknowledges Curtis D. Mobley in any presentations, reports,    *
; * publications, or other works that make use of the code or its output.  Depending on the extent  *
; * of use of the code or its outputs, suitable acknowledgement can range from a footnote to offer  *
; * of coauthorship.  Further questions can be directed to curtis.mobley@sequoiasci.com.            *
; ***************************************************************************************************

;-----------------------------------------------------------------------------------------------------------
;| This routine uses Coyote Graphics routines rather than standard IDL for plotting; see www.idlcoyote.com |
;| and "Coyote's Guide to Traditional IDL Graphics: Using Familiar Tools Creatively" by David Fanning      |
;| Any comments below like "CG page xxx" refer to this book.                                               |
;| CG routines are wrappers for standard IDL routines, which make it easier to do certain things like      |
;| create vector eps output files; cgPlot replaces the standard Plot, etc.                                 |
;-----------------------------------------------------------------------------------------------------------

; Completed 14 April 2016 by Curtis Mobley.

;-----------------  Begin user input

; avg values from Horoshenkov Table 3
sigmaw =  0.22 ; Correlation radius in meters
Lo = 0.17 ; Characteristic spatial period in meters
var = 2.5e-7 ; surface elevation variance in meters (0.5 mm std dev; see Tab 1)

L = 4.0 ; length of surface region in meters (= wavelength of largest resolvable wave)
Nsamp = 1024 ; number of samples of sea surface elevation to be generated; should be a power of 2 for the FFT        
Nsurf = 1L ; number of surface realizations ; must be long-word integer if more than 32K 
seed = 5838392L; 538832L; 33202L; 838832L 

PlotFileName = 'WK-H-nu-1024-1surf.eps'

rootname = STRMID(Plotfilename,4,STRLEN(Plotfilename)-8) ; used for saving files
idebug = 0 ; = 1 for extra debugging printout; 0 otherwise
iplotsurfs = 1 ; = 1 to plot surfaces (takes a long time for Nsurf > 1000L, so can skip)

;-----------------  End of user input

Deltax = L/float(Nsamp) ; x spacing of the generated zsurf(x) points
minwave = 2.0*Deltax ; min resolvable wavelength in m (2 point wave)
numin = 1.0/L ; min (fundamental) spatial frequency in 1/m; frequency of longest resolvable wave
nuNyquist = 1.0/(2.0*Deltax) ; Nyquist spatial freq 1/(2 Delta) in 1/m
numax = nuNyquist; Nyquist
x = Deltax*findgen(Nsamp) ; surface x values
qo = 2.0*!pi/Lo

print,' '
print,' Study of the Wiener-Khinchin theorem using the Horoshenkov autocovariance and variance spectrum S(nu)'
print,' '
print, format='(4x,a,f8.1)', 'Max resolvable wavelength = L [m] =',L
print, format='(4x,a,i5)', 'Number of spatial samples = Nsamp =',Nsamp
print, format='(4x,a,f7.4)', 'Sample spacing Deltax = L/Nsamp [m] =', Deltax
print, format='(4x,a,f7.4)', 'Min resolvable wavelength = 2*Deltax [m] =', minwave
print, format='(4x,a,f8.4)', 'Min resolvable (fundamental) spatial frequency nu(1) = 1/L [1/m]= ',numin
print, format='(4x,a,f8.4)', 'Max resolvable (Nyquist) spatial frequency nuNyquist = 1/(2.0*Deltax) [1/m] = ',nuNyquist 
print,' '
print,' Horoshendov parameters:
print,' '
print, format='(4x,a,f6.3)', 'Characteristic spatial wave period Lo [m] =',Lo
print, format='(4x,a,f7.3)', 'Characteristic wave angular spatial frequency qo = 2pi/Lo [rad/m] =',qo
print, format='(4x,a,f6.3)', 'Spatial correlation length sigma_w [m] =',sigmaw
print, format='(4x,a,e11.3)', 'Wave elevation variance Czz(0) [m^2] =',var
print, format='(4x,a,i5)', 'Number of surfaces generated = Nsurf =',Nsurf
print,' '
print, format='(4x,a,i12)', 'Seed for random number generation =',seed

; Compute positive frequencies used for sampling the variance spectrum; these do not include 0 frequency
; Even spacing Deltanu in frequency nu between the fundamental and Nyquist frequencies
Nnupos = Nsamp/2
Deltanu = (numax - numin)/float(Nnupos-1) ; spacing for positive frequencies; Deltanu = Nyquist freq
print,'numax,numin,deltanu = ',numax,numin,Deltanu
nupos = numin + Deltanu*findgen(Nnupos) ; for the one-sided energy spectrum, excluding the nu = 0 DC component

nu = [0.0,nupos,-reverse(nupos[0:Nsamp/2-2])] ; total of Nsamp nu values in FFT order
help,nu
numath = SHIFT(nu,Nsamp/2-1) ; total of Nsamp nu values in math order for plotting
if idebug eq 1 then print,'x = ',x
if idebug eq 1 then print,'nupos = ',nupos
if idebug eq 1 then print,'nu = ',nu
if idebug eq 1 then print,'numath = ',numath
;-----------------

; Values for plotting a visually smooth energy spectrum; pos frequencies only
; use equal spacing in log(nu) for nice appearance in plots
Nplotthy = 100 ; 100 is large enough for smooth curves for most spectra
numinplotthy = 1.0 ; adjust min and max values for the spectrum plotted
numaxplotthy = 10.0
numinlog = alog10(numinplotthy)
numaxlog = alog10(numaxplotthy)
dnuplotlog = (numaxlog - numinlog)/float(Nplotthy-1)
nuplotlog = numinlog + dnuplotlog*findgen(Nplotthy)
nuplotthy = 10.0^nuplotlog

;-----------------

; lag values for plotting the ACF
lagpos = findgen(Nsamp/2)
lagindx = [-reverse(lagpos[1:Nsamp/2-1]),lagpos,Nsamp/2]  ; Nsamp total neg and pos lags in math order
lag = lagindx*deltax
print,' max lag = ',max(lag)
if idebug eq 1 then print,'new lag =',lag

; array for saving ACFs from generated surfaces
ACFsave = fltarr(Nsurf,n_elements(lagindx))

; array for saving the surface elevation variances
varzsave = fltarr(Nsurf)

;-----------------

; choose a variance spectrum
print,' '
print,format='(4x,a,f5.1)', 'The chosen variance spectrum is Horoshenkov(nu)' 

; The Horoshenkov spectrum is two-sided.  The factor of 2 makes a 1 sided spectrum, 
; which is used to replace the 1 sided Pierson-Moskowitz spectrum in the WK_PM_nu.pro routine.
; This is just an expedient to minimize reprogramming.
S1s = 2.0*Horoshenkov_S2snu(nupos, sigmaw,Lo,var)  ; discrete, one-sided values for wave generation with FFT
      
S1splotthy = 2.0*Horoshenkov_S2snu(nuplotthy, sigmaw,Lo,var) ; values for plotting the whole spectrum as smooth function of positive frequencies    

S1sNyquist = 2.0*Horoshenkov_S2snu(nuNyquist, sigmaw,Lo,var) ; value for plotting the Nyquist freq value


; +++++++++++++ DO ALL CALCULATIONS FIRST, THEN ALL PLOTS ++++++++++++++

; Create random surfaces

S2sided = 0.5*[0.0, S1s, REVERSE(S1s[0:Nsamp/2-2])] ; S2s = S1s/2; the continuous 2-sided spectrum sampled at discrete points
help,S2sided

nu2splot = SHIFT(nu,Nsamp/2-1) ; shift to math order for plotting
S2splot = SHIFT(S2sided,Nsamp/2-1)
help,S2splot

; S2root is the square root term in Tutorial Eq. (2.8)
S2root = SQRT(S2sided*Deltanu) ; sqrt of the variance density spectrum times Deltanu gives the amplitude at each positive k

C3 = 1.0/SQRT(4.0) ; remaining 1/sqrt(2) factors

; Define arrays for saving the generated sea surfaces and computing the one-sided periodogram from the generated sea surfaces
; zsurfavg = fltarr(Nsurf,Nsamp) ; needed only if surfaces are to be saved for multiple surface profile plots
P1S = fltarr(Nsamp/2+1) ; 0 and pos nu values only
P1Savg = fltarr(Nsurf,Nsamp/2+1)


; +++++++ Begin loop over Nsurf surface realizations.
isurf = 0L ; make longword integer
Npnt = Nsurf/10L ; for message every 10% of loop

for isurf = 0L,Nsurf-1L do begin
   
   if  (isurf+1L) mod Npnt eq 0L then print,format='(3x,a,i7,a,i7)',' isurf = ',isurf+1L,' of ',Nsurf
   
   ; Create Hermitian random amplitudes zhat as in Tutorial Eqs. (2.8) and (2.9).
   
   rj = RANDOMU(seed,Nsamp,/normal) ; for random numbers r(k); Tutorial rho
   sj = RANDOMU(seed,Nsamp,/normal) ; for random numbers s(k); Tutorial sigma
   
   zhat = COMPLEXARR(Nsamp) ; define the random complex Hermitian zhat array
   
   ; note that for the IDL frequency ordering, -nu = Nsamp-nu
   for j=1,Nsamp-1 do zhat(j) = C3*S2root(j)*COMPLEX(rj[j] + rj[Nsamp-j], sj[j] - sj[Nsamp-j])
   
   zhat[0] = COMPLEX(0.0,0.0)  ; set k = 0 to 0.0 to force the mean sea level = 0
   
   ; take the inverse FFT to get the surface elevations as the real part of the inverse
   
   zcomplx = FFT(zhat,/inverse)
   
   zsurf = REAL_PART(zcomplx)
   
   ; zsurfavg[isurf,*] = zsurf ; save only if want to plot multiple surfaces
   
   varz = VARIANCE(zsurf) ; variance of the surface elevation
   varzsave(isurf) = varz ; save to computing ensemble averages
   
   if isurf eq 0L then begin
      ; save the first surface for plotting
      zsurf0 = zsurf
      ; print these checks only for the first surface 
      print,' '
      print,' Checks on the generated z(x) for isurf = ',isurf
      ; compute the mean sea surface, which should be zero
      print,'   zavg (should = 0) = ',TOTAL(zsurf)/float(Nsamp)
      ; check on inversion of Hermitian amps to get a real z
      ; imaginary part of zcomplx should be 0 to within roundoff error (order 10^(-10) or less)
      print,'   max |Imag{z}| (should = 0) = ',max(abs(IMAGINARY(zcomplx)))
      ; elevation variance of the surface
      print,'   sample var{zsurf} =',varz
      ; Compare the sample variance with the theoretical value for Pierson-Moskowitz spectrum
;      g = 9.82 ; accel of gravity
;      varzPM = 0.00304*(U10^4)/g^2
;      print,'   Pierson-Moskowitz var{zsurf} =',varzPM
      ;check Parseval's relation for the amplitudes and generated surface (Eq. 1.20)
      print,"   Parseval's relation:"
      sumz2 = 0.0
      for i=0,Nsamp-1 do sumz2 = sumz2 + abs(zsurf(i))^2
      print,'      sum z^2 =',sumz2
      sumzhat2 = 0.0
      for i=0,Nsamp-1 do sumzhat2 = sumzhat2 + abs(zhat(i))^2
      print,'      N * sum zhat^2 =',Nsamp*sumzhat2
   endif ; isurf = 0 check
   
   ; ***** compute and save the autocovariance of each surface
    
   ACFsave[isurf,*] = a_correlate(zsurf,lagindx,/covariance)
   
   ;***** take the forward FFT of the generated surface to obtain the periodogram for comparison with S1s(nu)
     
   zcomplx = COMPLEX(zsurf) ; make z a complex number for the IDL FFT routine
   
   zhatamp = FFT(zcomplx) ; take the FFT to get the Fourier amplitudes zhat for this particular surface
   
   ; compute the 1-sided periodogram P1S for this particular surface; Eq. (2.10)
   P1S[0] = abs(zhatamp[0])^2 ; 0 freq
   P1S[Nsamp/2] = abs(zhatamp[Nsamp/2])^2 ; Nyquist freq
   for j=1,Nsamp/2-1 do P1S(j) = 2.0*abs(zhatamp[j])^2
   
   P1S = P1S/Deltanu ; divide |zhat|^2 by Deltanu to get a density estimate (the periodogram)
   ; save the periodogram for ensemble averaging
   P1Savg[isurf,*] = P1S

endfor ; ++++++ end of isurf loop for surface generation


; ***** compute the ensemble averages (over the Nsurf surfaces)

avgvarz = TOTAL(varzsave)/Nsurf
print,' '
print,' ensemble average var{z} = ',avgvarz,' m^2'

ACFest = TOTAL(ACFsave,1)/Nsurf
print,' '
;print,' ensemble average sample ACF(lag=0) from zsurf = var{z} = ',ACFest[Nlag-1],' m^2'
print,' ensemble average sample ACF(lag=0) from zsurf = var{z} = ',ACFest[Nsamp/2-1],' m^2'
print,' '

P1sest = TOTAL(P1savg,1)/Nsurf ; the average periodogram of the Nsurf surfaces
help,P1Sest

;----------------

; Compute the theoretical ACF from the P-M spectrum as ACF2s(lag) = invFFT{S2s(nu)} (Wiener-Khinchin).
; Since possibly only part of the spectrum was sampled above (according to the Nsamp value), 
;   re-do with a wider range of sampled points to get closer to the "true" ACF for the "whole" P-M spectrum
;   Want to have a large number of points with lags roughly -L to L, like for the sample ACF derived from z(x)
; Compute positive frequencies used for plotting the variance spectrum; these do not include 0 frequency
; even spacing Deltanu in frequency nu; the last frequency is the Nyquist frequency

; Nsamp2 is used only for the green theory curve in the lower 2 plots; can use a large number if needed for a smooth plot

print,' '
print,'******* Computing the theoretical ACF from the Horoshenkov spectrum as ACF2s(lag) = invFFT{S2s(nu)}'
print,' '

; Choose Nsamp2 and L2 to give full coverage of the desired L range and a smooth plot (green curves as LL and LR)
Nsamp2 = 2048 ;Nsamp; 2048 ;Nsamp
L2 = 4.0 ; 300.0
Nsamp2pos = Nsamp2/2
DeltaxNsamp2 = L2/float(Nsamp2) ; deltax for theory ACF
numinNsamp2 = 1.0/L2 
numaxNsamp2 = 1.0/(2.0*DeltaxNsamp2)
DeltanuNsamp2 = (numaxNsamp2 - numinNsamp2)/float(Nsamp2pos-1) ; spacing for the Nsamp2/2 positive frequencies
nuposNsamp2 = numinNsamp2 + DeltanuNsamp2*findgen(Nsamp2pos)
print,' numinNsamp2, numaxNsamp2, DeltanuNsamp2 = ',numinNsamp2, numaxNsamp2, DeltanuNsamp2

; nu values for plotting the S2s(nu) derived from the theoretical S2s(nu) = invFFT(ACF)
nuACF2 = [0.0,nuposNsamp2,-reverse(nuposNsamp2[0:Nsamp2pos-2])] ; 2*Nsamp2 values; for use in lower right plot
help,nuACF2
numath2 = shift(nuACF2,Nsamp2pos-1) ; neg and pos, math order
;if idebug eq 1 then print,'nuposNsamp2 = ',nuposNsamp2
;if idebug eq 1 then print,'nuACF2 = ',nuACF2
;if idebug eq 1 then print,'numath2 = ',numath2

; sample the S1s(nu) spectrum at positive nu values
S1sACF2 = 2.0*Horoshenkov_S2snu(nuposNsamp2, sigmaw,Lo,var)

help,S1sACF2
;if idebug eq 1 then print,'S1sACF2 = ',S1sACF2

; make a discrete 2 sided spectrum for ACF computation
S2sACF2 = [0.0,S1SACF2,REVERSE(S1SACF2[0:Nsamp2pos-2])]*0.5*DeltanuNsamp2 ; in FFT order ; must include 0.5 for 1s to 2s 
help,S2sACF2
;if idebug eq 1 then print,'S2sACF2 = ',S2sACF2
S2sACF2math = shift(S2sACF2,Nsamp2pos-1)/DeltanuNsamp2 ; convert back to a 2-sided density for plotting in UR as a check

; Now take the inverse FFT of the discretized Pierson-Moskowitz spectrum to get the theoretical ACF (via Wiener-Khinchin)
; This ACF has no random noise effects.

ThyACF2 = FFT(S2sACF2,/inverse) ; inv FFT of UR orange dots 
help,ThyACF2
print,'LL green:  max |Imag{ThyACF2}| (should = 0) = ',max(abs(IMAGINARY(ThyACF2)))

realThyACF2 = REAL_PART(ThyACF2)
if idebug eq 1 then print,'realThyACF2 = ',realThyACF2
maxthyACF2 = max(realthyACF2,lag0)
; realThyACF2 has lags in the FFT order, so lag = 0 is index 0
print,'LL green: IDL PM thyACF(lag=0) = var{z} = ',realthyACF2[0],' m^2'

; shift RealThyACF2 to math order for plotting
plotThyACF2 = SHIFT(REAL_PART(realThyACF2),Nsamp2/2-1)
help,plotThyACF2
if idebug eq 1 then print,'plotThyACF2 = ',plotThyACF2

; lag for plotting the theoretical ACF
lagpos2 = findgen(Nsamp2pos)
lagindx2 = [-reverse(lagpos2[1:Nsamp2pos-1]),lagpos2,Nsamp2pos]  ; neg and pos lags in math order
lagThyACF2 = lagindx2*deltaxNsamp2
help,lagThyACF2
help,plotThyACF2
if idebug eq 1 then print,'lagThyACF2 = ',lagThyACF2

; Take the 1D inverse FFT of plotThyACF2(lagThyACF2)
; This is really just a check on the round-trip FFT calculation: S2s(nu) to ACF(lag) to S2s(nu)
; ThyACF2 is in lag math order from the plots in panel (d)

ThyS2snu = FFT(ThyACF2)
;if idebug eq 1 then print,'ThyS2snu = ',ThyS2snu

print,'LR green:   max |Imag{ThyS2snu}| (should = 0) = ', max(abs(IMAGINARY(ThyS2snu)))

;ThyS2snu is in IDL freq order; shift to math order for plotting
ThyS2snuplot = REAL_PART(SHIFT(ThyS2snu,Nsamp2/2-1))/DeltanuNsamp2
help,ThyS2snuplot

;if idebug eq 1 then print,'ThyS2snuplot = ',ThyS2snuplot

; DEBUGGING CHECK on the above. This correctly reproduces the original S2sACF2:
; plotThyACF2 is in math order for plotting; use the to recover the S2sThy spectrum from which it was obtained
;ThyACF2check = COMPLEX(SHIFT(plotThyACF2,-Nsamp2/2+1))/DeltanuNsamp2 ; convert to 2-sided density and shift back to FFT order
;S2ThyACF2check = FFT(ThyACF2check) ; in FFT order
;plotS2ThyACF2check = REAL_PART(SHIFT(S2ThyACF2check,Nsamp2/2-1))


;-----------

; Compute the 2 sided spectrum derived from the ensemble average ACF
; Use the same pattern as the theoretical ACF2 above

print,' '
print,' ***** Computing the 2 sided spectrum derived from the ensemble average ACF'
print,'

help,ACFest
if idebug eq 1 then print,'ACFest = ',ACFest

; ACFest is in math order for plotting; must shift to FFT order before transforming 
ACFestfft = COMPLEX(SHIFT(ACFest,-Nsamp/2+1)) ; ACFestfft is in FFT order

S2sACFest = FFT(ACFestfft) ; S2sACFest is in FFT order

; The ACF estimate; shift back to math order for plotting and divide by Deltanu to get a density estimate
plotS2sACFest = REAL_PART(SHIFT(S2sACFest,Nsamp/2-1))/Deltanu 



; +++++++++++++ NOW DO ALL PLOTS ++++++++++++++

;***** Define a 5 panel plot layout

; region 0  |  region 1
; region 2 (wide)
; region 3  |  region 4

reg = fltarr(5,4,/nozero)

; plot coords for !p.region are in normal coordinates (0 to 1)
; adjust values as needed for the desired appearance

xl = 0.04
xr = 0.50
dx = 0.43

yb = 0.02 ; the bottom of the bottom row
ym = 0.32
yt = 0.62 ; 0.48
dy = 0.3

reg[0,*] = [xl,yt,xl+dx,yt+dy]
reg[1,*] = [xr,yt,xr+dx,yt+dy]
reg[2,*] = [xl,ym,xr+dx,ym+dy]
reg[3,*] = [xl,yb,xl+dx,yb+dy]
reg[4,*] = [xr,yb,xr+dx,yb+dy]

; offsets for label positions
dxlab = 0.06
dylab = 0.09

; set up output for a vector eps file

;PlotFileName ='WK1D-Nsurf10000-lag100-Nsamp512.eps'

!x.margin = [6,1]
!y.margin = [4,1]
;!x.Omargin = [2,2]
;!y.Omargin = [0,3]

aspectratio = 1.2
xsize= 6.5
ysize=xsize*aspectratio
charsize = 1.0
!p.charsize = charsize

PS_Start,filename=PlotFileName, /encapsulated, font=1, tt_font='Helvetica', $
   charsize=charsize, default_thickness=2, /nomatch, xsize=xsize, ysize=ysize

; choose color order; can see with colors = PickColorName()
kolors = ['Black','Magenta','Blue','Cyan','Green','Gold','ORG5','red','TG5','TG2','forest green','ORG6']

;***** end plot set up

;+++++ plot the one-sided variance spectrum S1s(nu) in upper left panel
;      calls to plot routines must use /noerase

!p.region = reg(0,*)

xlabel = Greek('nu',/PS) + ' [1/m]'
ylabel = 'S!d1s!n [m!u2!n/(1/m)]'
numinpltlog = 1.0
numaxpltlog = 10.0
S1sminplot = 1.0e-14
S1smaxplot = 1.0e-6

; creat nice axis labels
LogLabels, S1sminplot,S1smaxplot, nticksy, yticklabs

cgPlot,nuplotthy,S1splotthy, /nodata,/noerase,/xlog,/ylog, $
   xrange=[numinpltlog,numaxpltlog],xtitle=xlabel, $
   yrange=[S1sminplot,S1smaxplot],ytitle=ylabel,yticks = nticksy-1, ytickname=yticklabs

cgText,xl+dx-dxlab,yt+dylab,'(a)',charsize=1.25*!p.charsize,/normal

; plot the analytical spectrum
cgOplot,nuplotthy,S1splotthy,color=cgColor('blue'),linesty=0, thick=2 ;,psym=-plotsym(/diamond,scale=1.0)

; plot the sampled points with dots at first and last sampled points; vertical lines at intermediate sample points
cgOplot,nupos[0],S1s[0],color=cgColor('red'),thick=2,psym=plotsym(/circle,/fill,scale=0.7),linesty=0
cgOplot,nupos[1:Nsamp/2-2],S1s[1:Nsamp/2-2],color=cgColor('lime green'),thick=2,psym=plotsym(/vline,/fill,scale=0.7),linesty=0
cgOplot,nupos[Nsamp/2-1],S1s[Nsamp/2-1],color=cgColor('lime green'),thick=2,psym=plotsym(/vline,/fill,scale=0.7),linesty=0
; plot Nyquist freq as a red dot
cgOplot,nuNyquist,S1sNyquist,color=cgColor('red'),thick=2,psym=plotsym(/circle,/fill,scale=0.7),linesty=0


;+++++ plot S1s, S2s, and ensemble average P1s, which contains random noise

!p.region = reg(1,*)  ; the upper right plot panel

minS = min(S1s,max=maxS)

xlabel = Greek('nu',/PS) +' [1/m]'
;ylabel = 'S!d2s!n & |zhat|!u2!n/' + Greek('Delta',/PS) + Greek('nu',/PS) + ' [m!u2!n/(1/m)]'
ylabel = 'S!d2s!n & P!d1s!n [m!u2!n/(1/m)]'
numaxplt = 10.0 ;for the rest of the spectrum plots
minS = -0.5e-7
maxS = 1.5e-7

cgPlot,nuplotthy,S1splotthy, /nodata,/noerase,$
   xrange=[-numaxplt,numaxplt],xtitle=xlabel,/xstyle, $
   yrange=[minS,maxS],ytitle=ylabel,/ystyle

cgText,xr+dx-dxlab,yt+dylab,'(b)',charsize=1.25*!p.charsize,/normal

; plot the 2 sided theory spectrum at the sampled points
CgOplot,nu2splot,S2splot,color=cgColor('lime green'),thick=2,linesty=0 ;,psym=-plotsym(/circle,/fill,scale=0.7)


; check: plot the Nsamp2 points used to obtain the theoretical ACF plotted in the LL
;cgOplot,numath2,S2sACF2math,color=cgColor('ORG6'),thick=2,linesty=0 ,psym=-plotsym(/circle,/fill,scale=0.7)

; plot the estmated 1-sided spectrum; nupos is pos only, but P1sest includes 0 freq
cgOplot,[0,nupos],P1Sest,color=cgColor('red'),thick=2,linesty=0 ;,psym=-plotsym(/circle,/fill,scale=0.7)


; debugging:  plot the check on S2sACF2 to ACF2 to S2sACF2
;cgOplot,numath2,plotS2ThyACF2check,color=cgColor('purple'),thick=2,linesty=0 ,psym=-plotsym(/diamond,scale=1.2)

;print,'nuplotthy = ',nuplotthy
; replot the theoretical spectrum of panel (a) for comparison
cgOplot,nuplotthy,S1splotthy,color=cgColor('blue'),thick=2,linesty=0;,psym=-plotsym(/diamond,scale=1.0)



;+++++ plot the generated surface(s) in the middle panel
; the plotting itself can take a long time if Nsurf is large (and is visually ugly for Nsurf > 2 or 3), so can skip to speed up the run

if iplotsurfs ne 0 then begin

!p.region = reg(2,*)

maxz = max(abs(zsurf))

   cgPlot,x,zsurf, /nodata, /noerase, $
      xrange=[0.0,L],xtitle='x [m]',/xstyle, $
      yrange=[-maxz,maxz],ytitle='z [m]'

   cgText,xr+dx-dxlab,ym+dylab,'(c)',charsize=1.25*!p.charsize,/normal

;if isurf eq 0L then begin

   ;if Nsurf eq 1 then begin
   ; plot in red for just one surface
   cgOPlot,x,zsurf0,color=cgColor('red'),thick=2;,psym=-plotsym(/circle,/fill,scale=0.7)
   ;endif else begin
   ; different colors for different surfaces if have more than one surface
   ;   cgOPlot,x,zsurf,color=kolors[isurf mod 10+1],thick=2;,psym=-plotsym(/circle,/fill,scale=0.7)
   ;endelse

;endif
endif ; end if iplotsurfs 


;+++++ plot autocovariance functions in lower left panel

!p.region = reg(3,*)

print,' '
print,' lower left plot'

ymax = var
ymin = -var
xmaxplt = 1.0

   cgPlot,lag,ACFest, /nodata,/noerase, $
     ; xrange=[-Nlag*deltax,Nlag*deltax],/xstyle,xtitle='lag [m]', $
      xrange=[-xmaxplt,xmaxplt],/xstyle,xtitle='lag [m]', $
      yrange=[ymin,ymax], ytitle='C!dzz!n [m!u2!n]';,/ystyle

   cgText,xl+dx-dxlab,yb+dylab,'(d)',charsize=1.25*!p.charsize,/normal

   ; plot the theoretical ACF derived from the FFT of the Pierson-Moskowitz spectrum S2s(nu)
   ; This green curve is the ACF of the high-resolution green curve in panel (b)
   cgOplot,lagThyACF2,plotThyACF2,color=cgColor('lime green'),thick=4,linesty=0 ;,psym=-plotsym(/circle,/fill,scale=1.0)
   help,lagThyACF2
   help,lagThyACF2
   minlag = min(lagThyACF2,max=maxlag)
   print,'min, max lagThyACF2 for lagThyACF2 (green) = ',minlag,maxlag

; ACFest is the ensemble average ACF computed from the z(x) surfaces

cgOplot,lag,ACFest,thick=2,linesty=0,color=cgColor('red') ;,psym=-plotsym(/circle,/fill,scale=0.7)
help,lag
help,ACFest
minlag = min(lag,max=maxlag)
print,'min, max lag for ACFest (red) = ',minlag,maxlag

; overplot the analytic Czz(lag) curve

ACFtheory = Horoshenkov_Czz(lag, sigmaw,Lo,var)
cgOplot,lag,ACFtheory,thick=4,linesty=0,color=cgColor('purple') ;,psym=-plotsym(/circle,/fill,scale=0.7)



;*******************

!p.region = reg(4,*)

print,' '
print,' lower right plot'

; plot FFT{ACF} in lower right panel

; construct a 2 sided, real, symmetric spectrum S2S(k), which will have a real, symmetric FFT (real, symmetric ACF)
; note that S2S(k) must be in IDL frequency order for the FFT routine
; The S2sided spectrum created above for surface generation can be reused here

; S2sided from above, hence S2S defined here, is in FFT freq order
S2S = 0.5*S2sided*Deltanu ; must include 0.5 to convert from one-sided to two-sided; was not included in S2sided above
help,S2S
;print,'green: S2S = ',S2S

; must make a real, symmetric, "2-sided" ACF (neg and pos lags) so that the FFT will be real, symmetric

;nu = [0.0,nupos,-reverse(nupos[0:Nsamp/2-2])] ; Nsamp values in IDL freq order
;numath = SHIFT(nu,Nsamp/2-1) ; Nsamp values in math freq order

Snuplot = SHIFT(S2S,Nsamp/2-1)/Deltanu ; density for comparison with theory
;;print,'Snuplot[0],[1],[Nsamp/2-1] = ',Snuplot[0],Snuplot[Nsamp/2-1]
;;print,'orange: kmath = ',kmath
;;print,'orange: Snuplot = ',Snuplot

xlabel = Greek('nu',/PS) + ' [1/m]'
ylabel = 'S [m!u2!n/(1/m)]'

cgPlot,nuplotthy,S1splotthy, /nodata,/noerase,$
   xrange=[-numaxplt,numaxplt],xtitle=xlabel,/xstyle, $
   yrange=[minS,maxS],ytitle=ylabel,/ystyle;,/ylog

cgText,xr+dx-dxlab,yb+dylab,'(e)',charsize=1.25*!p.charsize,/normal

; Snuplot is the analytical S2s
;cgOplot,kmath,Snuplot,color=cgColor('purple'),thick=2 ,linesty=0 ,psym=-plotsym(/circle,/fill,scale=0.7)

; plot the theoretical 1-sided spectrum S(k) for reference
;help,nuplotthy
;help,S1splotthy
cgOplot,nuplotthy,S1splotthy,color=cgColor('blue'),thick=2 ,linesty=0 ;,psym=-plotsym(/circle,/fill,scale=0.7)

; plot the retrieved theoretical spectrum for reference
; this is a two-sided S2s, so should be one-half the value of the blue S1s curve
;help,numath2
;help,ThyS2snuplot
;print,'min, max numathsamp = ',min(numath2),max(numath2)
;print,'numath2[0:3] =',numath2[0:3]
cgOplot,numath2,ThyS2snuplot,color=cgColor('lime green'),thick=4 ,linesty=0 ; ,psym=-plotsym(/circle,/fill,scale=1.0)

; plot the retrieved sample S2s spectrum; this spectrum contains random noise
;help,numathsamp
;help,SampleSnuplot
;print,'min, max numathsamp = ',min(numathsamp),max(numathsamp)
;print,'numathsamp[0:3] =',numathsamp[0:3]
;;cgOplot,numathsamp,SampleSnuplot,color=cgColor('red'),linesty=0,thick=2 ,psym=-plotsym(/circle,/fill,scale=1.0)
;cgOplot,numathsamp/(2.0*!pi),SampleSnuplot,color=cgColor('red'),linesty=0,thick=2 ,psym=-plotsym(/circle,/fill,scale=0.7)

help,numath
help,plotS2sACFest
cgOplot,numath,plotS2sACFest,color=cgColor('red'),thick=2 ,linesty=0 ; ,psym=-plotsym(/circle,/fill,scale=1.0)


;******************

; plot header and bottom labels

cgText,0.5,0.97,'Wiener-Khinchin Study of Horoshenkov Surfaces',align=0.5,charsize=1.1*!p.charsize,/normal

toplabel = Greek('sigma',/PS) + '!dw!n =' + string(sigmaw,format='(f5.2)') + ';  L!do!n =' +  string(Lo,format='(f5.2)') + $
  ';  C!dzz!n(0) = ' + string(var,format='(e10.2)') + ';  N!dsamp!n =' + string(Nsamp,format='(i5)') + ';  N!dsurf!n =' + string(Nsurf,format='(i5)')

cgText,0.5,0.93,toplabel,align=0.5,charsize=1.1*!p.charsize,/normal

cgText,0.05,0.01,'WienerKhinchin_Horo_nu.pro',charsize=0.7*!p.charsize,/normal;,color=cgColor('gray')

;-----------

PS_End,Resize=50,/PNG ;close and fix the eps file and create a png version

print,'image file ', PlotFileName,' closed OK'

;!p.multi = 0 ; reset to normal layout
;
; At the end of plotting, restore default plot region
; !p.region[x0,y0,x1,yt], where (x0, y0) is the origin, and (x1, y1) is the upper right corner. 
!p.region = [0,0,1,1]

print,'end WK_H_nu'
end


