@ECKV2side_cos2S_k_phi

pro Animate2D

; This routine shows how to animate a 2D sea surface using a two-sided (2S) variance spectrum Psi_2s(kx,ky).
; (For generation of single surfaces with ancillary output, use routine cgGenerate_2D_SeaSurface.pro)

; This routine uses a TWO-sided, directional (2D) variance spectrum and 
; 2D inverse FFTs to generate a time series of random 2D sea surfaces.

; The Elfouhaily et al (1997) omnidirectional variance spectrum is used along with an 
; asymmetric (Phi(-kvec) ne Phi(+kvec)) cosine-2S spreading function to create a 
; two-sided, 2D variance spectrum. This is computed by routine ECKV2Side_cos2S_k_phi.

; This version uses angular frequencies kx,ky = 2 pi/wavelength [rad/m] as the spatial frequency variables.

; Options are included for
;   Discretizing the temporal frequencies so that the surface repeats itselt at the last time step
;   Re-scaling the postive part of the variance spectrum so that the slope variance is fully resolved

; Each time step either overwrites the window or creates a new output file.
 
; ***************************************************************************************************
; * This code is copyright (c) 2016 by Curtis D. Mobley.                                            *
; * Permission is hereby given to reproduce and use this code for non-commercial academic research, *
; * provided that the user suitably acknowledges Curtis D. Mobley in any presentations, reports,    *
; * publications, or other works that make use of the code or its output.  Depending on the extent  *
; * of use of the code or its outputs, suitable acknowledgement can range from a footnote to offer  *
; * of coauthorship.  Further questions can be directed to curtis.mobley@sequoiasci.com.            *
; ***************************************************************************************************
 
;-----------------------------------------------------------------------------------------------------------
;| This routine uses Coyote Graphics routines rather than standard IDL for plotting; see www.idlcoyote.com |
;| and "Coyote's Guide to Traditional IDL Graphics: Using Familiar Tools Creatively" by David Fanning      |
;| Any comments below like "CG page xxx" refer to this book.                                               |
;| CG routines are wrappers for standard IDL routines, which make it easier to do certain things like      |
;| create vector eps output files; cgPlot replaces the standard Plot, etc.                                 |
;-----------------------------------------------------------------------------------------------------------

; Written 18 December 2013 by Curtis Mobley.

; ***** BEGIN USER INPUT *****

; pick the type of plot to make
;plottype = 'WIN' ; for on-screen display time step by time step
plottype = 'PNG' ; to create high-quality PNG files; one file per time step

IF plottype EQ 'PNG' THEN BEGIN
  ; pick a root name for output plot files.  One file is saved for each time step, which can
  ; be used to creat a frame of a video.  
  ; Output plot files will be named on the scheme plotroot_timestep.png,
  ; e.g. 'plotroot_0001.png','plotroot_0.0002.png', etc. 
  plotroot = 'U10_512_S20'
ENDIF

; ***** DEFINE THE PHYSICAL REGION AND SAMPLING ***** 

; define the wind speed at 10 m above MSL for use in the variance spectrum
U10 = 10.0 ; in m/s

Sparam = 20.0 ;10.0

Nx = 512 ; number of downwind (x direction) samples of sea surface elevation; MUST be a power of 2 for the FFT
Ny = 512 ; number of crosswind (y) samples of sea surface elevation; MUST be a power of 2 for the FFT

; define the time step Deltat and the number of time steps for the animation
Deltat = 0.1 ; in seconds
Ntimes = 5 ; 201 ; frame are at times 0, Deltat, ..., (Ntimes-1)*Deltat

QUANTOMEGA = 0 ; = 1 to quantize the temporal frequencies omega(k) so that the pattern repeats exactly after time = TotalTime;
               ; use this for video loops; = 0 to use exact omega(k)
TotalTime = (Ntimes-1)*Deltat ; used only if quantomega = 1
omega0 = 2.0*!pi/TotalTime ; used only if quantomega = 1

RESCALE = 1 ; = 1 to rescale the positive variance spectrum to fully resolve the slope variance; 
          ; = 0 for unresolved slope variance
; Define the delta(k) parametersto be used for this simulation
; These parameters are computed by cgPlot_EnergySlopeSpectra.pro
; These parameters are used only if tilde = 1
; values for 10 m/s, Lx = 200, Nx = 1024; or 10, 100, 512
kp = 0.065794
kh = 16.0850
deltah = 7.800


InputOption = 2

CASE InputOption OF
  1: BEGIN
        labelInOpt = ' Spatial resolution is defined by the sample spacing and number of sample points'
        ; smallest resolvable wavelength is 2*Deltax
        Deltax = 0.1; 0.0025
        Deltay = 0.1
        Lx = Deltax*float(Nx)
        Ly = Deltax*float(Ny)
     END 
  2: BEGIN
        labelInOpt = ' Spatial resolution is defined by the spatial region and number of sample points'
        ; largest resolvable wave is the length of the spatial region
        Lx = 100.0 ; length of surface region in the x direction, in meters 
        Ly = 100.0 ; length of surface region in the y direction, in meters 
        Deltax = Lx/float(Nx)
        Deltay = Ly/float(Ny)
     END    
ENDCASE

; define an initial seed for random number generation
seed =  833202L 

; ***** END USER INPUT *****

; ***** DEFINE THE PLOT *****

CASE plottype OF

'WIN' : BEGIN

  ; On-screen display in a window for real-time viewing.
  ; Set windows pixel x,y sizes and other info.
  ; A window will be opened below with cgDisplay, which gives a device-independent output device with
  ; the pixel size for Windows or the same aspect ratio, ypixsise/xpixsize, for PS
  
  thisDevice = !D.name
  
  xpixsize = 1100 ; select for desired visual appearance
  ypixsize = 550

  !P.font = 1 ; 1 for TrueType fonts
  !P.thick = 2.0
  !P.charsize = 2.5
  !x.margin = [6,2]
  !y.margin = [4,3]

  ; backgroundcolor = cgColor('gray') ; white is the default
 
  ; cgDisplay is called below at each time step to open a window with these properties

  ; cgDisplay,XSIZE=xpixsize,YSIZE=ypixsize,WID=0;,color=backgroundcolor
   
END

'PNG' : BEGIN
  
  ; Set up output for a vector eps file.  A corresponding high-quality PNG file will be created
 
  !x.margin = [5,4]
  !y.margin = [3,4]
  !y.omargin = [0,0]
  !P.thick = 2.0
  charsize = 1.2
  !p.charsize = charsize
      
  aspectratio = 0.6
  xsize= 6.5
  ysize=xsize*aspectratio

;  PS_Start is called below at each time step to open a new plot file for each time step
 
;  PS_Start,filename=PlotFileName, /encapsulated, font=1, tt_font='Helvetica', $
;           charsize=charsize, default_thickness=2, /nomatch, xsize=xsize, ysize=ysize

END

ENDCASE

;----- time-independent info for plots

xlabel = 'x [m]'
ylabel = 'y [m]'
zlabel = 'z [m]'
xplot = Deltax*findgen(Nx+1)
yplot = Deltay*findgen(Ny+1)
zplot=fltarr(Nx+1,Ny+1)

; can see color tables with Help --> Color Tables --> samples
cgLoadCT,1 ; used to shade the surface


; ***** COMPUTE THE SPATIAL FREQUENCIES ***** 

kxmin = 2.0*!pi/Lx ; min frequency (min spatial frequency); frequency of longest resolvable wave
kymin = 2.0*!pi/Ly
minxwave = 2.0*Deltax ; min resolvable wavelength (2 point wave)
minywave = 2.0*Deltay ; min resolvable wavelength (2 point wave)
Nyquistx = !pi/Deltax ; Nyquist spatial freq in rad/m
Nyquisty = !pi/Deltay ; Nyquist spatial freq in rad/m
kxmax = Nyquistx
kymax = Nyquisty

print,' '
print,' Animation of a 2D sea surface from a 2D two-sided wave variance spectrum Psi_2S(kx,ky)'
print,' '
print, format='(4x,a,2i5)',   'Numbers of spatial samples = Nx, Ny =',Nx,Ny
print, format='(4x,a,2f7.4)', 'Sample spacings Deltax, Deltay [m] =', Deltax,Deltay
print, format='(4x,a,2f8.1)', 'Max resolvable wavelengths = Lx, Ly [m] =',Lx,Ly
print, format='(4x,a,2f8.3)', 'Min resolvable wavelengths = 2*Deltax, 2*Deltay [m] =', minxwave,minywave
print, format='(4x,a,2f8.3)', 'Min resolvable frequencies kxmin, kymin [rad/m] = ',kxmin ,kymin
print, format='(4x,a,2f8.3)', 'Max resolvable (Nyquist) frequencies kxmax, kymax [rad/m] = ',kxmax ,kymax
print, format='(4x,a,i12)',   'Seed for random number generation =',seed
print,' '
print, format='(4x,a,f6.3)', 'Time step Deltat [s] = ',Deltat
print, format='(4x,a,i5)','Number of time steps = Ntimes =',Ntimes


; ***** NOTE ON FREQUENCY ARRAY ORDER:

; When plotting 1-sided power spectra, it is customary to plot only positive frequencies in the order
; kpos[1 to N/2] = [1, 2,..., N/2-1, N/2(Nyquist freq)]*(2pi/L)  ; total of N/2 pos frequencies

; When plotting 2-sided power spectra, and when going from z(x,y) to zhat(kx,ky) = FFT2D{z(x,y)},
; mathematicians usually write the frequencies in the order
; kmath[1 to N] = [-(N/2-1),...,-2,-1,0,+1,+2,...,+N/2-1,+N/2(Nyquist freq)]*(2pi/L)  ; total of N freqs in rad/m
 
; IDL (and Matlab) stores these frequencies in the order (see the IDL documentation)
; kIDL[1 to N] = [0.0, 1 2, ..., (N-1)/2, N/2(Nyquist freq), -(N-1)/2, ..., -2, -1]*(2pi/L) ; total of N freqs
; This is the (kx,ky) order of zhat(kx,ky) arrays returned by IDL routine FFT.

; The IDL order is generated as follows:
; IDL = FINDGEN((Nx-1)/2) + 1.0 ; = [1,2,...,(N-1)/2]
; kIDL = (2.0*!pi/L)*[0.0, IDL, N/2.0 ,-N/2.0+IDL] ; in rad/m

; The math order can be obtained from the IDL order by a circular shift by N/2-1 to the right:
; kmath = SHIFT(kIDL,Nx/2-1)

; The IDL order can be obtained from the math order by a circular shift by N/2-1 to the left:
; kIDL = SHIFT(kmath,-(N/2-1))

; When defining the one-sided variance spectrum Psi1s for random sampling, we will use (kvec,phi) with 
; vector kvec = sqrt(kx^2 + ky^2) including zero frequencies and -pi/2 <= phi <= pi/2.  This 
; corresponds to 0 and positive kx, and positive and negative ky.  These frequencies are 
; denoted by kx1S = [0.0, kxpos] (Nx/2 +1 values) and ky1S = kymath (Ny values)

; These forms of frequency arrays are computed next for use below.

;----- Compute positive frequencies used for plotting the 1-sided variance spectrum; 
;   these do not include 0 frequency;
;   even spacing in frequency k; 
;   the last frequency is the Nyquist frequency

Nkxpos = Nx/2
Nkypos = Ny/2
; spacing for positive frequencies; Delta's include the 2pi/L values via the kmax and kmin values
; These Deltakx, Deltaky values are used to convert the continuous variance spectrum to a discrete variance function.
Deltakx = (kxmax - kxmin)/float(Nkxpos-1)  
Deltaky = (kymax - kymin)/float(Nkypos-1)
kxpos = kxmin + Deltakx*findgen(Nkxpos) ; in rad/m
kypos = kymin + Deltaky*findgen(Nkypos)

;----- Compute the IDL frequencies needed by the FFT routine:
xIDL = FINDGEN((Nx-1)/2) + 1.0 ; = [1,2,...,(Nx-1)/2]
yIDL = FINDGEN((Ny-1)/2) + 1.0
kxIDL = (2.0*!pi/Lx)*[0.0, xIDL, Nx/2.0 ,-Nx/2.0+xIDL] ; in rad/m
kyIDL = (2.0*!pi/Ly)*[0.0, yIDL ,Ny/2.0, -Ny/2.0+yIDL]

;----- Compute the math frequencies
kxmath = SHIFT(kxIDL,Nx/2-1)
kymath = SHIFT(kyIDL,Ny/2-1)

;----- Compute the 1S frequencies 
kx1S = [0.0, kxpos]
ky1S = kymath

;if(idebug eq 1) then begin
;  print,' kxpos  = ',kxpos
;  print,' kxIDL  = ',kxIDL
;  print,' kxmath = ',kxmath
;  print,' kx1S   = ',kx1S
;  print,' kypos  = ',kypos
;  print,' kyIDL  = ',kyIDL
;  print,' kymath = ',kymath 
;  print,' ky1S   = ',ky1S 
;endif


; ***** COMPUTE THE TWO-SIDED, 2D VARIANCE SPECTRUM Psi2s(kxmath,kymath) *****  

print,' '
print,format='(4x,a,f5.1)', 'The 2D, two-sided variance spectrum is Elfouhaily with a cosine-2S spreading function, with U10 =',U10 
print,' '

; Compute the 2-sided variance spectrum Psi2s(kx,ky) for all kx,ky values. 
; The variance spectrum is computed separately for "downwind" (kx > 0) and upwind (kx < 0)
; directions (kxmath,kymath) since no +/-kvec symmetric is assumed for Psi2s(kvec). 
; However, Psi2s is symmetric in phi about the wind direction, so Psi2s(kx,-ky) = Psi2s(kx,+ky).
; To minimize having multiple large arrays, make multiple calls to ECKV2S_k_phi with 
; scalar k and phi values. This calculation is done only once if multiple random 
; surfaces are generated for a given variance spectrum.
; Psi2s(kx,ky) = Psi2s(k,phi) for the corresponding (kx,ky) and (k,psi) points.
; Note that Psi2s has the magnitude of the one-sided variance spectrum for positive kx,
; even though it has been extended to negative frequencies.

Psi2s = fltarr(Nx,Ny) ; holds all positive and negative kxmath and kymath frequencies
for ikx = 0,Nx-1 do begin ; loop over all kxmath value
  for iky = Ny/2-1,Ny-1 do begin ; non-negative kymath values
    k = SQRT(kxmath[ikx]*kxmath[ikx] + kymath[iky]*kymath[iky])
    phirad = ATAN(kymath[iky],kxmath[ikx])   

    ; option to rescale the positive part of the variance spectrum to resolve the slope variance
    IF (RESCALE eq 0 or k le kp or abs(phirad) gt !pi/2.0) THEN tildefunc = 1.0 ELSE tildefunc = 1.0 + deltah*(k - kp)/(kh - kp)

    Psi2s[ikx,iky] = tildefunc*ECKV2Side_cos2S_k_phi(k,phirad,U10,Sparam)

    ; get negative ky values by symmetry about the downwind-upwind direction phi=0-180
    if iky ge Ny/2 and iky le Ny-2 then Psi2s[ikx,Ny-2-iky] = Psi2s[ikx,iky] 
  endfor
endfor

; set (0,0) frequency to 0 to force MSL = 0:
Psi2s[Nx/2-1,Ny/2-1] = 0.0 

; Psi2s now has all frequencies in math order

;print,' '
;print,' Psi2s(kxmath,kymath); (0,0) at center'
;print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
;print,format='(a8,9x,16f12.3)',' kx =',kxmath
;for iky=Ny-1,0,-1 do begin
;    print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kymath(iky),Psi2s(*,iky)
;endfor

; shift from math to IDL frequency order

Psi2s = SHIFT( Psi2s, -(Nx/2-1), -(Ny/2-1) )

; NOTE: All subsequent calculations are done in IDL frequency order

;print,' '
;print,'Frequency-shifted Psi1s(0,0) [should = (0,0)] =',Psi1s(0,0)

;print,' '
;print,' Psi1s(kxIDL,kyIDL); (0,0) at lower left'
;print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
;print,format='(a8,9x,16f12.3)',' kx =',kxIDL
;for iky=Ny-1,0,-1 do begin
;    print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kyIDL(iky),Psi1s(*,iky)
;endfor



; ***** COMPUTE TIME-INDEPENDENT TERMS FOR USE IN COMPUTING zhat(kx,ky) AT EACH TIME *****
 
; ----- Compute the variance spectrum term

; Mobley's correction to Tessendorf's equations, which correspond to C3 = 1.0:
;C3 = 1.0/SQRT(8.0) ; for 1-sided variance spectra, which need to be divided by 2 to get a 2 sided spectrum
C3 = 0.5 ; for 2-sided variance spectra

; Psi2s is the value of the two-sided, continuous variance spectrum at freq (kx,ky) in units of m^2/(rad/m)^2.  
; Must multiply by Deltakx and Deltakx to get variance (m^2) contained in the finite spatial 
; frequency intervals Deltakx and Deltaky.  See Light and Water Eq. (4.78).

Psiroot = C3*SQRT(Psi2s*Deltakx*Deltaky)

;print,' '
;print,' C3*SQRT(Psi2s*Deltakx*Deltaky)'
;print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
;print,format='(a8,9x,16f12.3)',' kx =',kxIDL
;for iky=Ny-1,0,-1 do begin
;    print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kyIDL(iky),Psiroot(*,iky)
;endfor


; ----- Draw a pair of independent N(0,1) random number for each kx and ky

; These will set the random amplitudes and phases of the waves, or real and imagnary parts of zhat

; RANDOMU is the IDL random number generator (period ~ 10^8); /normal gives normal(0,1) random variables

r = fltarr(Nx,Ny) ; for the random real parts of zhat
s = fltarr(Nx,Ny) ; for the random imaginary parts
for ikx=0,Nx-1 do r(ikx,*) = RANDOMU(seed,Ny,/normal) 
for ikx=0,Nx-1 do s(ikx,*) = RANDOMU(seed,Ny,/normal)

; ----- Compute omega(k) = omega(kx,ky) for all frequencies
 
; The dispersion relation for deep-water gravity waves is omega^2 = g k, 
; where omega(k) = 2 pi/T(k) = angular temporal frequency in rad/sec, 
; T(k) = period of the wave with spatial freq k, and 
; k = |kvec| = angular spatial frequency; k = sqrt(kx^2 + ky^2)

omegak = fltarr(Nx,Ny)
g = 9.81 ; accel of gravity, m/sec^2

for iky=0,Ny-1 do begin
  for ikx = 0,Nx-1 do begin
    k = SQRT( kxIDL(ikx)*kxIDL(ikx) + kyIDL(iky)*kyIDL(iky) )

    IF (QUANTOMEGA eq 0) THEN BEGIN
       ; for physically correct omega use
       omegak(ikx,iky) = SQRT( g * k )
    ENDIF ELSE BEGIN
    ; for quantized omega, which gives exact repeat after TotalTime seconds, use
    omegak(ikx,iky) = FIX(SQRT( g * k )/omega0)*omega0
    ENDELSE
    
  endfor
endfor

print,'omegak(1,1) = ',omegak(1,1)
;print,' '
;print,' omega(k) = omega(kx,ky)'
;print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
;print,format='(a8,9x,16f12.3)',' kx =',kxIDL
;for iky=Ny-1,0,-1 do begin
;    print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kyIDL(iky),omegak(*,iky)
;endfor

; ----- Define zhat(kx,ky), the complex, random Hermitian amplitude array

zhat = !VALUES.F_NAN*COMPLEXARR(Nx,Ny) ; check by initializing zhat as NaN's


;***** LOOP OVER TIME STEPS *****

for itime = 0, Ntimes-1 do begin
  
  time = itime * Deltat
 
; compute cos(omega*t) and sin(omega*t) for the current time step and all spatial frequencies

cosomegat = COS(omegak * time)
sinomegat = SIN(omegak * time)

;----- Compute zhat(kx,ky) at the current time

; The method is based on Tessendorf (2004; Simulating Water Surfaces), as corrected for consistency with
; the chosen wave variance spectrum

; Now loop over all (kx,ky) values, with special cases for the 0 and Nyquist frequencies.
; These equations do NOT not assume that E(-kvec) = E(+kvec):
; Note that for the IDL frequency ordering, k(-j) = k(N-j)

; The non-zero frequencies:
for iky=1,Ny-1 do begin
   for ikx=1,Nx-1 do begin

      zhat(ikx,iky) = COMPLEX( ( r[ikx,iky]*Psiroot[ikx,iky] + r[Nx-ikx,Ny-iky]*Psiroot[Nx-ikx,Ny-iky] )*cosomegat[ikx,iky] $
                             + ( s[ikx,iky]*Psiroot[ikx,iky] + s[Nx-ikx,Ny-iky]*Psiroot[Nx-ikx,Ny-iky] )*sinomegat[ikx,iky] , $
                             - ( r[ikx,iky]*Psiroot[ikx,iky] - r[Nx-ikx,Ny-iky]*Psiroot[Nx-ikx,Ny-iky] )*sinomegat[ikx,iky] $
                             + ( s[ikx,iky]*Psiroot[ikx,iky] - s[Nx-ikx,Ny-iky]*Psiroot[Nx-ikx,Ny-iky] )*cosomegat[ikx,iky] )
   endfor
endfor


idebug = 0
if idebug eq 1 then begin

   print,' '
   print,'After non-zero and non-Nyquist loops, FFT order:'
   print,' '
   print,' Real{zhat}(kxFFT,kyFFT); (0,0) at lower left;  Real{zhat(-k)} = Real{zhat(N-k)}'
   print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
   print,format='(a8,9x,16f12.3)',' kxIDL =',kxIDL
   for iky=Ny-1,0,-1 do begin
      print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kyIDL(iky),REAL_PART(zhat(*,iky))
   endfor

   print,' '
   print,' Imag{zhat}(kx,FFTkyFFT); (0,0) at center;  Imag{zhat(-k)} = -Imag{zhat(N-k)}'
   print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
   print,format='(a8,9x,16f12.3)',' kx =',kxIDL
   for iky=Ny-1,0,-1 do begin
      print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kyIDL(iky),IMAGINARY(zhat(*,iky))
   endfor

   ; for plotting the real and imaginary parts in math order, shift back to math order:
   ; The math order can be obtained from the IDL order by a circular shift by N/2-1 to the right:
   ; kmath = SHIFT(kIDL,Nx/2-1)

   Zreal = SHIFT( REAL_PART(zhat), Nx/2-1, Ny/2-1)
   Zimag = SHIFT( IMAGINARY(zhat), Nx/2-1, Ny/2-1)

   print,' '
   print,'After non-zero and non-Nyquist loops, math order:'
   print,' '
   print,' Real{zhat}(kxmath,kymath); (0,0) at center;  Real{zhat(-k)} = Real{zhat(k)}'
   print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
   print,format='(a8,9x,16f12.3)',' kx =',kxmath
   for iky=Ny-1,0,-1 do begin
      print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kymath(iky),zreal(*,iky)
   endfor

   print,' '
   print,' Imag{zhat}(kxmath,kymath); (0,0) at center;  Imag{zhat(-k)} = -Imag{zhat(k)}'
   print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
   print,format='(a8,9x,16f12.3)',' kx =',kxmath
   for iky=Ny-1,0,-1 do begin
      print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kymath(iky),zimag(*,iky)
   endfor

endif


; Loop over all ky for kx = 0 freq at index 0 and the Nyquist frequency at kx index Nx/2:
 FOR iky=1,Ny/2 DO BEGIN
   
   ikx = 0
      zhat(ikx,iky) = COMPLEX( ( r(ikx,iky)*Psiroot[ikx,iky] + r[ikx,Ny-iky]*Psiroot[ikx,Ny-iky] )*cosomegat[ikx,iky]  $
                             + ( s[ikx,iky]*Psiroot[ikx,iky] + s[ikx,Ny-iky]*Psiroot[ikx,Ny-iky] )*sinomegat(ikx,iky) , $
                             - ( r[ikx,iky]*Psiroot[ikx,iky] - r[ikx,Ny-iky]*Psiroot[ikx,Ny-iky] )*sinomegat(ikx,iky) $
                             + ( s[ikx,iky]*Psiroot[ikx,iky] - s[ikx,Ny-iky]*Psiroot[ikx,Ny-iky] )*cosomegat(ikx,iky) )
      zhat(ikx,Ny-iky) = CONJ( zhat(ikx,iky) )
 ENDFOR
 
 ; Loop over all kx for ky = 0 and Nyquist frequency at ky index Ny/2:
 FOR ikx=1,Nx/2 DO BEGIN

   iky = 0
      zhat(ikx,iky) = COMPLEX( ( r(ikx,iky)*Psiroot[ikx,iky] + r[Nx-ikx,iky]*Psiroot[Nx-ikx,iky] )*cosomegat[ikx,iky]  $
                             + ( s[ikx,iky]*Psiroot[ikx,iky] + s[Nx-ikx,iky]*Psiroot[Nx-ikx,iky] )*sinomegat(ikx,iky) , $
                             - ( r[ikx,iky]*Psiroot[ikx,iky] - r[Nx-ikx,iky]*Psiroot[Nx-ikx,iky] )*sinomegat(ikx,iky) $
                             + ( s[ikx,iky]*Psiroot[ikx,iky] - s[Nx-ikx,iky]*Psiroot[Nx-ikx,iky] )*cosomegat(ikx,iky) )
    zhat(Nx-ikx,iky) = CONJ( zhat(ikx,iky) )
 ENDFOR
 
 idebug = 0
 if idebug eq 1 then begin

    print,' '
    print,'After zero loops, FFT order:'
    print,' '
    print,' Real{zhat}(kxFFT,kyFFT); (0,0) at lower left;  Real{zhat(-k)} = Real{zhat(N-k)}'
    print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
    print,format='(a8,9x,16f12.3)',' kxIDL =',kxIDL
    for iky=Ny-1,0,-1 do begin
       print,format='(a8,i5,f9.3,16e12.2)',' iky, kyIDL =',iky,kyIDL(iky),REAL_PART(zhat(*,iky))
    endfor

    print,' '
    print,' Imag{zhat}(kx,FFTkyFFT); (0,0) at center;  Imag{zhat(-k)} = -Imag{zhat(N-k)}'
    print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
    print,format='(a8,9x,16f12.3)',' kx =',kxIDL
    for iky=Ny-1,0,-1 do begin
       print,format='(a8,i5,f9.3,16e12.2)',' iky, kyIDL =',iky,kyIDL(iky),IMAGINARY(zhat(*,iky))
    endfor

    ; for plotting the real and imaginary parts in math order, shift back to math order:
    ; The math order can be obtained from the IDL order by a circular shift by N/2-1 to the right:
    ; kmath = SHIFT(kIDL,Nx/2-1)

    Zreal = SHIFT( REAL_PART(zhat), Nx/2-1, Ny/2-1)
    Zimag = SHIFT( IMAGINARY(zhat), Nx/2-1, Ny/2-1)

    print,' '
    print,'After zero loops, math order:'
    print,' '
    print,' Real{zhat}(kxmath,kymath); (0,0) at center;  Real{zhat(-k)} = Real{zhat(k)}'
    print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
    print,format='(a8,9x,16f12.3)',' kx =',kxmath
    for iky=Ny-1,0,-1 do begin
       print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kymath(iky),zreal(*,iky)
    endfor

    print,' '
    print,' Imag{zhat}(kxmath,kymath); (0,0) at center;  Imag{zhat(-k)} = -Imag{zhat(k)}'
    print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
    print,format='(a8,9x,16f12.3)',' kx =',kxmath
    for iky=Ny-1,0,-1 do begin
       print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kymath(iky),zimag(*,iky)
    endfor

 endif 
 
; set (kx,ky) = (0,0) to (0.0,0.0) to force mean sea level = 0

zhat[0,0] = COMPLEX(0.0,0.0) 

; zhat as just defined has the frequencies in IDL order, as needed by the FFT routine.

idebug = 0
if idebug eq 1 then begin
 
   print,' '
   print,'Extra calculations: Final zhat, FFT order:'
   print,' '
   print,' Real{zhat}(kxFFT,kyFFT); (0,0) at lower left;  Real{zhat(-k)} = Real{zhat(N-k)}'
   print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
   print,format='(a8,9x,16f12.3)',' kxIDL =',kxIDL
   for iky=Ny-1,0,-1 do begin
      print,format='(a8,i5,f9.3,16e12.2)',' iky, kyIDL =',iky,kyIDL(iky),REAL_PART(zhat(*,iky))
   endfor

   print,' '
   print,' Imag{zhat}(kx,FFTkyFFT); (0,0) at center;  Imag{zhat(-k)} = -Imag{zhat(N-k)}'
   print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
   print,format='(a8,9x,16f12.3)',' kx =',kxIDL
   for iky=Ny-1,0,-1 do begin
      print,format='(a8,i5,f9.3,16e12.2)',' iky, kyIDL =',iky,kyIDL(iky),IMAGINARY(zhat(*,iky))
   endfor
   
   ; for plotting the real and imaginary parts in math order, shift back to math order:
   ; The math order can be obtained from the IDL order by a circular shift by N/2-1 to the right:
   ; kmath = SHIFT(kIDL,Nx/2-1)

   Zreal = SHIFT( REAL_PART(zhat), Nx/2-1, Ny/2-1)
   Zimag = SHIFT( IMAGINARY(zhat), Nx/2-1, Ny/2-1)

   print,' '
   print,'Final zhat, math order:'
   print,' '
   print,' Real{zhat}(kxmath,kymath); (0,0) at center;  Real{zhat(-k)} = Real{zhat(k)}'
   print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
   print,format='(a8,9x,16f12.3)',' kx =',kxmath
   for iky=Ny-1,0,-1 do begin
      print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kymath(iky),zreal(*,iky)
   endfor

   print,' '
   print,' Imag{zhat}(kxmath,kymath); (0,0) at center;  Imag{zhat(-k)} = -Imag{zhat(k)}'
   print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
   print,format='(a8,9x,16f12.3)',' kx =',kxmath
   for iky=Ny-1,0,-1 do begin
      print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kymath(iky),zimag(*,iky)
   endfor

endif


; check to make sure zhat is Hermitian
;idebug = 1
;if(idebug eq 1) then begin
;  Zreal = REAL_PART(zhat)
;  Zimag = IMAGINARY(zhat)
;  print,' '
;  print,' Check that zhat is Hermitian: Real{zhat(-nu(j))} = Real{zhat(nu(j))} and Imag{zhat(-nu(j))} = -Imag{zhat(nu(j))}'
;  print,'    j      nu(j)       Real{zhat(j)}  Imag{zhat(j)}   omega(j)t  cos(omega(j)t)  sin(omega(j)t)'
;  for j=0,Nsamp-1 do print,format='(i5,6e15.4)',j,nu(j),Zreal(j),Zimag(j),omegajt(j),cosomegat(j),sinomegat(j) 
;endif

; ----- Take the inverse FFT to get the complex surface values 

zcomplx = FFT(zhat,/inverse)

; The sea surface elevations are the real part of the complex surface

zsurf = REAL_PART(zcomplx)


; ----- Optional checks on the generated surface

;print,' '
;print,'Checks on the generated z(x,y):'
;print,' '

; compute the mean sea surface, which should be zero, and mean squared surface elevation
sumz = 0.0
sumzsq = 0.0
for ix = 0,Nx-1 do begin
  for iy = 0,Ny-1 do begin
    sumz = sumz + zsurf(ix,iy)
    sumzsq = sumzsq + zsurf(ix,iy)*zsurf(ix,iy)
  endfor
endfor

zavg = sumz/(float(Nx)*Float(Ny))
;print,'   avg z(x,y) (should = 0) = ',zavg

; the significant wave height from avg of z^2
avgzsq = sumzsq/(float(Nx)*Float(Ny))
H13 = 4.0*sqrt(avgzsq) ; approximate significant wave height
;print,'   significant wave height H_1/3 = 4 SQRT(avg{z^2}) =',H13

; compute the mean square slopes and mean slope angle in x and y directions
dzdx2 = 0.0
thetax = 0.0
for ix = 1,Nx-1 do begin
  for iy = 0,Ny-1 do begin
    dzdx2 = dzdx2 + (zsurf(ix,iy) - zsurf(ix-1,iy))^2
    thetax = thetax + abs(atan(zsurf(ix,iy) - zsurf(ix-1,iy),Deltax)*!radeg)
  endfor
endfor
dzdx2 = dzdx2/(Deltax*Deltax*float(Nx-1)*float(Ny))
thetax = thetax/(float(Nx-1)*float(Ny))

dzdy2 = 0.0
thetay = 0.0
for iy = 1,Ny-1 do begin
  for ix = 0,Nx-1 do begin
    dzdy2 = dzdy2 + (zsurf(ix,iy) - zsurf(ix,iy-1))^2
    thetay = thetay + abs(atan(zsurf(ix,iy) - zsurf(ix,iy-1),Deltay)*!radeg)
  endfor
endfor
dzdy2 = dzdy2/(Deltay*Deltay*float(Nx)*float(Ny-1))
thetay = thetay/(float(Nx)*float(Ny-1))

; check on inversion of Hermitian amps to get a real z
; imaginary part of zcomplx should be 0 to within roundoff error
maximagz = max(abs(IMAGINARY(zcomplx)))
if(maximagz gt 1.0e-10) then print,' ERROR:  max |Imag{z}| (should = 0) = ',maximagz

; ***** THE SEA SURFACE HAS NOW BEEN GENERATED; SAVE OR PLOT AS DESIRED *****

; ***** PLOT THE GENERATED SURFACE *****

CASE plottype OF

'WIN' : BEGIN

   ; open a window; cgDisplay makes the background white
   ; This window is recreated for each time step

   cgDisplay,XSIZE=xpixsize,YSIZE=ypixsize,WID=0;,color=backgroundcolor
   Device,Set_Font='Helvetica',/TT_Font
   
        END

'PNG' : BEGIN

   ; create the plot file name for this time step
   ; (the CASE statement is valid with the test is "true", which is equivalent to test variable "1"
   CASE 1 OF
      itime lt 10 : PlotFileName = plotroot + '_000' + string(itime,format='(i1)') + '.eps'
      (10 le itime) and (itime lt 100) : PlotFileName = plotroot + '_00' + string(itime,format='(i2)') + '.eps'
      (100 le itime) and (itime lt 1000) : PlotFileName = plotroot + '_0' + string(itime,format='(i3)') + '.eps'
      (itime ge 1000) : PlotFileName = plotroot + '_' + string(itime,format='(i4)')
   ENDCASE
 
   ; open the file for the eps plot
   
   PS_Start,filename=PlotFileName, /encapsulated, font=1, tt_font='Helvetica', $
           charsize=charsize, default_thickness=2, /nomatch, xsize=xsize, ysize=ysize,/quiet
   
       END

ENDCASE

; ----- for plotting of surface elevation as a periodic function of (x,y), 
; add values at z(Lx,*) = z(0,*) and z(*,Ly) = z(*,0) 

;zplot=fltarr(Nx+1,Ny+1) was established above
zplot(0:Nx-1,0:Ny-1) = zsurf
zplot(Nx,0:Ny-1) = zsurf(0,*)
zplot(0:Nx-1,Ny) = zsurf(*,0)
zplot(Nx,Ny) = zsurf(0,0)

minz = min(zplot,max=maxz)
;print,'minz, maxz = ',minz, maxz
;maxz = max(abs(zsurf))

; reset z axis range to nice values
minz = -3.0
maxz = 3.0


; ----- Set up plot for a shaded surface seen in 3D perspective

; axes are plotted below for best appearance

cgSurf,zplot,xplot,yplot, /save, $ 'skirt=minz,$
  /shaded, /elevation_shading, shades=BytScl(zplot), $ ;,/nodata
  xrange=[0,Lx], xtitle=xlabel, xstyle=5, $  ; xstyle=5 forces no x axis
  yrange=[0,Ly], ytitle=ylabel, ystyle=5,$
  zrange=[minz,maxz], ztitle=zlabel, rotx=75,rotz=-60, zstyle=5, $ ; rotx=75,rotz=-50,
  charsize=1.2*!p.charsize

; overlay particular lines if desired
   jy = Ny/2
   ; cgplotS,xplot[*],yplot[jy],zplot[*,jy],/T3D,color=cgColor('orange'),thick=3
   ix = Nx/2
   ; cgplotS,xplot[ix],yplot[*],zplot[ix,*],/T3D,color=cgColor('red'),thick=3


; save particular lines if desired   
isave = 0
zsaveFileName = 'Y1D_U10_L100_1024.txt'

IF isave EQ 1 THEN BEGIN

   openw,nunit,zsaveFileName,/get_lun
   printf,nunit,'1D slice of a random sea surface realization'
   printf,nunit,format='(2i5,3f10.1,f10.2)',Nx,Ny,Lx,Ly,U10,time,' = Nx,Ny, Lx,Ly, U10, time'
   printf,nunit,format='("    i    j      x(i)      y(j)    z(x,y)")'

    ; Save an x slice at constant y(jy) value:
    ; FOR i=0,Nx DO printf,nunit,format='(2i5,3f10.3)',i,j,xplot(i),yplot(jy),zplot(i,jy)
    ; 
    ; Save a y slice at constant x(ix) value:
     FOR j=0,Ny DO printf,nunit,format='(2i5,3f10.3)',i,j,xplot(ix),yplot(j),zplot(ix,j)
  
   free_lun,nunit
    
ENDIF    

; ----- Add axes in desired locations

xaxmin = !x.crange[0]
xaxmax = !x.crange[1]
yaxmin = !y.crange[0]
yaxmax = !y.crange[1]
zaxmin = !z.crange[0]
zaxmax = !z.crange[1]

; xaxis
axis,xaxmax,yaxmin,zaxmin,xaxis=0,xtitle=xlabel,charsize=1.2*!p.charsize,/T3D,ticklen=-0.02,color=cgColor('black')

; yaxis:
axis,xaxmax,yaxmin,zaxmin,yaxis=1,ytitle=ylabel,charsize=1.2*!p.charsize,/T3D,ticklen=-0.02,color=cgColor('black')

; zaxis:
axis,xaxmin,yaxmin,zaxmin,zaxis=1,ztitle=zlabel,charsize=3.5*!p.charsize,/T3D,ticklen=-0.02,color=cgColor('black')

  ; optionally add a skirt, since skirt=zmin doesn't seem to work on cgSurf
  
skirt = 0
IF skirt ne 0 then begin

  xskirt = [0.0,xplot,xplot(Nx),0.0]
  yskirt = replicate(0.0,Ny+4)
  ztemp = REFORM(zplot[*,0])
  zskirt = [zaxmin,ztemp,zaxmin,zaxmin]
  cgPolygon,xskirt,yskirt,zskirt,/fill,color='gray',fcolor='gray',/T3D
   
  yskirt = [0.0,yplot,yplot(Ny),0.0]
  xskirt = replicate(xplot(Nx),Nx+4)
  ztemp = REFORM(zplot[Nx,*])
  zskirt = [zaxmin,ztemp,zaxmin,zaxmin]
  cgPolygon,xskirt,yskirt,zskirt,/fill,color='dark gray',fcolor='dark gray',/T3D

endif ; skirt

  
  ; add a label at the top
  toplabel = 'U!d10!n =' +  string(U10,format='(f5.1)') + ' m s!u-1!n;' +  $
   '  (N!dx!n,N!dy!n) = (' + string(Nx,format='(i4)') + ',' + string(Ny,format='(i4)') + ')'
   
  toplabel2 = 'ECKV; ' + Greek('Phi',/PS) + '= C!dS!ncos!u2S!n(' + Greek('phi',/PS) + '/2), S =' + string(Sparam,format='(i3)')

  cgText,0.5,0.94,toplabel,align=0.5,charsize=1.0*!p.charsize,/normal
  cgText,0.5,0.87,toplabel2,align=0.5,charsize=1.0*!p.charsize,/normal
   
;   ; add time label 
  timelabel = 'time =' + string(time,format='(f7.2)') + ' s'
  ytop = 0.85
  cgText,0.18,ytop,timelabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
;    
   ; add statistics labels
  swhlabel  = 'H!d1/3!n =' + string(H13,format='(f7.2)') + ' m'
  mssxlabel = 'mss!dx!n =' + string(dzdx2,format='(f7.3)')
  mssylabel = 'mss!dy!n =' + string(dzdy2,format='(f7.3)')
  msstotlabel = 'mss!d !n =' + string(dzdx2 + dzdy2,format='(f7.3)')
  thetaxlabel =  '!9' + string("341B) + '!x' + Greek('theta',/PS) + '!dx!n !9' $
                      + string("361B) + '!x =' + string(thetax,format='(f7.3)') + ' deg'
  thetaylabel =  '!9' + string("341B) + '!x' + Greek('theta',/PS) + '!dy!n !9' $
                      + string("361B) + '!x =' + string(thetay,format='(f7.3)') + ' deg'                      
  xlab = 0.75
  dylab = 0.05
  cgText,xlab,ytop,swhlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red') 
  cgText,xlab,ytop-dylab,mssxlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red') 
  cgText,xlab,ytop-2*dylab,mssylabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
  cgText,xlab,ytop-3*dylab,msstotlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
  cgText,xlab,ytop-4*dylab,thetaxlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
  cgText,xlab,ytop-5*dylab,thetaylabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
    
; close the plot just created

CASE plottype OF

'WIN' : BEGIN
  ; return to the original device
  Set_Plot,thisDevice
        END
        
'PNG': BEGIN  

   ; clean up and close the plot file
   ; /PNG creates a high-resolution png file
   ; /delete_ps deletes the intermediate Postscript file (omit to also create an EPS file)

   PS_End,Resize=50,/PNG,/delete_ps

   ;print,'image file ', PlotFileName,' closed OK'

   END
   
ENDCASE 
 
print,' time step',itime,' of',Ntimes,' completed' ; at ', SYSTIME(1),' sec'

endfor ;itime loop


; ******* save the last surface if desired
; move inside the time step loop to save every surface

isave = 0
if isave ne 0 then begin
   openw,nunit,'zxy_tlast.txt',/get_lun
   printf,nunit,format='(2i5,3f10.3,a)',Nx,Ny,Lx,Ly,U10,time,' = Nx,Ny, Lx,Ly, U10, time'
   printf,nunit,format='("    j      x(j)      z(j)")'

   for j=0,Nsamp-1 do printf,nunit,format='(i5,2f10.3)',j,x(j),zsurf(j)
   free_lun,nunit
endif




print,'end Animate2D'
end


