@Pierson_Moskowitz_k
@ECKVomni_k

pro FFT1Dround

; This routine shows how to use a one-sided variance spectrum S_1s and 1D inverse FFTs to generate 
; random 1D sea surfaces.  A round trip calculation is done to check the results.

; This generated tutorial Fig 2.4.

; This version uses angular spatial frequency k = 2 pi/wavelength [rad/m] as the spatial frequency variable.

; The upper left panel is the original wave variance spectrum and sampled points
; The upper right panel is the retrived wave variance spectrum (the periodigram for the generated surface)
; The bottom panel is the generated surface wave profile

; ***************************************************************************************************
; * This code is copyright (c) 2016 by Curtis D. Mobley.                                            *
; * Permission is hereby given to reproduce and use this code for non-commercial academic research, *
; * provided that the user suitably acknowledges Curtis D. Mobley in any presentations, reports,    *
; * publications, or other works that make use of the code or its output.  Depending on the extent  *
; * of use of the code or its outputs, suitable acknowledgement can range from a footnote to offer  *
; * of coauthorship.  Further questions can be directed to curtis.mobley@sequoiasci.com.            *
; ***************************************************************************************************

;-----------------------------------------------------------------------------------------------------------
;| This routine uses Coyote Graphics routines rather than standard IDL for plotting; see www.idlcoyote.com |
;| and "Coyote's Guide to Traditional IDL Graphics: Using Familiar Tools Creatively" by David Fanning      |
;| Any comments below like "CG page xxx" refer to this book.                                               |
;| CG routines are wrappers for standard IDL routines, which make it easier to do certain things like      |
;| create vector eps output files; cgPlot replaces the standard Plot, etc.                                 |
;-----------------------------------------------------------------------------------------------------------

; Written 27 July 2014 by Curtis Mobley.

;-----------------  BEGIN USER INPUT  ----------------

idebug = 0 ; set to 1 for extra debugging printout

; define the wind speed at 10 m above MSL for use in the variance spectrum
U10 = 5.0 ; in m/s

Nsamp = 1024 ; number of samples of sea surface elevation to be generated; MUST be a power of 2 for the FFT

; define the spatial region and sampling resolution for the waves to be generated

InputOption = 2

CASE InputOption OF
  1: BEGIN
        labelInOpt = ' The spatial resolution is defined by the sample spacing and number of sample points'
        ; smallest resolvable wavelength is 2*Deltax
        Deltax = 0.005 
        L = Deltax*float(Nsamp)
     END 
  2: BEGIN
        labelInOpt = ' The spatial resolution is defined by the spatial region and number of sample points'
        ; largest resolvable wave is the length of the spatial region
        L = 100 ; ; length of surface region in meters 
        Deltax = L/float(Nsamp)
     END    
ENDCASE

; choose a variance (variance) spectrum

Espectrum = 'PM' ; for Pierson and Moskowicz (1964)
;Espectrum = 'ECKV' ; for ElFouhaily et al (1997)

; choose a root name for the output plots.
; PNG and EPS plots will be generated with the names of PlotRootName.eps and PlotRootName.png
PlotRootName ='Fig2.4'

; define an initial seed for random number generation; an IDL longword integer
seed =   838239L; 538832L ;33202L; 838392L

;-----------------  END USER INPUT  ----------------------

minwave = 2.0*Deltax ; min resolvable wavelength in m (2 point wave)
kmin = 2.0*!pi/L ; min (fundamental) spatial frequency in rad/m; angular frequency of longest resolvable wave
Nyquist = !pi/Deltax ; Nyquist spatial freq (2 pi)/(2 Delta) in rad/m
kmax = Nyquist

print,' '
print,' Generation of a random 1D sea surface z(x) given a one-sided variance spectrum S_1s(k)'
print,' '
print, format='(2x,a)',labelInOpt
print, format='(4x,a,i5)', 'Number of spatial samples = Nsamp =',Nsamp
print, format='(4x,a,f7.4)', 'Sample spacing Deltax [m] =', Deltax
print, format='(4x,a,f8.1)', 'Max resolvable wavelength = L [m] =',L
print, format='(4x,a,f7.4)', 'Min resolvable wavelength = 2*Deltax [m] =', minwave
print, format='(4x,a,f8.4)', 'Min resolvable (fundamental) frequency k(0) = 2 pi/L [rad/m]= ',kmin
print, format='(4x,a,f8.4)', 'Max resolvable (Nyquist) frequency k(Nsamp/2) = 2 pi/(2.0*Deltax) [rad/m] = ',kmax 
print,' '
print, format='(4x,a,i12)', 'Seed for random number generation =',seed

; Compute positive frequencies used for plotting the variance spectrum; these do not include 0 frequency
; even spacing Deltak in frequency k; the last frequency is the Nyquist frequency
Nkpos = Nsamp/2
Deltak = (kmax - kmin)/float(Nkpos-1) ; spacing for positive frequencies; Deltak = Nyquist freq
kpos = kmin + Deltak*findgen(Nkpos)

kE = kpos; for the one-sided variance spectrum, excluding the k = 0 DC component

; compute Nsamp frequencies ordered as needed for inv FFT in IDL
; These frequencies are used to compute the random zhat; frequency = 0 is included, but there is no negative Nyquist frequency
k = [0.0,kpos,-reverse(kpos[0:Nsamp/2-2])] ; Nsamp 
if(idebug eq 1) then print,' k = ',k

; values for plotting a smooth variance spectrum; positive frequencies only
; use equal spacing in log(k) for nice appearance
Nkplot = 100
kminplot = 0.01
kmaxplot = 100.0
kminlog = alog10(kminplot)
kmaxlog = alog10(kmaxplot)
dkplotlog = (kmaxlog - kminlog)/float(Nkplot-1)
kplotlog = kminlog + dkplotlog*findgen(Nkplot)
kplot = 10.0^kplotlog

;print,'Deltak = ',Deltak

;------------------------

; select the variance spectrum

CASE Espectrum OF

'PM': BEGIN

        print,' '
        print,format='(4x,a,f5.1)', 'The chosen variance spectrum is Pierson-Moskowitz with U10 =',U10 
        print,' '

        ; discrete values for wave generation with FFT:
        Sk = Pierson_Moskowitz_k(kE,U10)
        if(idebug eq 1) then print,'S_PM(k) = ',Sk
 
        ; values for plotting the whole spectrum as smooth function of positive frequencies       
        Skplot = Pierson_Moskowitz_k(kplot,U10)
        if(idebug eq 1) then print,'S_PMplot(k) = ',Skplot
        
        toplabel = 'Pierson-Moskowitz: U!d10!n =' + string(U10,format='(i2)') + ' m s!u-1!n; N =' + string(Nsamp,format='(i5)')
  
      END

'ECKV': BEGIN

        print,' '
        print,format='(4x,a,f5.1)', 'The chosen variance spectrum is Elfouhaily et al. with U10 =',U10 
        print,' '

        ; discrete values for wave generation with FFT:
        Sk = ECKVomni_k(kE,U10)
        if(idebug eq 1) then print,'S_ECKV(k) = ',Sk
 
        ; values for plotting the whole spectrum as smooth function of positive frequencies       
        Skplot = ECKVomni_k(kplot,U10)
        if(idebug eq 1) then print,'S_ECKVplot(k) = ',Skplot
        
        toplabel = 'Elfouhaily: U!d10!n =' + string(U10,format='(f5.1)') + ' m s!u-1!n; N =' + string(Nsamp,format='(i5)')
 
        END

ENDCASE


;------------------------

;***** Define a 3 panel layout

; region 0  |  region 1
; region 2 (wide)

reg = fltarr(3,4,/nozero)

; plot coords for !p.region are in normal coordinates (0 to 1)
; adjust values as needed for the desired appearance

xl = 0.04
xr = 0.50
dx = 0.43

yb = 0.01 ; the bottom of the bottom row
yt = 0.46
dy = 0.47

reg[0,*] = [xl,yt,xl+dx,yt+dy]
reg[1,*] = [xr,yt,xr+dx,yt+dy]
reg[2,*] = [xl,yb,xr+dx,yb+dy]

; set up output for a vector eps file

PlotFileName =PlotRootName + '.eps'

!x.margin = [6,1]
!y.margin = [4,1]
;!x.Omargin = [2,2]
;!y.Omargin = [0,3]

aspectratio = 0.8
xsize= 6.5
ysize=xsize*aspectratio
charsize = 1.2
!p.charsize = charsize

PS_Start,filename=PlotFileName, /encapsulated, font=1, tt_font='Helvetica', $
         charsize=charsize, default_thickness=2, /nomatch, xsize=xsize, ysize=ysize
 
; choose color order; can see with colors = PickColorName()
kolors = ['Black','Magenta','Blue','Cyan','Green','Gold','ORG5','red','TG5','TG4','forest green']

;------------------------

;  plot E1S(k) in upper left panel
;  calls to plot routines must use /noerase

!p.region = reg(0,*)

xlabel = 'spatial frequency k [rad/m]'
ylabel = 'variance density S [m!u2!n/(rad/m)]'
Eminplot = 1.0e-12
Emaxplot = 1.0

cgPlot,kplot,Skplot, /nodata,/noerase,/xlog,/ylog, $
  xrange=[kminplot,kmaxplot],xtitle=xlabel, $
  yrange=[Eminplot,Emaxplot],ytitle=ylabel

cgOplot,kplot,Skplot,color=cgColor('blue'),thick=2 ;,psym=-plotsym(/circle,/fill,scale=0.7),linesty=0
cgOplot,kE,Sk,color=cgColor('red'),thick=2,psym=plotsym(/circle,/fill,scale=0.7),linesty=0
;cgOplot,kE[0],Sk[0],color=cgColor('red'),thick=2,psym=plotsym(/circle,/fill,scale=0.7),linesty=0
;cgOplot,kE[1:Nsamp/2-2],Sk[1:Nsamp/2-2],color=cgColor('red'),thick=2,psym=plotsym(/vline,/fill,scale=0.7),linesty=0
;cgOplot,kE[Nsamp/2-1],Sk[Nsamp/2-1],color=cgColor('red'),thick=2,psym=plotsym(/circle,/fill,scale=0.7),linesty=0

print,'  Resolved frequency range: kE[0], kE[Nsamp/2-1] = ',kE[0], kE[Nsamp/2-1]


; ******** generate a random Hermitian zhat(frequency)

; When going from z(x) to zhat = FFT(z), the math usually writes the frequencies in the order
; nu[1 to Nsamp] = [-(Nsamp/2-1),...,-2,-1,0,+1,+2,...,+Nsamp/2-1,+Nsamp/2]/L
; 
; IDL stores these frequencies in the order (see the IDL documentation)
; [0.0, 1/L, 2/L, ..., (Nsamp-1)/2L, Nsamp/2L(2-point wave; Nyquist freq), -(Nsamp-1)/2L, ..., -1/L] ; total of Nsamp freqs
; 
; When creating a random zhat, the order of the random amplitudes must be in the IDL frequency order, and
; the real and imag parts must give a Hermitian complex amplitude, so that the inverse FFT gives a real function
; Hermitian means H*(-nu) = H(nu); H* = complex conjugate of H
;
; The frequencies returned by the IDL forward FFT are computed using (see the IDL documentation)
; xnu = FINDGEN((Nsamp-1)/2) + 1.0 ; = [1,2,...,(Nsamp-1)/2]
; nu = [0.0,xnu,Nsamp/2.0,-Nsamp/2.0+xnu]/L ; in 1/m

; make a two-sided variance spectrum S2sided(nu) for easy indexing and with frequencies ordered as needed by IDL
; Note that variance spectra satisfy E(-k) = E(k) and E(k=0) = 0
; Recall that Sk is positive frequencies only
; Numerical factors are all contained in C3 below
S2sided = [0.0,Sk,REVERSE(Sk[0:Nsamp/2-2])] ; the 1/2 to converts a one-sided to a two-sided spectrum is included in C3 below

;help,S2sided
if(idebug eq 1) then print,'S2sided = ',S2sided

S2root = SQRT(S2sided*Deltak) ; sqrt of the variance density times Deltak gives the amplitude at each positive k
if(idebug eq 1) then print,'S2root = ',S2root

; C3 is the product of
; the the missing 1/sqrt(2) from sqrt(E1S/2) in Tessendorf's Eq. 41
; the 1/sqrt(2) in the definition of hhat_o in Tessendorf's Eq 41
; the missing 1/sqrt(2) in Tessendorf's equation 43
C3 = 1.0/SQRT(8.0)

; RANDOMU is the IDL random number generator (period ~ 10^8); /normal gives normal(0,1) random variables

rj = RANDOMU(seed,Nsamp,/normal) ; for random numbers r(k)
sj = RANDOMU(seed,Nsamp,/normal) ; for random numbers s(k)

if(idebug eq 1) then print,'r(j) = ',rj
if(idebug eq 1) then print,'s(j) = ',sj

; define the random complex Hermitian zhat array

zhat = COMPLEXARR(Nsamp)

; define a random Hermitian zhat; C3 is the normalizing 1/sqrt(2) factor missing from Tessendorf eq 43
; note that for the IDL frequency ordering, nu(-k) = nu(Nsamp-k)
for j=1,Nsamp-1 do zhat(j) = C3*S2root(j)*COMPLEX(rj[j] + rj[Nsamp-j], sj[j] - sj[Nsamp-j])

; set k = 0 to 0.0 to force MSL = 0
zhat[0] = COMPLEX(0.0,0.0)  

Zreal = REAL_PART(zhat)
Zimag = IMAGINARY(zhat)
; check to make sure zhat is Hermitian
if(idebug eq 1) then begin
   print,' '
   print,' Check that zhat is Hermitian: Real{zhat(-j)} = Real{zhat(j)} and Imag{zhat(-j)} = -Imag{zhat(j)}'
   print,'    j      k(j)       Real{zhat(j)}  Imag{zhat(k)}'
   for j=0,Nsamp-1 do print,format='(i5,3e15.4)',j,k(j),Zreal(j),Zimag(j) 
endif

; take the inverse FFT to get the surface ampltudes as the real part of the inverse

zcomplx = FFT(zhat,/inverse)
;help,zcomplx

zsurf = REAL_PART(zcomplx)
x = Deltax*findgen(Nsamp)

if(idebug eq 1) then print,'zsurf = ',zsurf

print,' '
print,'Checks on the generated z(x):'
print,' '
; compute the mean sea surface, which should be zero
zavg = TOTAL(zsurf)/float(Nsamp)
print,'   zavg (should = 0) = ',zavg

; check on inversion of Hermitian amps to get a real z
; imaginary part of zcomplx should be 0 to within roundoff error
maximagz = max(abs(IMAGINARY(zcomplx)))
print,'   max |Imag{z}| (should = 0) = ',maximagz

zimag = IMAGINARY(zcomplx)
if(idebug eq 1) then print,'zimag = ',zimag ; should be all zeros

avgzsq = total(zsurf*zsurf)/float(Nsamp)
H13 = 4.0*sqrt(avgzsq) ; approximate significant wave height
print,'   significant wave height H_1/3 = 4 avg{z^2} =',H13,' m'

; plot the generated surface

!p.region = reg(2,*)

maxz = max(abs(zsurf))

cgPlot,x,zsurf, /nodata, /noerase, $
   xrange=[0.0,L],/xstyle,xtitle='distance [m]', $
   yrange=[-maxz,maxz],ytitle='surface elevation [m]'
   
cgOPlot,x,zsurf,color=cgColor('red'),thick=2;,psym=-plotsym(/circle,/fill,scale=0.7) 

; plot header

cgText,0.5,0.96,toplabel,align=0.5,charsize=1.1*!p.charsize,/normal

; ******* save the surface if desired

isave = 0
if isave ne 0 then begin
  
   openw,nunit,'z1D_' + PlotRootName + '.txt',/get_lun
   printf,nunit,format='(i5,2f10.3,a)',Nsamp,L,U10,' = Nsamp, L, U10'
   printf,nunit,format='("    j      x(j)      z(j)")'

   for j=0,Nsamp-1 do printf,nunit,format='(i5,2f10.3)',j,x(j),zsurf(j)
   free_lun,nunit
   
endif


;***** take the forward FFT of the generated surface to obtain the periodogram

; make z a complex number for the IDL FFT routine
zcomplx = COMPLEX(zsurf)
help,zcomplx
;print,'zcomplx = ',zcomplx

;check Parseval's relation for the amplitudes and generated surface:
sumz2 = 0.0
for i=0,Nsamp-1 do sumz2 = sumz2 + abs(zsurf(i))^2
print,"Parseval's identity"
print,'   sum z^2 =',sumz2

sumzhat2 = 0.0
for i=0,Nsamp-1 do sumzhat2 = sumzhat2 + abs(zhat(i))^2
print,'   N * sum zhat^2 =',Nsamp*sumzhat2

;sumz2 = 0.0
;for i=0,Nsamp-1 do sumz2 = sumz2 + abs(zcomplx(i))^2
;print,'   complex: sum zhat^2 =',sumz2

; take the FFT to get the Fourier amplitudes for the particular surface

zhatamp = FFT(zcomplx)

; compute the 1-sided periodogram P1S for the particular surface

P1S = fltarr(Nsamp/2+1)
P1S[0] = abs(zhatamp[0])^2
P1S[Nsamp/2] = abs(zhatamp[Nsamp/2])^2
for j=1,Nsamp/2-1 do P1S(j) = 2.0*abs(zhatamp[j])^2

; divide by Deltak to get a density
P1S = P1S/Deltak

if(idebug eq 1) then begin
  print,'kE = ',kE
  print,'
  print,'P1S = ',P1S
endif
;help,kplot
;help,Skplot
;help,kE
;help,P1S

; plot P1S, which contains random noise, and compare with the original E1S

!p.region = reg(1,*)

if(idebug eq 1) then print,'max(kplot) = ',max(kplot)
minE = min(Sk,max=maxE)
if(idebug eq 1) then print,'min,max(E) = ',minE,maxE


cgPlot,kplot,Skplot, /nodata, /xlog,/ylog,/noerase,$
  xrange=[kminplot,kmaxplot],xtitle=xlabel, $
  yrange=[Eminplot,Emaxplot],ytitle=ylabel

cgOplot,kE,P1S[1:Nsamp/2],color=cgColor('red'),thick=2,linesty=0
cgOplot,kplot,Skplot,color=cgColor('blue'),thick=2 ;,psym=-plotsym(/circle,/fill,scale=0.7),linesty=0

; reference label at bottom left of plot
cgText,0.05,0.01,"cgFFT1D_roundtrip.pro",charsize=0.7*!p.charsize,/normal;,color=cgColor('gray')

; clean up and close the plot file
; Use /PNG to create a high-resolution png file
; Use /delete_ps to delete the intermediate Postscript file (omit to also create an EPS file)
; NOTE: see documentation in PS_End for additional software (ImageMagick) required to create PNG files from EPS files.

PS_End,Resize=50,/PNG ;close and fix the eps file and create a png version

print,'image file ', PlotFileName,' closed OK'

;!p.multi = 0 ; reset to normal layout
;
; At the end of plotting, restore default plot region
; !p.region[x0,y0,x1,yt], where (x0, y0) is the origin, and (x1, y1) is the upper right corner. 
!p.region = [0,0,1,1]

print,'end FFT1Dround'
end


