@plotsym ; a routine for plotting symbols of various kinds

pro FFT2D4

; This routine is for learning about 2D FFTs.

; This routine is intended to aid in understanding the math vs FFT frequency orders, the shifting of 
; array orders for plotting vs FFT routines, Hermitian symmetry of the amplitudes, etc.

; This is the "forward" problem: sea surface to amplitudes to spectrum
 
; This routine generates a 2D wave surface in physical space, and then takes a 2D forward FFT
; to get the corresponding complex amplitudes and variance spectrum in Fourier space.
; This version uses angular spatial frequency k = 2 pi nu = 2 pi/wavelength [rad/m] as the frequency variable.

; The results are presented as 4 contour plots:
;   The upper left panel is the wave surface in physical space z(x,y)
;   The upper right panel is the real part of the FFT
;   The lower left panel is the imaginary part of the FFT
;   The lower right panel is the variance spectrum Psi(kx,ky)
; 
; ***************************************************************************************************
; * This code is copyright (c) 2016 by Curtis D. Mobley.                                            *
; * Permission is hereby given to reproduce and use this code for non-commercial academic research, *
; * provided that the user suitably acknowledges Curtis D. Mobley in any presentations, reports,    *
; * publications, or other works that make use of the code or its output.  Depending on the extent  *
; * of use of the code or its outputs, suitable acknowledgement can range from a footnote to offer  *
; * of coauthorship.  Further questions can be directed to curtis.mobley@sequoiasci.com.            *
; ***************************************************************************************************

;-----------------------------------------------------------------------------------------------------------
;| This routine uses Coyote Graphics routines rather than standard IDL for plotting; see www.idlcoyote.com |
;| and "Coyote's Guide to Traditional IDL Graphics: Using Familiar Tools Creatively" by David Fanning      |
;| Any comments below like "CG page xxx" refer to this book.                                               |
;| CG routines are wrappers for standard IDL routines, which make it easier to do certain things like      |
;| create vector eps output files; cgPlot replaces the standard Plot, etc.                                 |
;-----------------------------------------------------------------------------------------------------------

; Written 09 December 2013 by Curtis Mobley

;---------------  BEGIN USER INPUT  ----------------

; DEFINE THE PHYSICAL REGION AND SAMPLING

idebug = 1 ; = 1 for full output
isavesurf = 0 ; 1 to save the z(x,y) surface, 0 to not save
idrawgrid = 1 ; to plot the physical and frequency grid points
gridcolor = 'medium gray' ;'light gray' ; used only if idrawgrid = 1

seed = 4838832L ; seed for random number generation

; WARNING: don't use Nx and Ny greater than 16 or the printout will be unreadably large and messy
Nx = 8 ; number of samples of sea surface elevation to be generated in the x direction; MUST be a power of 2 for the FFT
Ny = 8 ; number of samples of sea surface elevation to be generated in the y direction; MUST be a power of 2 for the FFT

InputOption = 2

CASE InputOption OF
  1: BEGIN
        labelInOpt = ' Spatial resolution is defined by the sample spacing and number of sample points'
        ; smallest resolvable wavelength is 2*Deltax
        Deltax = 0.1; 0.0025
        Deltay = 0.1
        Lx = Deltax*float(Nx)
        Ly = Deltax*float(Ny)
     END 
  2: BEGIN
        labelInOpt = ' Spatial resolution is defined by the spatial region and number of sample points'
        ; largest resolvable wave is the length of the spatial region
        Lx = 10.0 ; length of surface region in the x direction, in meters 
        Ly = 10.0 ; length of surface region in the y direction, in meters 
        Deltax = Lx/float(Nx)
        Deltay = Ly/float(Ny)
     END    
ENDCASE

; pick a root name for the output plot files
; an EPS file named PlotFileRootName.eps will be created
; a  PNG file named PlotFileRootName.png will also be created

PlotFileRootName ='FFT2D4_Norm01_z'

; set order of y (row) array printout for comparison with either IDL plots or Fortran output:

; for easy comparison with IDL plots, which have (0,0) at the lower left, use "for iy=Ny-1,0,-1 do begin" order
Ny1 = Ny-1
Ny2 = 0
Nystep = -1
printorder = " Array printout y order is in IDL plot order: Array(0,0) at lower left"

; for easy comparison with F95 array printouts, which have (0,0) at the upper left, use "for iy=0,Ny-1,0 do begin" order
;Ny1 = 0
;Ny2 = Ny-1
;Nystep = 1
;printorder = " Array printout y order is in Fortran order: Array(0,0) at upper left"

; Also see surfoption below for the option on how the surface is created.

;---------------  END USER INPUT  ----------------


; SET UP OUTPUT FOR A MULTIPANEL LAYOUT

nplotcol = 2 ; number of plot columns
nplotrow = 2 ; number of plot rows

!p.multi = [0,nplotcol,nplotrow] 

; set up output for a vector eps file

PlotFileName = PlotFileRootName + '.eps'

!x.margin = [5,6]
!y.margin = [4,3]
!x.Omargin = [2,2]
!y.Omargin = [0,3]

aspectratio = 1.0
xsize= 6.5
ysize=xsize*aspectratio
charsize = 1.0
!p.charsize = charsize

PS_Start,filename=PlotFileName, /encapsulated, font=1, tt_font='Helvetica', $
         charsize=charsize, default_thickness=2, /nomatch, xsize=xsize, ysize=ysize
 
; pick color tables below, since may want different colors or ranges for different contour plots 
nbarlabs = 10 ; number of value labels on the color bars = nbarlabs + 1

;------------------------

; COMPUTE FREQUENCIES

kxmin = 2.0*!pi/Lx ; min wavenumber (min spatial frequency); wavenumber of longest resolvable wave
kymin = 2.0*!pi/Ly
minxwave = 2.0*Deltax ; min resolvable wavelength (2 point wave)
minywave = 2.0*Deltay 
Nyquistx = !pi/Deltax ; Nyquist spatial freq in rad/m
Nyquisty = !pi/Deltay 
kxmax = Nyquistx
kymax = Nyquisty

print,' '
print,' Output from cgFFT2D_learning.pro'
print,' '
print,' Generation of a 2D FFT amplitudes and variance spectrum from a 2D sea surface z(x,y)'
print,' '
print, format='(4x,a,2i5)', 'Numbers of spatial samples (= number or frequencies) = Nx, Ny =',Nx,Ny
print, format='(4x,a,2f7.4)', 'Sample spacings Deltax, Deltay [m] =', Deltax,Deltay
print, format='(4x,a,2f8.1)', 'Max resolvable wavelengths = Lx,Ly [m] =',Lx,Ly
print, format='(4x,a,2f7.4)', 'Min resolvable wavelengths = 2*Deltax,2*Deltay [m] =', minxwave,minywave
print, format='(4x,a,2f7.2)', 'Min resolvable (fundamental) frequencies kx(0) = 2 pi/(2 Deltax), ky [rad/m] = ',kxmin ,kymin
print, format='(4x,a,2f7.2)', 'Max resolvable (Nyquist) frequencies kx(Nx/2) = pi/Deltax, ky [rad/m] = ',kxmax ,kymax
print,' '


; NOTE ON FREQUENCIES:
;   Frequencies can be confusing.
;   "positive" frequencies (N/2 in number) are used for ploting 1-sided variance spectra
;   "math" frequencies (N in number) are used for plotting 2-sided spectra
;   "FFT" frequencies (N in number) are used in computing FFTs
; 
; Compute the positive frequencies 
; These do not include the 0 frequency because the DC signal is usually 0
; Use even spacing in frequency k
;   The first frequency is the fundamental freq (freq of the longest resolvable wave)
;   The last frequency is the Nyquist frequency (freq of the shortest resolvable wave)
Nkxpos = Nx/2
Nkypos = Ny/2
dkx = (kxmax - kxmin)/float(Nkxpos-1) ; spacing for positive wavenumbers; dk includes the Deltax value
dky = (kymax - kymin)/float(Nkypos-1)
kxpos = kxmin + dkx*findgen(Nkxpos)
kypos = kymin + dky*findgen(Nkypos)

; The IDL FFT returns frequencies in the FFT order (see the IDL documentation)
; kFFT = [0.0(DC), 1, 2, ..., N/2-1, N/2 (Nyquist freq), -(N/2-1), ..., -1]*(2 pi/L) 
; for a total of N freqs (N = Nx or Ny)
; The 0 frequency is IDL array index 0; Nyquist is index N/2
; This FFT order is generated as follows:
xFFT = FINDGEN(Nx/2-1) + 1.0 ; = [1,2,...,Nx/2]
yFFT = FINDGEN(Ny/2-1) + 1.0
kxFFT = (2.0*!pi/Lx)*[0.0, xFFT, Nx/2.0, -Nx/2.0+xFFT] ; in rad/m
kyFFT = (2.0*!pi/Ly)*[0.0, yFFT, Ny/2.0, -Ny/2.0+yFFT]

; for plotting 2-sided spectra, it is common to use the math frequency order of 
; kmath = [-(N/2-1), ..., -1,0,1,2,...,N/2-1,N/2]*2pi/L
; which runs negative to positive and has 
; the 0 freq in the middle (IDL index N/2) and Nyquist at the right (index N-1)
; This can be obtained from the IDL order by a circular shift to the right by N/2-1 spaces:
kxmath = SHIFT(kxFFT,Nx/2-1)
kymath = SHIFT(kyFFT,Ny/2-1)
; or by
;kxmath = [-reverse(kxpos(0:Nx/2-2)),0,kxpos]
;kymath = [-reverse(kypos(0:Ny/2-2)),0,kypos]

; NOTE ON FREQUENCY SHIFTS:
;    In IDL, a POSITIVE shift moves array elements to the RIGHT (a negative shift moves elements to the left), so 
;    kmath = SHIFT(kFFT, N/2-1)
;    kFFT = SHIFT(kmath, -(N/2-1))
;    In F95, a POSITIVE shift moves array elements to the LEFT (a positive shift moves elements to the right), so
;    kmath = CSHIFT(kFFT, -(N/2-1))
;    kFFT = CSHIFT(kmath, N/2-1)



;********** CREATE A SEA SURFACE z(x,y)

; define the (x,y) locations in meters where wave surface is sampled, 

; The Lx by Ly region is layed out as 0 <= x < Lx and 0 <= y < Ly
x = deltax*findgen(Nx) 
y = deltay*findgen(Ny)

; Can also use -Lx/2 <= x < Lx/2 and -Ly/2 <= y < Ly/2):
; x = -Lx/2.0 + deltax*findgen(Nx)
; y = -Ly/2.0 + deltay*findgen(Ny)

z = fltarr(Nx,Ny)

; print the spatial and frequency coordinates
if idebug eq 1 then begin
   print,format='(4x,a15,16i8)',  'array index j =',indgen(Nx)
   print,format='(4x,a17,16f8.3)','x [m] =          ',x
   print,format='(4x,a17,16f8.3)','y [m] =          ',y
   print,format='(4x,a17,16f8.3)','kxpos [rad/m] =  ',kxpos
   print,format='(4x,a17,16f8.3)','kypos [rad/m] =  ',kypos
   print,format='(4x,a17,16f8.3)','kxmath [rad/m] = ',kxmath
   print,format='(4x,a17,16f8.3)','kymath [rad/m] = ',kymath
   print,format='(4x,a17,16f8.3)','kxFFT [rad/m] =  ',kxFFT
   print,format='(4x,a17,16f8.3)','kyFFT [rad/m] =  ',kyFFT
   print,' '
   print,format='(4x,a)', "kmath(j) = -kmath(N-2-j), j=0,...,N-2; k=0 is index N/2-1; Nyquist is index N-1"
   print,format='(4x,a)', "kFFT(j) = -kFFT(N-j), j=1,...,N/2-1, N/2+1,...,N-1; k=0 is index 0; Nyquist is index N/2"
   print,' '
endif

surfoption = 1

CASE surfoption OF

1: BEGIN
;------Option 1: Create a sea surface by drawing random numbers

      z = RANDOMU(seed,Nx,Ny,/normal)
      print,' '
      print,' The sea surface elevations are N(0,1) random numbers'
      print,' ' 
   END
   
2: BEGIN   
;------Option 2: Create a 2D sinusoidal wave field

      Nwavex1 = 2 ; # waves in the x direction in [0,Lx]
      kx1 = 2.0*!pi*Nwavex1/Lx
      
      Nwavey1 = 1 ; # waves in the y direction in [0,Ly]
      ky1 = 2.0*!pi*Nwavey1/Ly
      ; compute the direction of the wave train
      phi1 = !radeg*atan(ky1,kx1)
      Amp1 = 1.0
      
      ; alternatively, specify the direction of the wave train
     ; phi = 30.0
     ; ky = kx*tan(phi/!radeg)
      
      phase1 = 0.0 ; = 0 for cosine waves; !pi/2 for sines; !pi/4 to split equally
      
      ; for a second wave pattern:
      Nwavex2 = 4 ; # waves in the x direction in [0,Lx]
      kx2 = 2.0*!pi*Nwavex2/Lx
      
      Nwavey2 = 3 ; # waves in the y direction in [0,Ly]
      ky2 = 2.0*!pi*Nwavey2/Ly
      ; compute the direction of the wave train
      phi2 = !radeg*atan(ky2,kx2)
      phase2 = !pi/2
      Amp2 = 0.5
      
      for ix = 0,Nx-1 do begin
        for iy = 0,Ny-1 do begin
           z[ix,iy] = Amp1*cos(kx1*x[ix] + ky1*y[iy] + phase1) + Amp2*cos(kx2*x[ix] + ky2*y[iy] + phase2)
        endfor
      endfor  
      
      print,' '
      print,format='(a)', ' The sea surface is a 2D sinusoid defined by'
      print,format='("   Nwavex1, Nwavey1, kx1, ky1, phase1, phi1, Amp1 =",2i5,5f10.3)', Nwavex1, Nwavey1, kx1, ky1, phase1, phi1, Amp1
      print,format='("   Nwavex2, Nwavey2, kx2, ky2, phase2, phi2, Amp2 =",2i5,5f10.3)', Nwavex2, Nwavey2, kx2, ky2, phase2, phi2, Amp2
      print,' '

   END
   
3: BEGIN   

;------Option 3: create a sea surface by adding up some random sinusoids

      ; These functions are only for Nx, Ny = 16 or so.
      ; define N/2+1 amplitudes of wave components:
      ;Amp(0) is the DC signal, Amp(1) is longest wave, Amp(N/2) is 2 point wave
      Ampx = fltarr(Nx/2 + 1)
      Ampy = fltarr(Ny/2 + 1)
      
      ; These are ad hoc (and unphysical) amplitude functions for testing:
      ; Amp[j] is the amplitude of the jth frequency component
      Ampx[0] = 0.0 ; 0 for MSL = 0
      for j=1,Nx/2 do Ampx[j] = 0.15*exp(-3.0*j/Nx)
      Ampy[0] = 0.0
      for j=1,Ny/2 do Ampy[j] = 0.15*exp(-4.0*j/Ny)
      print,' '
      print,' The sea surface elevations are sinusoids with ad hoc amplitudes'
      print,' '
      
      ; examples for nonrandom sinusoids
      ; for an x sinusoid at only the fundamental freq:
      ;Ampx = REPLICATE(0.0,Nx/2+1)
      ;Ampx[1] = 0.15
      ; for an x sinusoid at only the Nyquist freq:
      ;Ampx[Nx/2] = 0.15
      ;Ampy = REPLICATE(0.0,Ny/2+1)
      ; for an x sinusoid at only the Nyquist freq and y at the fund freq:
      ;Ampx[Nx/2] = 0.15
      ;Ampy = REPLICATE(0.0,Ny/2+1)
      ;Ampy[1] = 0.15
      
       
      ; sum the individual random-phase waves for each frequency in each direction 
      ; at each (x,y) point to get the sea surface
      ; kpos(0) is the fundamental freq corresponding to Amp(1), so offset k indices
      
      for ix = 0,Nx-1 do begin
        for iy = 0,Ny-1 do begin
          for jx=1,Nx/2 do begin
            for jy=1,Ny/2 do begin
               ; for adding random waves use:
      ;        z[ix,iy] = z[ix,iy] $
      ;                 + Ampx[jx]*cos(kxpos[jx-1]*x[ix] + 2.0*!pi*RANDOMU(seed)) $ ; random wave in x direction
      ;                 + Ampy[jy]*cos(kypos[jy-1]*y[iy] + 2.0*!pi*RANDOMU(seed))   ; random wave in y direction
               ; for sinusoids without random phases use:
              z[ix,iy] = z[ix,iy] $
                       + Ampx[jx]*cos(kxpos[jx-1]*x[ix]) $ ; + 2.0*!pi*RANDOMU(seed)) $ ; random wave in x direction
                       + Ampy[jy]*cos(kypos[jy-1]*y[iy]) ; + 2.0*!pi*RANDOMU(seed))   ; random wave in y direction
            endfor
          endfor
        endfor
      endfor

   END
ENDCASE 
  
; compute the mean sea level (MSL) and total variance (z^2) from wave elevations

MSL = TOTAL(z)/float(Nx*Ny)
sumz2 = TOTAL(z*z)
print,' MSL as generated, sumz2 = ',TOTAL(z),TOTAL(z*z)

; A single random surface realization will not in general have a zero mean; reset the surface to MSL = 0.0

z = z - MSL

MSL = TOTAL(z)/float(Nx*Ny)
sumz2 = TOTAL(z*z)
print,' MSL reset to 0, sumz2 = ',TOTAL(z),TOTAL(z*z)

minz = min(z,max=maxz)
print,' min, max z(x,y) = ',minz,maxz

print,' '
print,printorder

; print z(x,y) for debugging of contour plot
print,' '
print,' surface elevations z(x,y)'
print,format='(a8,12x,16i7)',' ix =',indgen(Nx)
print,format='(a8,12x,16f7.3)','  x =',x
for iy=Ny1,Ny2,Nystep do begin
    print,format='(a8,i5,17f7.3)',' iy, y =',iy,y(iy),z(*,iy)
endfor

; optionally save the surface

IF isavesurf eq 1 then begin
openw,nunit,'z2D_' + PlotFileRootName + '.txt',/get_lun
printf,nunit,format='("   ix   iy      x(ix)      y(iy)    z(ix,iy)")'
printf,nunit,format='(2i5,a)',Nx,Ny,' = Nx, Ny'
for ix=0,Nx-1 do begin
  for iy=0,Ny-1 do begin
     printf,nunit,format='(2i5,3f10.3)',ix,iy,x(ix),y(iy),z(ix,iy)
  endfor
endfor     
free_lun,nunit
ENDIF



;********** PLOT z(x,y) IN THE UPPER LEFT PANEL

; set contour range to nice values and define contour levels

minz = -2.0 ;-1.0 ; can reset for nice appearance
maxz = 2.0 ;1.0
Nlevels = 100 ; = num of contour lines including minz and maxz
zlevels = minz + (maxz - minz)/float(Nlevels-1)*findgen(Nlevels)  
;print,'contour levels for z = ',zlevels 

;zlevels = cgConLevels(MINVALUE=minz,MAXVALUE=maxz,NLEVELS=11) ;, Factor=1)

; The surface is perodic in x and y. Use this to define 
; extra x,y points at (Lx,Ly) at indices IDL (Nx,Ny) for nice contouring.

xcont = [x,Lx]
ycont = [y,Ly]
zcont = fltarr(Nx+1,Ny+1)
zcont[0:Nx-1,0:Ny-1] = z
zcont[Nx,0:Ny-1] = z[0,*]
zcont[0:Nx-1,Ny] = z[*,0]
zcont[Nx,Ny] = z[0,0]   

;print,' '
;print,' zcont(x,y)'
;print,format='(a8,12x,17i7)',' ix =',indgen(Nx+1)
;print,format='(a8,12x,17f7.3)','  x =',cont
;for iy=Ny,0,-1 do begin
;    print,format='(a8,i5,18f7.3)',' iy, y =',iy,ycont(iy),zcont(*,iy)
;endfor

; load a color table for this contour plot
firstcolor = 1 ; index of first color in the color table to use for the min contour
ncolors = Nlevels-1 ; number of colors to use in contouring

cgLoadCT, 33, NColors=ncolors, Bottom=firstcolor ; color table 33 is blue to red
 
cgContour,zcont,xcont,ycont, title='surface elevation [m]', $
  xrange=[0,Lx],/xstyle,xtitle='x [m]', $
  yrange=[0,Ly],/ystyle,ytitle='y [m]', $
  levels=zlevels,fill=1,C_Colors=Indgen(ncolors-1)+1

; add a color bar at the right
barpos = [0.47, 0.55, 0.49, 0.87] ; color bar position, normal coords [xll,yll,xur,yur]

cgColorbar, NColors=ncolors, Bottom=firstcolor, /vertical, position=barpos, divisions=nbarlabs,$ ;/Fit,/Discrete, 
           Range=[Min(zlevels), Max(zlevels)], OOB_High='white', OOB_Low='black'  


; draw the (x,y) grid points

if idrawgrid ne 0 then begin
   for iy=0,Ny-1 do cgOplot,xcont(0:Nx),REPLICATE(ycont(iy),Nx),psym=plotsym(/circle,/fill,scale=1),color=gridcolor,/noclip
   ; values at Lx and Ly from symmetry
   for iy=0,Ny do cgOplot,xcont(Nx),ycont(iy),psym=plotsym(/circle,thick=4,scale=1),color=gridcolor,/noclip
   for ix=0,Nx do cgOplot,xcont(ix),ycont(Ny),psym=plotsym(/circle,thick=4,scale=1),color=gridcolor,/noclip
endif

 
; ******** TAKE THE 2D forward FFT OF z(x,y) TO GET AMPLITUDE zhat

; make z a complex number for the IDL FFT routine
zcomplx = COMPLEX(z)

; take the 2D forward FFT

zhat = FFT(zcomplx) ; zhat is (Nx,Ny) and complex

Zreal = REAL_PART(zhat)
Zimag = IMAGINARY(zhat)

; optionally save the amplitudes
;openw,nunit,'zrealIDL.txt',/get_lun
;  printf,nunit,format='(a)'," zreal in FFT order from IDL"
;  printf,nunit,format='(a6,6x,16i7)',' ikx =',indgen(Nx)
;  printf,nunit,format='(a6,6x,16f7.3)','  kx =',kxFFT
;  for iky=Ny-1,0,-1 do printf,nunit,format='(i6,f6.1,16e14.5)',iky,kyFFT(iky),zreal(*,iky)
;free_lun,nunit
;
;openw,nunit,'zimagIDL.txt',/get_lun
;  printf,nunit,format='(a)'," zimag in FFT order from IDL"
;  printf,nunit,format='(a6,6x,16i7)',' ikx =',indgen(Nx)
;  printf,nunit,format='(a6,6x,16f7.3)','  kx =',kxFFT
;  for iky=Ny-1,0,-1 do printf,nunit,format='(i6,f6.1,16e14.5)',iky,kyFFT(iky),zimag(*,iky)
;free_lun,nunit

print,' '
print,' Real{zhat} from IDL FFT; kx, ky in FFT order'
print,format='(a8,12x,16i7)',' ikx =',indgen(Nx)
print,format='(a8,12x,16f7.3)',' kx =',kxFFT
for iky=Ny1,Ny2,Nystep do begin
    print,format='(a8,i5,17f7.3)',' iky, ky =',iky,kyFFT(iky),zreal(*,iky)
endfor

print,' '
print,'  Imag{zhat} from IDL FFT; kx, ky in FFT order'
print,format='(a8,12x,16i7)',' ikx =',indgen(Nx)
print,format='(a8,12x,16f7.3)',' kx =',kxFFT
for iky=Ny1,Ny2,Nystep do begin
    print,format='(a8,i5,17f7.3)',' iky, ky =',iky,kyFFT(iky),zimag(*,iky)
endfor

; See the Note on Frequencies above!

; The frequencies returned by the IDL forward FFT are in FFT order; kxFFT and kyFFT 
 
; Must reorder the IDL FFT frequency order and the amplitudes, before contouring in
; the math freq order 

; Shift the elements of the amplitude array so that
;    the smallest neg freq (N/2-1) is at the "left/bottom" of the plot
;    the 0 freq is at the center 
;    the largest positive (Nyquist) freq  is at the "right/top" of the plot
; This is the preferred frequency order for contouring
; Note: the IDL /center option in FFT shifts so that the Nyquist freq is at the left/bottom

zreal = SHIFT(zreal,Nx/2-1,Ny/2-1) ; shift from FFT to math order in both indices
zimag = SHIFT(zimag,Nx/2-1,Ny/2-1)
zRealplot = zreal ; save arrays in math freq order for plotting below
zImagplot = zimag

; optionally save the real and imag amplitudes in math order
;openw,nunit,'zrealmath.txt',/get_lun
;  printf,nunit,format='(a)'," zreal in math freq order"
;  printf,nunit,format='(a6,6x,16i7)',' ikx =',indgen(Nx)
;  printf,nunit,format='(a6,6x,16f7.3)','  kx =',kxFFT
;  for iky=Ny-1,0,-1 do printf,nunit,format='(i6,f6.1,16e14.5)',iky,kyFFT(iky),zreal(*,iky)
;free_lun,nunit
;
;openw,nunit,'zimagmath.txt',/get_lun
;  printf,nunit,format='(a)'," zimag in math freq order"
;  printf,nunit,format='(a6,6x,16i7)',' ikx =',indgen(Nx)
;  printf,nunit,format='(a6,6x,16f7.3)','  kx =',kxFFT
;  for iky=Ny-1,0,-1 do printf,nunit,format='(i6,f6.1,16e14.5)',iky,kyFFT(iky),zimag(*,iky)
;free_lun,nunit

print,' '
print,' Shifted Real{zhat} from FFT; kx, ky in math order'
print,format='(a8,12x,16i7)',' ikx =',indgen(Nx)
print,format='(a8,12x,16f7.3)',' kx =',kxmath
for iky=Ny1,Ny2,Nystep do begin
    print,format='(a8,i5,17f7.3)',' iky, ky =',iky,kymath(iky),zreal(*,iky)
endfor

print,' '
print,' Shifted Imag{zhat} from FFT; kx, ky in math order'
print,format='(a8,12x,16i7)',' ikx =',indgen(Nx)
print,format='(a8,12x,16f7.3)',' kx =',kxmath
for iky=Ny1,Ny2,Nystep do begin
    print,format='(a8,i5,17f7.3)',' iky, ky =',iky,kymath(iky),zimag(*,iky)
endfor

; shift back to FFT order as a check
zreal = SHIFT( zreal, -(Nx/2-1), -(Ny/2-1) )
zimag = SHIFT( zimag, -(Nx/2-1), -(Ny/2-1) )

print,' '
print,' Real{zhat} shifted  back to FFT order'
print,format='(a8,12x,16i7)',' ikx =',indgen(Nx)
print,format='(a8,12x,16f7.3)',' kx =',kxFFT
for iky=Ny1,Ny2,Nystep do begin
    print,format='(a8,i5,17f7.3)',' iky, ky =',iky,kyFFT(iky),zreal(*,iky)
endfor

print,' '
print,' Imag{zhat} shifted  back to FFT order'
print,format='(a8,12x,16i7)',' ikx =',indgen(Nx)
print,format='(a8,12x,16f7.3)',' kx =',kxFFT
for iky=Ny1,Ny2,Nystep do begin
    print,format='(a8,i5,17f7.3)',' iky, ky =',iky,kyFFT(iky),zimag(*,iky)
endfor

;print,' '
;print,' Centered Zreal from FFT(z,/center); kx, ky in math order; Nyquist at left/bottom'
;print,format='(a8,12x,16i7)',' ikx =',indgen(Nx)
;print,format='(a8,12x,16f7.3)',' kx =',kxmath
;for iky=Ny-1,0,-1 do begin
;    print,format='(a8,i5,17f7.3)',' iky, ky =',iky,kymath(iky),zreal(*,iky)
;endfor

; compute total variance from the Fourier amplitudes zhat as the sum of all Nx,Ny amplitudes squared
sumzhat2 = 0.0
for ikx=0,Nx-1 do begin
  for iky=0,Ny-1 do begin
    sumzhat2 = sumzhat2 + abs(zhat(ikx,iky))^2
  endfor  
endfor

print,' '
print,'amplitudes: sum Nx*Ny*zhat^2 =',Nx*Ny*sumzhat2
print,' '
print,'variance from amplitudes: sum Nx*Ny*zhat*CONJ(zhat) =',Nx*Ny*TOTAL(zhat*CONJ(zhat))


;****** PLOT REAL PART OF FFT(kx,ky) IN UPPER RIGHT PANEL

maxReZ = max(Zreal,min=minReZ)
print,'min Zreal, max Zreal = ',minReZ ,maxReZ

; set contour range to nice values and define contour levels

minplot = -0.5 ;-0.5
maxplot = 0.5; 0.5
Nlevels = 100 ; = num of contour lines including minz and maxz
ReZlevels = minplot + (maxplot - minplot)/float(Nlevels-1)*findgen(Nlevels)  
;print,'contour levels for Real{z} = ',Rezlevels 

; load a color table for this contour plot
firstcolor = 1 ; index of first color in the color table to use for the min contour
ncolors = Nlevels-1 ; number of colors to use in contouring

;LOADCT,39 
cgLoadCT, 33, NColors=Ncolors, Bottom=firstcolor
;kolzero = cgColor("wheat",50) ; color for values of 0.0

; set zero values to a special color

;ReZlevels = cgConLevels(MINVALUE=minReZ,MAXVALUE=maxReZ) ;, Factor=10) ;, MINVALUE=minReZ)

;print,'zRealPlot(3,3) = ',zrealplot(3,3)

cgContour,zrealplot,kxmath,kymath, title='Real{zhat}', $ ;[m!u2!n/(rad/m)!u2!n]', $
  xrange=[-kxmax,kxmax],/xstyle,xtitle='k!dx!n [rad/m]', $
  yrange=[-kymax,kymax],/ystyle,ytitle='k!dy!n [rad/m]', $
  levels=ReZlevels,fill=1,C_Colors=Indgen(ncolors-1)+1

barpos = [0.95, 0.55, 0.97, 0.87] ; color bar position, normal coords [xll,yll,xur,yur]
cgColorbar, NColors=ncolors, Bottom=1, /vertical, position=barpos,divisions=nbarlabs, $ ;/Fit,/Discrete, 
           Range=[minplot,maxplot], OOB_High='white', OOB_Low='black'  

; optionally draw the (kxmath,kymath) grid points

if idrawgrid ne 0 then begin
   for iy=0,Ny-1 do cgOplot,kxmath(0:Nx-1),REPLICATE(kymath(iy),Nx),psym=plotsym(/circle,/fill,scale=1),color=gridcolor,/noclip
   ; values at Lx and Ly from symmetry
   ;for iy=0,Ny do cgOplot,kxmath(Nx-1),kxmath(iy),psym=plotsym(/circle,thick=3,scale=1),color='light gray'
  ; for ix=0,Nx do cgOplot,kxmath(ix),kxmath(Ny-1),psym=plotsym(/circle,thick=3,scale=1),color='light gray'
endif

;iax = 3
;iay = 2 
;cgOplot,0.0,0.0,psym=4
;cgArrow,0.0,0.0,kxmath(iax),kymath(iay),/data,/solid,color='black'
;cgArrow,0.0,0.0,kxmath(Nx-1-iax),kymath(Ny-1-iay),/data,/solid,color='black'

;********  PLOT IMAGINARY PART OF FFT(kx,ky) IN LOWER LEFT PANEL


maxImZ = max(zimagplot,min=minImZ)
print,'min Zimag, max Zimag = ',minImZ ,maxImZ

; set contour range to nice values and define contour levels

;minplot = -0.3
;maxplot = 0.3
Nlevels = 100; 9 ; = num of contour lines including minz and maxz
ReZlevels = minplot + (maxplot - minplot)/float(Nlevels-1)*findgen(Nlevels)  
;print,'contour levels for Imag{z} = ',Rezlevels 

; load a color table for this contour plot
firstcolor = 1 ; index of first color in the color table to use for the min contour
ncolors = Nlevels-1 ; number of colors to use in contouring

;LOADCT,39 
cgLoadCT, 33, NColors=Ncolors, Bottom=firstcolor
;kolzero = cgColor("wheat",50) ; color for values of 0.0

;ReZlevels = cgConLevels(MINVALUE=minReZ,MAXVALUE=maxReZ) ;, Factor=10) ;, MINVALUE=minReZ)
 
cgContour,zimagplot,kxmath,kymath, title='Imag{zhat}', $ ;[m] or [m!u2!n/(rad/m)!u2!n]??', $
  xrange=[-kxmax,kxmax],/xstyle,xtitle='k!dx!n [rad/m]', $
  yrange=[-kymax,kymax],/ystyle,ytitle='k!dy!n [rad/m]', $
  levels=ReZlevels,fill=1,C_Colors=Indgen(ncolors-1)+1


barpos = [0.47, 0.085, 0.49, 0.405] ; color bar position, normal coords [xll,yll,xur,yur]
cgColorbar, NColors=ncolors, Bottom=1,  /vertical, position=barpos, divisions=nbarlabs,$ ;/Fit,/Discrete,
           Range=[minplot,maxplot], OOB_High='white', OOB_Low='black'  
; optionally draw the (kxmath,kymath) grid points

if idrawgrid ne 0 then begin
   for iy=0,Ny-1 do cgOplot,kxmath(0:Nx-1),REPLICATE(kymath(iy),Nx),psym=plotsym(/circle,/fill,scale=1),color=gridcolor,/noclip
   ; values at Lx and Ly from symmetry
   ;for iy=0,Ny do cgOplot,kxmath(Nx-1),kxmath(iy),psym=plotsym(/circle,thick=3,scale=1),color='light gray'
  ; for ix=0,Nx do cgOplot,kxmath(ix),kxmath(Ny-1),psym=plotsym(/circle,thick=3,scale=1),color='light gray'
endif


;******** COMPUTE THE VARIANCE SPECTRUM

; use the unshifted zhat and the first N/2 (unshifted) amplitudes 
; to compute the 1-sided 2D variance spectrum for positive frequencies

; Psi1s is the one-sided 2D variance spectrum; 0 and pos x freqs only, but neg and pos y freqs.
; This corresponds to pos k = sqrt(kx^2 + ky^2) and -90 <= phi= atan(ky/kx) <= 90

Psi1s = fltarr(Nx/2+1,Ny) 
; Psi1s(0,0) freq has 0 variance for MSL = 0
; Psi1s(1,1) is variance(fundamental x freq, fundamental y freq)
; Psi1s(Nx/2,Ny/2) is (Nyquistx, Nyquisty)
; zhat is the unshifted FFT, so Psi1s freqs are in FFT order with 0,1,...,Nyquist at N/2-1, then neg freqs

for ikx=0,Nx/2 do begin
  for iky=0,Ny-1 do begin
    Psi1s(ikx,iky) = abs(zhat[ikx,iky])^2
  endfor
endfor

; Psi2s is the two-sided 2D variance spectrum
; freqs are in FFT order since zhat is still in FFT order
Psi2s = fltarr(Nx,Ny) 
 for ikx=0,Nx-1 do begin
  for iky=0,Ny-1 do begin
    ;Psi2s(ikx,iky) = abs(zhat[ikx,iky])^2
    Psi2s(ikx,iky) = REAL_PART( zhat[ikx,iky] * CONJ(zhat[ikx,iky]) )
  endfor
endfor

 
;print,' '
;print,' Psi1s from FFT; positive kx, ky in math order'
;print,format='(a8,12x,9i9)',' ikx =',indgen(Nx/2+1)
;print,format='(a8,14x,9f9.5)',' kx =',kxFFT
;for iky=Ny1,Ny2,Nystep do begin
;    print,format='(a8,i5,17f9.5)',' iky, ky =',iky,kyFFT(iky),Nx*Ny*variance1S(*,iky) ; fix this to pos math order
;endfor
 
print,' '
print,' Psi2s from FFT; kx, ky in FFT order'
print,format='(a8,12x,16i9)',' ikx =',indgen(Nx)
print,format='(a8,14x,16f9.5)',' kx =',kxFFT
for iky=Ny1,Ny2,Nystep do begin
    print,format='(a8,i5,17f9.5)',' iky, ky =',iky,kyFFT(iky),Nx*Ny*Psi2s(*,iky)
endfor
 
; compute total variance from the one-sided (1S) variance spectrum
; add terms that have only 0 or pos freqs once, and double terms than have both pos and neg freqs
sumPsi1s = Psi1s(0,0)
for iky=0,Ny-1 do sumPsi1S = sumPsi1s + Psi1s(0,iky) + Psi1s(Nx/2,iky) ; zero and Nyquist in x for all y; occur once
for ikx=1,Nx/2-1 do begin
  for iky=0,Ny-1 do begin
    sumPsi1s = sumPsi1s + 2.0*Psi1s(ikx,iky) ; all other freqs occur twice as pos and neg freqs
  endfor
endfor

; compute total variance from the two-sided (2S) variance spectrum: 
; sum over all pos and neg frequencies (still in unshifted order, but not relevant)
sumPsi2s = TOTAL(Psi2s)

print,' '
print,' Total variance from 1- and 2-sided spectra: Nx*Ny*sumPsi1s, Nx*Ny*sumPsi2s = ',Nx*Ny*sumPsi1s,Nx*Ny*sumPsi2s

print,' '
print,' Triple (Parseval) check on total variance:'
print,'    sum z^2 =',sumz2,' = Nx*Ny*sum zhat^2 = ',Nx*Ny*sumzhat2,' = Nx*Ny*sum Psi2s = ',Nx*Ny*sumPsi2s
print,' '



;******** CONTOUR THE VARIANCE SPECTRUM
;
;;**** OPTION 1: CONTOUR THE FULL two-sided Psi2s(kx,ky)

maxPsi = max(Nx*Ny*Psi2s,min=minPsi)
print,'minPsi ,maxPsi = ',minPsi ,maxPsi

; shift variance to math freq order for plotting, and normalize to the max value
Enorm = Nx*Ny*SHIFT(Psi2s,Nx/2-1,Ny/2-1)/maxPsi 
; set contour range to nice values and define contour levels
minPsi = 0.0
maxPsi = 1.0
Nlevels = 100 ; = num of contour lines including minz and maxz
Elevels = minPsi + (maxPsi - minPsi)/float(Nlevels-1)*findgen(Nlevels)  
;print,'contour levels for variance = ',Elevels 

; load a color table for this contour plot
firstcolor = 1 ; index of first color in the color table to use for the min contour
ncolors = Nlevels-1 ; number of colors to use in contouring

;LOADCT,39 
cgLoadCT, 33, NColors=Ncolors, Bottom=firstcolor
;kolzero = cgColor("wheat",firstcolor) ; color for values of 0.0

; shift energy2 from FFT to math order for plotting
Psi2s = SHIFT(Psi2s,(Nx/2-1),(Ny/2-1))

print,' '
print,' Psi2s from FFT; kx, ky in math order; y in plot order'
print,format='(a8,12x,16i9)',' ikx =',indgen(Nx)
print,format='(a8,14x,16f9.5)',' kx =',kxmath
for iky=Ny-1,0,-1 do begin
   print,format='(a8,i5,17f9.5)',' iky, ky =',iky,kymath(iky),Nx*Ny*Psi2s(*,iky)
endfor

cgContour,Enorm,kxmath,kymath, title=' variance/max(variance)', $ ;'variance = N!dx!n N!dy!n |zhat|!u2!n [m!u2!n]', $; 'N!dx!n N!dy!n |zhat|!u2!n [m!u2!n]', $
  xrange=[-kxmax,kxmax],xtitle='k!dx!n [rad/m]', $
  yrange=[-kxmax,kxmax],ytitle='k!dy!n [rad/m]', $
  levels=Elevels,fill=1,C_Colors=Indgen(ncolors-1)+1

barpos = [0.95, 0.085, 0.97, 0.405] ; color bar position, normal coords [xll,yll,xur,yur]
cgColorbar, NColors=ncolors, Bottom=firstcolor,  /vertical, position=barpos, divisions=nbarlabs, $ ;/Fit,/Discrete,
           Range=[minPsi,maxPsi], OOB_High='white', OOB_Low='black',format='(f5.1)'

; optionally draw the (kxmath,kymath) grid points

if idrawgrid ne 0 then begin
   for iy=0,Ny-1 do cgOplot,kxmath(0:Nx-1),REPLICATE(kymath(iy),Nx),psym=plotsym(/circle,/fill,scale=1),color=gridcolor,/noclip
   ; values at Lx and Ly from symmetry
   ;for iy=0,Ny do cgOplot,kxmath(Nx-1),kxmath(iy),psym=plotsym(/circle,thick=3,scale=1),color='light gray'
  ; for ix=0,Nx do cgOplot,kxmath(ix),kxmath(Ny-1),psym=plotsym(/circle,thick=3,scale=1),color='light gray'
endif

             
;**** OPTION 2: CONTOUR THE ONE-SIDED Psi1s(k,phi)

; NOTE: the Psi1s(k,phi) plots do not contour well for low Nx,Ny because of the irregular grid
; spacing for k and phi

;energy2 = SHIFT(energy2,Nx/2-1,Ny/2-1)
;
;;print,' '
;;print,' Shifted Energy2 from 1000*|zhat|^2; kx, ky in math order (fundamental at middle; Nyquist at top and right)'
;;print,format='(a8,12x,16i9)',' ikx =',indgen(Nx)
;;print,format='(a8,14x,16f9.5)',' kx =',kxmath
;;for iky=Ny-1,0,-1 do begin
;;    print,format='(a8,i5,17f9.5)',' iky, ky =',iky,kymath(iky),1000*energy2(*,iky)
;;endfor
;
;
;; Create 2D k and phi grid for contouring with uneven spacing of (k,phi) points
;; and compute E(k,phi) = E(kx,ky)/k.
;; This indexing assumes that energy2 has been shifted so that (0,0) is at element (Nx/2,Ny/2) and Nyquist is at right/top
;k = fltarr(Nx/2+1,Ny)
;phi = fltarr(Nx/2+1,Ny)
;Ekphi = fltarr(Nx/2+1,Ny)
;for ikx = Nx/2-1,Nx-1 do begin ; ikx and iky are for the shifted E2
;  for iky = 0,Ny-1 do begin
;    ik = ikx - Nx/2 + 1
;    iphi = iky
;    k(ik,iphi) = SQRT( kxmath(ikx)^2 + kymath(iky)^2 )
;    phi(ik,iphi) = ATAN( kymath(iky),kxmath(ikx) )*!radeg
;    Ekphi(ik,iphi) = energy2(ikx,iky)/k(ik,iphi)
;  endfor
;endfor
;Ekphi(0,Ny/2-1) = 0.0
;
;;print,' '
;;print,' k = sqrt(kx^2 + ky^2)'
;;print,format='(a8,12x,16i9)',' ik =',indgen(Nx/2+1)
;;print,format='(a8,14x,16f9.5)',' kx =',kxmath(Nx/2-1:Nx-1)
;;for iky=Ny-1,0,-1 do begin
;;    print,format='(a8,i5,17f9.5)',' iy, ky =',iky,kymath(iky),k(*,iky)
;;endfor  
;; 
;;print,' '
;;print,' phi = atan(ky/kx)'
;;print,format='(a8,12x,16i9)',' ik =',indgen(Nx/2+1)
;;print,format='(a8,14x,16f9.5)',' kx =',kxmath(Nx/2-1:Nx-1)
;;for iky=Ny-1,0,-1 do begin
;;    print,format='(a8,i5,17f9.2)',' iy, ky =',iky,kymath(iky),phi(*,iky)
;;endfor  
;; 
;;print,' '
;;print,' 1000*E(k,phi) = E(kx,kx)/k'
;;print,format='(a8,12x,16i9)',' ik =',indgen(Nx/2+1)
;;print,format='(a8,14x,16f9.5)',' kx =',kxmath(Nx/2-1:Nx-1)
;;for iky=Ny-1,0,-1 do begin
;;    print,format='(a8,i5,17f9.5)',' iy, ky =',iky,kymath(iky),1000*Ekphi(*,iky)
;;endfor 
; 
;Ekphi = 1000.0*Ekphi
;
;maxPsi = max(Ekphi,min=minPsi)
;print,'minPsikphi ,maxPsikphi = ',minPsi ,maxPsi
;
;; set contour range to nice values and define contour levels
;minz = 0.0
;maxz = 3.0
;Nlevels = 10 ; = num of contour lines including minz and maxz
;Elevels = minz + (maxz - minz)/float(Nlevels-1)*findgen(Nlevels)  
;;print,'contour levels for variance = ',Elevels 
;
;; load a color table for this contour plot
;firstcolor = 1 ; index of first color in the color table to use for the min contour
;ncolors = Nlevels-1 ; number of colors to use in contouring
;
;;LOADCT,39 
;cgLoadCT, 33, NColors=Ncolors, Bottom=firstcolor
;
;kmax = max(k)
;cgContour,Ekphi[1:Nx/2,*],k[1:Nx/2,*],phi[1:Nx/2,*],/irregular,/cell_fill, title='1000E(k,phi) [m!u2!n/(rad/m)]', $
;  xrange=[0,kmax],xtitle='k [rad/m]', $
;  yrange=[-90.0,90.0],ytitle='phi [deg]',yticks=6,yminor=3, $
;  levels=Elevels,fill=1,C_Colors=Indgen(ncolors-1)+1
;
;barpos = [0.95, 0.085, 0.97, 0.405] ; color bar position, normal coords [xll,yll,xur,yur]
;cgColorbar, NColors=ncolors, Bottom=1, /Discrete, /vertical, position=barpos, $ ;/Fit,
;           Range=[Min(Elevels), Max(Elevels)], OOB_High='white', OOB_Low='black',format='(f5.3)'

    
ENDPLOT: print,'endplot'

label = 'N!dx!n, N!dy!n =' + string(Nx,format='(i3)') + ',' + string(Ny,format='(i3)')
;  k!dfund!n =' + $
;         string(kfund,format='(f5.2)') + ' m!u-1!n; k!dNy!n =' + string(Nyquistk,format='(f5.2)') + ' m!u-1!n'
cgText,0.5,0.95,label,align=0.5,charsize=1.1*!p.charsize,/normal

PS_End,Resize=50,/png ;,/delete_PS ;close and fix the eps file, create a png version, delete the eps version

print,'image file ', PlotFileName,' closed OK'
;
!p.multi = 0 ; reset to normal layout

print,' '
print,'end FFT2D4'
end


