@ECKV2D_k_phi ; the one-sided, 2D ECKV spectrum with the symmetric ECKV Phi(k,phi) spreading function

pro gen2Dsurf

; This routine shows how to use a one-sided 2D variance spectrum Psi_1s(k,phi) and 2D inverse FFTs 
; to generate a random 2D sea surface.
; (To generate a time series of 2D surfaces for creation of videos, use routine cgAnimate_2D_SeaSurface.pro)

; This routine created Tutorial Fig. 3.3

; The Elfouhaily et al (1997) directional variance spectrum is used (the Psi(k,phi) of their Eq. 67). 
; This is computed by routine ECKV2D_k_phi.

; This version uses angular frequencies kx,ky = 2 pi/wavelength [rad/m] as the spatial frequency variables.

; Special care is taken to define the values at the ky and ky Nyquist frequencies, which are
; always special cases.

; Results are shown as 4 contour plots:
; The upper left  panel is the one-sided wave variance spectrum Pis_1S(kx,ky)
; The upper right panel is the real part of the random zhat(kx,ky)
; The lower left  panel is the imaginary part of the random zhat(kx,ky)
; The lower right panel is the generated random sea surface z(x,y)

; ***************************************************************************************************
; * This code is copyright (c) 2016 by Curtis D. Mobley.                                            *
; * Permission is hereby given to reproduce and use this code for non-commercial academic research, *
; * provided that the user suitably acknowledges Curtis D. Mobley in any presentations, reports,    *
; * publications, or other works that make use of the code or its output.  Depending on the extent  *
; * of use of the code or its outputs, suitable acknowledgement can range from a footnote to offer  *
; * of coauthorship.  Further questions can be directed to curtis.mobley@sequoiasci.com.            *
; ***************************************************************************************************

;-----------------------------------------------------------------------------------------------------------
;| This routine uses Coyote Graphics routines rather than standard IDL for plotting; see www.idlcoyote.com |
;| and "Coyote's Guide to Traditional IDL Graphics: Using Familiar Tools Creatively" by David Fanning      |
;| Any comments below like "CG page xxx" refer to this book.                                               |
;| CG routines are wrappers for standard IDL routines, which make it easier to do certain things like      |
;| create vector eps output files; cgPlot replaces the standard Plot, etc.                                 |
;-----------------------------------------------------------------------------------------------------------

; Written 13 August 2014 by Curtis Mobley.

;-----------------  BEGIN USER INPUT  ----------------------

idebug = 0 ; set to 1 for debugging printout

; ***** DEFINE THE PHYSICAL REGION AND SAMPLING ***** 

; define the wind speed at 10 m above MSL for use in the variance spectrum
U10 = 5.0 ; in m/s

Nx = 64 ; number of samples of sea surface elevation to be generated in the x direction; MUST be a power of 2 for the FFT
Ny = 64 ; number of samples of sea surface elevation to be generated in the y direction; MUST be a power of 2 for the FFT

InputOption = 2

CASE InputOption OF
  1: BEGIN
        labelInOpt = ' Spatial resolution is defined by the sample spacing and number of sample points'
        ; smallest resolvable wavelength is 2*Deltax
        Deltax = 0.1; 0.0025
        Deltay = 0.1
        Lx = Deltax*float(Nx)
        Ly = Deltax*float(Ny)
     END 
  2: BEGIN
        labelInOpt = ' Spatial resolution is defined by the spatial region and number of sample points'
        ; largest resolvable wave is the length of the spatial region
        Lx = 100.0 ; length of surface region in the x direction, in meters 
        Ly = 100.0 ; length of surface region in the y direction, in meters 
        Deltax = Lx/float(Nx)
        Deltay = Ly/float(Ny)
     END    
ENDCASE

; Define root names for output files.
; File names will have the form root name + wind speed + x spatial size + x sampling, e.g.
; PlotFileName = 'Fig3.3_U10_L100_N1024.eps'
; The info on wind speed, spatial size, and sampling size will be appended below.

PlotRootName = 'Fig3.3_ECKV'

; Optionally save a text file z2D_PlotRootName.txt of (x,y,z) values for postprocessing
; These z(x,y) files can be plotted by routines cgPlot2Dsurf_3D and cgPlot2Dsurf_contour
; Note: these files can be very large for large Nx, Ny
isave = 1 ; = 1 to save output, 0 to not save

; define an initial seed for random number generation
seed =  833202L; 538832L ;33202L; 838392L ;33202L;
; Note: seed = 833202L was used on a 32 bit computer to generate Tutorial Fig. 3.3.
; Use of a different seed, or this seed on a 64 bit computer, will generate a different
; surface realization because a different sequence of random numbers will be generated.

;-----------------  END USER INPUT  ----------------------

; create output file names

CASE 1 OF
 U10 lt 10.0 : Ustring = string(U10,format='(i1)')
 U10 ge 10.0 : Ustring = string(U10,format='(i2)')
ENDCASE

CASE 1 OF
 Lx lt 10.0 : Lstring = string(Lx,format='(i1)')
 Lx ge 10.0 and Lx lt 100.0 : Lstring = string(Lx,format='(i2)')
 Lx ge 100.0 :  Lstring = string(Lx,format='(i3)') 
ENDCASE

CASE 1 OF
 Nx lt 10 : Nstring = string(Nx,format='(i1)')
 Nx ge 10 and Nx lt 100 : Nstring = string(Nx,format='(i2)')
 Nx ge 100 and Nx lt 1000 :  Nstring = string(Nx,format='(i3)')
 Nx ge 1000 : Nstring = string(Nx,format='(i4)')
ENDCASE

PlotFileName  = PlotRootName  + '_U' + Ustring + '_L' + Lstring + '_N' + Nstring +'.eps' 

; ***** COMPUTE THE SPATIAL FREQUENCIES ***** 

kxmin = 2.0*!pi/Lx ; min frequency (min spatial frequency); frequency of longest resolvable wave
kymin = 2.0*!pi/Ly
minxwave = 2.0*Deltax ; min resolvable wavelength (2 point wave)
minywave = 2.0*Deltay ; min resolvable wavelength (2 point wave)
Nyquistx = !pi/Deltax ; Nyquist spatial freq in rad/m
Nyquisty = !pi/Deltay ; Nyquist spatial freq in rad/m
kxmax = Nyquistx
kymax = Nyquisty

print,' '
print,' Generation of a 2D sea surface from a one-sided, 2D variance spectrum Psi1s(kx,ky) = Psi(k,psi)'
print,' '
print,' Note: This routine does NOT account for unresolved variance'
print,' '
print, format='(4x,a,2i5)',   'Numbers of spatial samples = Nx, Ny =',Nx,Ny
print, format='(4x,a,2f7.4)', 'Sample spacings Deltax, Deltay [m] =', Deltax,Deltay
print, format='(4x,a,2f8.1)', 'Max resolvable wavelengths = Lx, Ly [m] =',Lx,Ly
print, format='(4x,a,2f7.4)', 'Min resolvable wavelengths = 2*Deltax, 2*Deltay [m] =', minxwave,minywave
print, format='(4x,a,2f7.2)', 'Min resolvable frequencies kxmin, kymin [rad/m] = ',kxmin ,kymin
print, format='(4x,a,2f7.2)', 'Max resolvable (Nyquist) frequencies kxmax, kymax [rad/m] = ',kxmax ,kymax
print, format='(4x,a,i12)',   'Seed for random number generation =',seed
print,' '


; ***** NOTE ON FREQUENCY ARRAY ORDER:

; When plotting 1-sided power spectra, it is customary to plot only positive frequencies in the order
; kpos[1 to N/2] = [1, 2,..., N/2-1, N/2(Nyquist freq)]*(2pi/L)  ; total of N/2 pos frequencies

; When plotting 2-sided power spectra, and when going from z(x,y) to zhat(kx,ky) = FFT2D{z(x,y)},
; mathematicians usually write the frequencies in the order
; kmath[1 to N] = [-(N/2-1),...,-2,-1,0,+1,+2,...,+N/2-1,+N/2(Nyquist freq)]*(2pi/L)  ; total of N freqs in rad/m
 
; IDL (and Matlab) stores these frequencies in the order (see the IDL documentation)
; kIDL[1 to N] = [0.0, 1 2, ..., (N-1)/2, N/2(Nyquist freq), -(N-1)/2, ..., -2, -1]*(2pi/L) ; total of N freqs
; This is the (kx,ky) order of zhat(kx,ky) arrays returned by IDL routine FFT.

; The FFT order is generated as follows:
; FFT = FINDGEN((Nx-1)/2) + 1.0 ; = [1,2,...,(N-1)/2]
; kFFT = (2.0*!pi/L)*[0.0, FFT, N/2.0 ,-N/2.0+FFT] ; in rad/m

; The math order can be obtained from the FFT order by a circular shift by N/2-1 to the right:
; kmath = SHIFT(kFFT,Nx/2-1)

; The FFT order can be obtained from the math order by a circular shift by N/2-1 to the left:
; kFFT = SHIFT(kmath,-(N/2-1))

; When defining the one-sided variance spectrum Psi1s for random sampling, we will use (kvec,phi) with 
; vector kvec = sqrt(kx^2 + ky^2) including zero frequencies and -pi/2 <= phi <= pi/2.  This 
; corresponds to 0 and positive kx, and positive and negative ky.  These frequencies are 
; denoted by kx1S = [0.0, kxpos] (Nx/2 +1 values) and ky1S = kymath (Ny values)

; These forms of frequency arrays are computed next for use below.

;----- Compute positive frequencies used for plotting the 1-sided variance spectrum; 
;   these do not include 0 frequency;
;   even spacing in frequency k; 
;   the last frequency is the Nyquist frequency
Nkxpos = Nx/2
Nkypos = Ny/2
; spacing for positive frequencies; Delta includes the 2pi/L values via the kmax and kmin values
Deltakx = (kxmax - kxmin)/float(Nkxpos-1)  
Deltaky = (kymax - kymin)/float(Nkypos-1)
kxpos = kxmin + Deltakx*findgen(Nkxpos) ; in rad/m
kypos = kymin + Deltaky*findgen(Nkypos)

;----- Compute the FFT frequencies needed by the FFT routine:
xFFT = FINDGEN((Nx-1)/2) + 1.0 ; = [1,2,...,(Nx-1)/2]
yFFT = FINDGEN((Ny-1)/2) + 1.0
kxFFT = (2.0*!pi/Lx)*[0.0, xFFT, Nx/2.0 ,-Nx/2.0+xFFT] ; in rad/m
kyFFT = (2.0*!pi/Ly)*[0.0, yFFT ,Ny/2.0, -Ny/2.0+yFFT]

;----- Compute the math frequencies
kxmath = SHIFT(kxFFT,Nx/2-1)
kymath = SHIFT(kyFFT,Ny/2-1)

;----- Compute the 1S frequencies
kx1S = [0.0, kxpos]
ky1S = kymath

;if(idebug eq 1) then begin
;  print,' kxpos  = ',kxpos
;  print,' kxFFT  = ',kxFFT
;  print,' kxmath = ',kxmath
;  print,' kx1S   = ',kx1S
;  print,' kypos  = ',kypos
;  print,' kyFFT  = ',kyFFT
;  print,' kymath = ',kymath 
;  print,' ky1S   = ',ky1S 
;endif


; ***** SET UP THE PLOT ***** 

; Set up a multipanel layout
;   (Note: do NOT use /noerase with multipanel plots!)

nplotcol = 2 ; number of plot columns
nplotrow = 2 ; number of plot rows

!p.multi = [0,nplotcol,nplotrow] 

; Set up for a vector eps output file; a high-quality png file will also be created

!x.margin = [5,7]
!y.margin = [4,3]
!x.Omargin = [2,2]
!y.Omargin = [1,3]

aspectratio = 1.0
xsize= 6.5
ysize=xsize*aspectratio
charsize = 1.0
!p.charsize = charsize

PS_Start,filename=PlotFileName, /encapsulated, font=1, tt_font='Helvetica', $
         charsize=charsize, default_thickness=2, /nomatch, xsize=xsize, ysize=ysize
 
; pick color tables below, since may want different colors or ranges for different contour plots 


; ***** COMPUTE THE 1-SIDED, 2D VARIANCE SPECTRUM Psi1S(kx1S,ky1S) ***** 

; Use the 1-sided, 2D variance spectrum using Psi(k,phi) of Elfouhaly et al Eq. (67) and 
; Psi(kx,ky) = k Psi(k,phi).  Psi1s is needed only for "downwind" to "crosswind" directions.

print,' '
print,format='(4x,a,f5.1)', 'The 2D variance spectrum is Elfouhaily et al. with U10 =',U10 
print,' '

;----- Compute the 1-sided variance spectrum Psi1s(kx,ky) for all kx,ky values. 

; The variance spectrum is computed only for "downwind" directions (kx1S,ky1S).  
; The upwind directions (negative kx) are obtained by symmetry: E(-kvec) = E(kvec).

; To avoid having multiple large arrays, make multiple calls to ECKV2D_k_phi with 
; scalar k and phi values. This calculation is done only once if multiple random 
; surfaces are generated for a given variance spectrum.

; Psi1s(kx,ky) = Psi(k,phi) for the corresponding (kx,ky) and (k,psi) points.

; Note that Psi1s still has the magnitude of the one-sided variance spectrum as usually
; computed and plotted, even though it has been extended to negative frequencies.
; The factor of 1/2 on Psi1s to get a two-sided spectrum Psi2S is included in C3.

Psi1s = fltarr(Nx,Ny) ;holds all kxmath and kymath frequencies
for ikx = 0,Nx/2 do begin ; loop over non-neg kx values kx1S
  for iky = Ny/2-1,Ny-1 do begin ; non-negative ky values
    k = SQRT(kx1S[ikx]*kx1S[ikx] + ky1S[iky]*ky1S[iky])
    phirad = ATAN(ky1S[iky],kx1S[ikx])   

    Psi1s[ikx+Nx/2-1,iky] = ECKV2D_k_phi(k,phirad,U10) ; Psi1s(kx,ky) = Psi1s(k,phi)
    if iky ge Ny/2 and iky le Ny-2 then Psi1s[ikx+Nx/2-1,Ny-2-iky] = Psi1s[ikx+Nx/2-1,iky] ; negative ky values by symmetry
  endfor
endfor

; fill in the negative kx values by symmetry
for iky=0,Ny-1 do begin
  for ikx = 0,Nx/2-2 do Psi1s(ikx,iky) = Psi1s(Nx-2-ikx,iky)
endfor

; set (0,0) frequency to 0 to force MSL = 0:
Psi1s[Nx/2-1,Ny/2-1] = 0.0 

; Psi1s now has all frequencies in math order

;print,' '
;print,' Psi1s(kxmath,kymath); (0,0) at center;  Psi1s(-kvec) = Psi1s(+kvec)'
;print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
;print,format='(a8,9x,16f12.3)',' kx =',kxmath
;for iky=Ny-1,0,-1 do begin
;    print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kymath(iky),Psi1s(*,iky)
;endfor

; ***** CONTOUR THE ONE-SIDED, 2D VARIANCE SPECTRUM; UPPER LEFT PANEL *****  

Psi1splot = Psi1s(Nx/2-1:Nx-1,*) ; temp array for plotting the right half of the symmetrical variance spectrum
;Psi1splot = Psi1s  ; for contouring the 2sided spectrum (incl neg kxmath)

minE = min(Psi1splot,max=maxE)
print,' min, max Psi1s = ',minE,maxE

Eminplot = 1.0e-10 ; adjust value for nice-looking output
Emaxplot = 10.0
Psi1splot[where(Psi1splot lt Eminplot,/NULL)] = Eminplot

logPsi1s = alog10(Psi1splot)
Psi1splot = 0.0 ; done with Psi1splot; free storage

; reset contour range to nice values and define contour levels
logEmin = FIX(alog10(Eminplot))
logEmax = FIX(alog10(Emaxplot))

Nlevels = -(logEmin - logEmax) + 1 ; = num of contour lines including minz and maxz
Elevels = logEmin + (logEmax - logEmin)/float(Nlevels-1)*findgen(Nlevels)  
;print,'contour levels for variance = ',Elevels 
contnames = strarr(Nlevels)
for i=0,n_elements(contnames)-1 do contnames(i) = '10!u' + string(Elevels(i),format='(i3)') + '!n'
;print,'contnames = ',contnames

; load a color table for this contour plot
firstcolor = 1 ; index of first color in the color table to use for the min contour
Ncolors = Nlevels-1 ; number of colors to use in contouring

cgLoadCT, 33, NColors=Ncolors, Bottom=firstcolor

title = Greek('Psi',/PS) + '!d1S!n(k!dx!n,k!dy!n) [m!u2!n]'
xlabel = 'k!dx!n [rad/m]'
ylabel = 'k!dy!n [rad/m]'

cgContour,logPsi1s,kx1S,ky1S,label=1,title = title, $
;cgContour,logPsi1s,kxmath,kymath,label=1,title = title, $ ; for contouring the 2sided spectrum (incl neg kxmath)
  levels=Elevels,fill=1,C_Colors=Indgen(Ncolors)+1, $
  xrange=[0,kxmax],xtitle=xlabel, $
  ;xrange=[-kxmax,kxmax],xtitle=xlabel, $  ; for contouring the 2sided spectrum (incl neg kxmath)
  yrange=[-kymax,kymax],ytitle=ylabel

; add a color bar at the right
barpos = [0.47, 0.55, 0.49, 0.87] ; color bar position, normal coords [xll,yll,xur,yur]

cgColorbar, NColors=ncolors, Bottom=firstcolor, /Discrete, /vertical, position=barpos, $ ;/Fit,
           Range=[Min(Elevels), Max(Elevels)], OOB_High='white', OOB_Low='black',format='(f5.1)', $
           ticknames=contnames,charsize=1.1*!p.charsize 

logPsi1s = 0.0 ; ; now done with logPsi1s array; free storage


; ***** GENERATE A RANDOM HERMITIAN zhat(kx,ky) ***** 

; Recall the note above on frequency order in FFTs.
 
; When creating a random zhat for the IDL inverse FFT routine:
;   The order of the random amplitudes must be in the FFT frequency order;
;   The real and imag parts must give a Hermitian complex amplitude, 
;      so that the inverse FFT gives a real function
;   Hermitian means H(-k) = H*(k); H* = complex conjugate of H

; The computation order is as follows:
 
; (1) Draw random numbers and define a random, complex zhat(kx1S,ky1S) using the
;     the one-sided variance spectrum Psi1s(kx1S,ky1S) computed above.
     
; (2) Use Hermitian symmetry to define zhat(kxmath,kymath) for the negative kxmath frequencies.
;     Note that variance spectra satisfy E(-kvec) = E(kvec) and E(kvec=0) = 0 ; kvec=(kxmath,kymath).
;     The math freq order is convenient for defining zhat and for contour plotting to check the symmetry of zhat.    
;     Check Hermitian requirements on Real{zhat} and Imag{zhat} by contouring these arrays:
;     Recall that Real{zhat} is an even functiion of kmath
;     Recall that Imag{zhat} is an odd functiion of kmath    

; (3) Shift zhat(kxmath,kymath) to get zhat(kxFFT,kyFFT) for the FFT

; (4) compute z(x,y) = Real{invFFT{zhat(kxFFT,kyFFT)}}
;     Check that Imag{invFFT{zhat}} = 0

; shift from math to FFT frequency order

Psi1s = SHIFT( Psi1s, -(Nx/2-1), -(Ny/2-1) )

; (0,0) freq should now be at array location [0,0]
print,'FFT freq order: Psi1s[0,0] (should = 0) = ',Psi1s[0,0]

; C3 is the product of
; the the missing 1/sqrt(2) from sqrt(Psi1s/2) in Tessendorf's Eq. 41
; the 1/sqrt(2) in the def of hhat_o in Tessendorf Eq 41
; the missing 1/sqrt(2) in Tessendorf's equation 43
C3 = 1.0/SQRT(8.0)

; Psi1s is the value of the one-sided, continuous variance spectrum at freq (kx,ky) in units of m^2/(rad/m)^2.  
; Must multiply by Deltakx and Deltakx to get variance (m^2) contained in the finite spatial 
; frequency intervals Deltakx and Deltaky.  See Light and Water Eq. (4.78).
; Psiroot is now the square root of the discrete TWO-sided spectrum, Psi_2s(u,v), including all factors 
; missing from the Tessnedorf equations.

Psiroot = C3*SQRT(Psi1s*Deltakx*Deltaky) 

Psi1s = 0.0 ; now done with Psi1s array; free storage


;***** CAN START LOOPING HERE TO GENERATE MULTIPLE SURFACES 
;      FOR THE SAME VARIANCE   SPECTRUM BUT DIFFERENT RANDOMIZATIONS.


; ----- Draw a pair of independent N(0,1) random numbers (ranr, rans) for each kx and ky

; These will set the random amplitudes and phases of the waves, or real and imagnary parts of zhat

; RANDOMU is the IDL random number generator (period ~ 10^8); /normal gives normal(0,1) random variables

ranr = fltarr(Nx,Ny) ; for the random real parts of zhat
rans = fltarr(Nx,Ny) ; for the random imaginary parts
for ikx=0,Nx-1 do ranr(ikx,*) = RANDOMU(seed,Ny,/normal) 
for ikx=0,Nx-1 do rans(ikx,*) = RANDOMU(seed,Ny,/normal)

; define the complex Hermitian zhat array for the full range of pos and negative frequencies

zhat = COMPLEXARR(Nx,Ny) 

;----- Compute zhat(kx,ky)

; NOTE: this code computes zhat at time = 0, which gives cos(omega t) = 1 and sin(omega t) = 0
; in the general time-dependent equations used in cgAnimate2D.pro.  The zhat equation below is
; therefore not the most general form for zhat(kx,ky,t).

; The general method is based on Tessendorf (2004; Simulating Water Surfaces), as 
; corrected for consistency with the chosen one-sided wave variance spectrum

; note that for the FFT frequency ordering, k(-j) = k(N-j)

; Must now loop over all frequencies, and treat the Nyquist frequencies as special cases

; the non-zero and non-Nyquist frequencies:
 FOR ikx=1,Nx/2-1 DO BEGIN
   FOR iky=1,Ny-1 DO BEGIN

      zhat(ikx,iky) = COMPLEX( ranr(ikx,iky) * Psiroot(ikx,iky) + ranr(Nx-ikx,Ny-iky) * Psiroot(Nx-ikx,Ny-iky) , $
                             rans(ikx,iky) * Psiroot(ikx,iky) - rans(Nx-ikx,Ny-iky) * Psiroot(Nx-ikx,Ny-iky) )
      zhat(Nx-ikx,Ny-iky) = CONJ( zhat(ikx,iky) )
   ENDFOR
 ENDFOR

; all ky for kx = 0 freq at index 0 and the Nyquist frequency at kx index Nx/2:
 FOR iky=1,Ny/2-1 DO BEGIN
    ikx = Nx/2
    zhat(ikx,iky) = COMPLEX( ranr(ikx,iky) * Psiroot(ikx,iky) + ranr(Nx-ikx,Ny-iky) * Psiroot(Nx-ikx,Ny-iky) , $
                             rans(ikx,iky) * Psiroot(ikx,iky) - rans(Nx-ikx,Ny-iky) * Psiroot(Nx-ikx,Ny-iky) )
    zhat(ikx,Ny-iky) = CONJ( zhat(ikx,iky) )
    ikx = 0
    zhat(ikx,iky) = COMPLEX( ranr(ikx,iky) * Psiroot(ikx,iky) + ranr(ikx,Ny-iky) * Psiroot(ikx,Ny-iky) , $
                             rans(ikx,iky) * Psiroot(ikx,iky) - rans(ikx,Ny-iky) * Psiroot(ikx,Ny-iky) )
    zhat(ikx,Ny-iky) = CONJ( zhat(ikx,iky) )
 ENDFOR

 ; all kx for ky = 0 and Nyquist frequency at ky index Ny/2:
 FOR ikx=1,Nx/2 DO BEGIN
   iky = Ny/2
    zhat(ikx,iky) = COMPLEX( ranr(ikx,iky) * Psiroot(ikx,iky) + ranr(Nx-ikx,Ny-iky) * Psiroot(Nx-ikx,Ny-iky) , $
                             rans(ikx,iky) * Psiroot(ikx,iky) - rans(Nx-ikx,Ny-iky) * Psiroot(Nx-ikx,Ny-iky) )
    zhat(Nx-ikx,iky) = CONJ( zhat(ikx,iky) )
   iky = 0
    zhat(ikx,iky) = COMPLEX( ranr(ikx,iky) * Psiroot(ikx,iky) + ranr(Nx-ikx,iky) * Psiroot(Nx-ikx,iky) , $
                             rans(ikx,iky) * Psiroot(ikx,iky) - rans(Nx-ikx,iky) * Psiroot(Nx-ikx,iky) )
    zhat(Nx-ikx,iky) = CONJ( zhat(ikx,iky) )
 ENDFOR

 ; Nyquist ky freq at ky index Ny/2
 ikx = 0
 iky = Ny/2
 zhat(ikx,iky) = COMPLEX( ranr(ikx,iky) * Psiroot(ikx,iky) + ranr(ikx,Ny-iky) * Psiroot(ikx,Ny-iky) , $
                          rans(ikx,iky) * Psiroot(ikx,iky) - rans(ikx,Ny-iky) * Psiroot(ikx,Ny-iky) )

 ; Nyquist kx freq at kx index Nx/2
 ikx = Nx/2
 iky = 0
 zhat(ikx,iky) = COMPLEX( ranr(ikx,iky) * Psiroot(ikx,iky) + ranr(Nx-ikx,iky) * Psiroot(Nx-ikx,iky) , $
                          rans(ikx,iky) * Psiroot(ikx,iky) - rans(Nx-ikx,iky) * Psiroot(Nx-ikx,iky) )

 ; the "double" Nyquist frequency at Nx/2,Ny/2
 ikx = Nx/2
 iky = Ny/2
 zhat(ikx,iky) = COMPLEX( ranr(ikx,iky) * Psiroot(ikx,iky) + ranr(Nx-ikx,Ny-iky) * Psiroot(Nx-ikx,Ny-iky) , $
                          rans(ikx,iky) * Psiroot(ikx,iky) - rans(Nx-ikx,Ny-iky) * Psiroot(Nx-ikx,Ny-iky) )

 ; set the (0,0) value to 0 (MSL = 0)
 zhat(0,0) = COMPLEX(0.0, 0.0)


; zhat as just defined has the frequencies in FFT order, as needed by the FFT routine.

; for plotting the real and imaginary parts in math order, shift back to math order:
; The math order can be obtained from the FFT order by a circular shift by N/2-1 to the right:
; kmath = SHIFT(kFFT,Nx/2-1)

Zreal = SHIFT( REAL_PART(zhat), Nx/2-1, Ny/2-1)
Zimag = SHIFT( IMAGINARY(zhat), Nx/2-1, Ny/2-1)

idebug = 0
if idebug eq 1 then begin

print,' '
print,' Real{zhat}(kxmath,kymath); (0,0) at center;  Real{zhat(-k)} = Real{zhat(k)}'
print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
print,format='(a8,9x,16f12.3)',' kx =',kxmath
for iky=Ny-1,0,-1 do begin
    print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kymath(iky),zreal(*,iky)
endfor

print,' '
print,' Imag{zhat}(kxmath,kymath); (0,0) at center;  Imag{zhat(-k)} = -Imag{zhat(k)}'
print,format='(a8,11x,16i12)',' ikx =',indgen(Nx)
print,format='(a8,9x,16f12.3)',' kx =',kxmath
for iky=Ny-1,0,-1 do begin
    print,format='(a8,i5,f9.3,16e12.2)',' iky, ky =',iky,kymath(iky),zimag(*,iky)
endfor

endif

; CONTOUR ZREAL AND ZIMAG TO CHECK SYMMETRY


minRe = min(zreal,max=maxre)
print,'minRe, maxRe = ',minRe, maxRe

; reset plot ranges to nice values and define nice contour levels
minzhat = -0.02;  min([minRe,minIM])
maxzhat = 0.02 ;max([maxRe,maxIM])

Nlevels = 11; -(minzhat - maxzhat) + 1 ; = num of contour lines including minz and maxz
zhatlevels = minzhat + (maxzhat - minzhat)/float(Nlevels-1)*findgen(Nlevels)  
;print,'contour levels for zhat = ',zhatlevels 
contnames = strarr(Nlevels)
;for i=0,n_elements(contnames)-1 do contnames(i) = '10!u' + string(zhatlevels(i),format='(i3)') + '!n'
for i=0,n_elements(contnames)-1 do contnames(i) = string(zhatlevels(i),format='(f6.3)')
;print,'contnames = ',contnames

; load a color table for this contour plot
firstcolor = 1 ; index of first color in the color table to use for the min contour
Ncolors = Nlevels-1 ; number of colors to use in contouring

cgLoadCT, 33, NColors=Ncolors, Bottom=firstcolor

; second plot, in upper right panel

;print,'zreal[128,*]= ',zreal[128,*]
cgContour,zreal,kxmath,kymath,title = 'Real{zhat} [m]', $
  xrange=[-kxmax,kxmax],xtitle=xlabel, $
  yrange=[-kymax,kymax],ytitle=ylabel, $
  levels=zhatlevels,fill=1,C_Colors=Indgen(Ncolors)+1
  
  ; add a color bar at the right
barpos = [0.95, 0.55, 0.97, 0.87]
;barpos = [0.47, 0.09, 0.49, 0.41] ; color bar position, normal coords [xll,yll,xur,yur]

cgColorbar, NColors=ncolors, Bottom=firstcolor, /Discrete, /vertical, position=barpos, $ ;/Fit,
           Range=[Min(Elevels), Max(Elevels)], OOB_High='white', OOB_Low='black',format='(f5.1)', $
           ticknames=contnames,charsize=0.8*!p.charsize 
 
  
; third plot, in lower left panel

minIm = min(zimag,max=maxIm)
print,'minIm, maxIm = ',minIm, maxIm
 
; ; reset plot ranges to nice values and define nice contour levels
;minzhat = -0.12
;maxzhat = 0.12

Nlevels = 11; -(minzhat - maxzhat) + 1 ; = num of contour lines including minz and maxz
zhatlevels = minzhat + (maxzhat - minzhat)/float(Nlevels-1)*findgen(Nlevels)  
;print,'contour levels for zhat = ',zhatlevels 
contnames = strarr(Nlevels)
;for i=0,n_elements(contnames)-1 do contnames(i) = '10!u' + string(zhatlevels(i),format='(i3)') + '!n'
for i=0,n_elements(contnames)-1 do contnames(i) = string(zhatlevels(i),format='(f6.3)')
;print,'contnames = ',contnames

; load a color table for this contour plot
firstcolor = 1 ; index of first color in the color table to use for the min contour
Ncolors = Nlevels-1 ; number of colors to use in contouring

cgLoadCT, 33, NColors=Ncolors, Bottom=firstcolor
   
cgContour,zimag,kxmath,kymath,title = 'Imag{zhat} [m]', $
  xrange=[-kxmax,kxmax],xtitle=xlabel, $
  yrange=[-kymax,kymax],ytitle=ylabel, $
  levels=zhatlevels,fill=1,C_Colors=Indgen(Ncolors)+1

; add a color bar at the right
barpos = [0.47, 0.09, 0.49, 0.41]
;barpos = [0.95, 0.09, 0.97, 0.41] ; color bar position, normal coords [xll,yll,xur,yur]

cgColorbar, NColors=ncolors, Bottom=firstcolor, /Discrete, /vertical, position=barpos, $ ;/Fit,
           Range=[Min(Elevels), Max(Elevels)], OOB_High='white', OOB_Low='black',format='(f5.1)', $
           ticknames=contnames,charsize=0.8*!p.charsize 
            

; ***** TAKE THE INVERSE FFT TO GET THE SEA SURFACE *****


print,' '
print,'Frequency-shifted zhat(0,0) [should = (0,0)] =',zhat(0,0)

; take the inverse FFT of zhat in FFT frequency order

zcomplx = FFT(zhat,/inverse)


; ***** EXTRACT THE SEA SURFACE *****

; The surface wave ampltudes are the real part of the inverse FFT
zsurf = REAL_PART(zcomplx)
;zimag = IMAGINARY(zcomplx)

; ----- Checks on the generated surface

print,' '
print,'Checks on the generated z(x,y):'
print,' '

; compute the mean sea surface, which should be zero, the mean squared surface elevation,
; and sum of amplitudes squared from full (two-sided) zhat array
sumz = 0.0
sumzsq = 0.0
sumzhat2 = 0.0
for ix = 0,Nx-1 do begin
  for iy = 0,Ny-1 do begin
    sumz = sumz + zsurf(ix,iy)
    sumzsq = sumzsq + zsurf(ix,iy)^2
    sumzhat2 = sumzhat2 + ABS(zhat(ix,iy)^2) ; (ix,iy) ranges same as (u,v) ranges
  endfor
endfor

; compute total energy from the one-sided (1S) energy spectrum
; add terms that have only 0 or pos freqs once, and double terms than have both pos and neg freqs
sumzhatsq = ABS(zhat(0,0))^2
for iky=0,Ny-1 do sumzhatsq = sumzhatsq + ABS(zhat(0,iky))^2 + ABS(zhat(Nx/2,iky))^2 ; zero and Nyquist in x for all y; occur once
for ikx=1,Nx/2-1 do begin
  for iky=0,Ny-1 do begin
    sumzhatsq = sumzhatsq + 2.0*ABS(zhat(ikx,iky))^2 ; all other freqs occur twice as pos and neg freqs
  endfor
endfor

zavg = sumz/(float(Nx)*Float(Ny))

; FIX THIS--BAD VALUES
print,'   avg z(x,y) (should = 0) = ',zavg

print,'   max |Imag{zcomplx}| (should = 0) =',MAX(ABS(IMAGINARY(zcomplx)))

print,' '
print,"Parseval's identity:"
print,'                            sum z^2 =',sumzsq
print,'   Nx * Nx * sum zhat^2 (one-sided) =',float(Nx)*float(Ny)*sumzhatsq
print,'   Nx * Nx * sum zhat^2 (two-sided) =',float(Nx)*float(Ny)*sumzhat2

print,' '
; the significant wave height from avg of z^2
avgzsq = sumzsq/(float(Nx)*Float(Ny))
H13 = 4.0*sqrt(avgzsq) ; approximate significant wave height
print,'   significant wave height H_1/3 = 4 SQRT(avg{z^2}) =',H13

; compute the mean square slopes and mean slope angle in x and y directions
dzdx2 = 0.0
thetax = 0.0
for ix = 1,Nx-1 do begin
  for iy = 0,Ny-1 do begin
    dzdx2 = dzdx2 + (zsurf(ix,iy) - zsurf(ix-1,iy))^2
    thetax = thetax + abs(atan(zsurf(ix,iy) - zsurf(ix-1,iy),Deltax)*!radeg)
  endfor
endfor
dzdx2 = dzdx2/(Deltax*Deltax*float(Nx-1)*float(Ny))
thetax = thetax/(float(Nx-1)*float(Ny))

dzdy2 = 0.0
thetay = 0.0
for iy = 1,Ny-1 do begin
  for ix = 0,Nx-1 do begin
    dzdy2 = dzdy2 + (zsurf(ix,iy) - zsurf(ix,iy-1))^2
    thetay = thetay + abs(atan(zsurf(ix,iy) - zsurf(ix,iy-1),Deltay)*!radeg)
  endfor
endfor
dzdy2 = dzdy2/(Deltay*Deltay*float(Nx)*float(Ny-1))
thetay = thetay/(float(Nx)*float(Ny-1))

print,'   sample mean square slope, alongwind = ', dzdx2
print,'   sample mean square slope, crosswind = ', dzdy2
print,'   total sample mean square slope      = ',dzdx2 + dzdy2
print,'   Cox-Munk mss_x = ',0.0316*U10
print,'   Cox-Munk mss_y = ',0.0192*U10
print,'   Cox-Munk mss   = ',0.001*(3.0 + 5.12*U10)
print,'   sample avg slope angle, alongwind = ', thetax
print,'   sample avg slope angle, crosswind = ', thetay

zcomplx = 0.0 ; now done with zcomplx array; free storage


; ***** PLOT THE SEA SURFACE; LOWER RIGHT PANEL *****


minz = min(zsurf,max=maxz)
print,'minz, maxz = ',minz, maxz

; reset contour range to nice values and define contour levels
minz = -0.6
maxz = 0.6

Nlevels = 9; -(minzhat - maxzhat) + 1 ; = num of contour lines including minz and maxz
zlevels = minz + (maxz - minz)/float(Nlevels-1)*findgen(Nlevels)  
;print,'contour levels for zhat = ',zlevels 
contnames = strarr(Nlevels)
;for i=0,n_elements(contnames)-1 do contnames(i) = '10!u' + string(zhatlevels(i),format='(i3)') + '!n'
for i=0,n_elements(contnames)-1 do contnames(i) = string(zlevels(i),format='(f6.2)')
;print,'contnames = ',contnames

; load a color table for this contour plot
firstcolor = 1 ; index of first color in the color table to use for the min contour
Ncolors = Nlevels-1 ; number of colors to use in contouring
;print,'Ncolors = ',Ncolors

cgLoadCT, 33, NColors=Ncolors, Bottom=firstcolor

; for nice plotting of surface elevation as a periodic function of (x,y), 
; add values at z(Lx,*) = z(0,*) and z(*,Ly) = z(*,0) 

zplot=fltarr(Nx+1,Ny+1)
zplot(0:Nx-1,0:Ny-1) = zsurf
zplot(Nx,0:Ny-1) = zsurf(0,*)
zplot(0:Nx-1,Ny) = zsurf(*,0)
zplot(Nx,Ny) = zsurf(0,0)

xplot = Deltax*findgen(Nx+1)
yplot = Deltay*findgen(Ny+1)

cgContour,zplot,xplot,yplot,title = 'z [m]', $
  xrange=[0.0,Lx],xtitle='x [m]', $
  yrange=[0.0,Ly],ytitle='y [m]', /ystyle, $
  levels=zlevels,fill=1,C_Colors=Indgen(Ncolors)+1
  
  ; add a color bar at the right
barpos = [0.95, 0.085, 0.97, 0.404]
;barpos = [0.47, 0.09, 0.49, 0.41] ; color bar position, normal coords [xll,yll,xur,yur]

cgColorbar, NColors=ncolors, Bottom=firstcolor, /Discrete, /vertical, position=barpos, $ ;/Fit,
           Range=[Min(Elevels), Max(Elevels)], OOB_High='white', OOB_Low='black',format='(f5.1)', $
           ticknames=contnames,charsize=0.8*!p.charsize 
 

; add a label at the top of the plot
  
label = 'U!d10!n =' + string(U10,format='(i2)') + ' m s!u-1!n; (L!dx!n, L!dy!n) = (' $
                    + string(Lx,format='(i4)') + ',' + string(Ly,format='(i4)') $
                    + '); (N!dx!n, N!dy!n) = (' + string(Nx,format='(i3)') + ',' + string(Ny,format='(i3)') +')'

cgText,0.5,0.95,label,align=0.5,charsize=1.3*!p.charsize,/normal

cgText,0.02,0.02,'cgGenerate_2D_SeaSurfaces',charsize=0.5*!p.charsize,/normal
    
;   ; add statistics labels
;  swhlabel  = 'H!d1/3!n =' + string(H13,format='(f7.2)') + ' m'
;  mssxlabel = 'mss!dx!n =' + string(dzdx2,format='(f7.3)')
;  mssylabel = 'mss!dy!n =' + string(dzdy2,format='(f7.3)')
;  thetaxlabel =  '!9' + string("341B) + '!x' + Greek('theta',/PS) + '!dx!n !9' $
;                      + string("361B) + '!x =' + string(thetax,format='(f7.3)') + ' deg'
;  thetaylabel =  '!9' + string("341B) + '!x' + Greek('theta',/PS) + '!dy!n !9' $
;                      + string("361B) + '!x =' + string(thetay,format='(f7.3)') + ' deg'                      
;  xlab = 0.75
;  dylab = 0.05
;  cgText,xlab,0.85,swhlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red') 
;  cgText,xlab,0.85-dylab,mssxlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red') 
;  cgText,xlab,0.85-2*dylab,mssylabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
;  cgText,xlab,0.85-3*dylab,thetaxlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
;  cgText,xlab,0.85-4*dylab,thetaylabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
;    


; clean up and close the plot file
; Use /PNG to create a high-resolution png file
; Use /delete_ps to delete the intermediate (large) Postscript file (omit to also create an EPS file)
; NOTE: see documentation in PS_End for additional software (ImageMagick) required to create PNG files from EPS files.

PS_End,Resize=50,/PNG,/delete_ps

print,'image file ', PlotFileName,' closed OK'

!p.multi = 0 ; reset to normal layout


; ***** SAVE THE SURFACE IF DESIRED *****

; Surface files on this format can be plotted by routines cgPlot2Dsurf_3D and cgPlot2Dsurf_contour

if isave ne 0 then begin
   zsaveFileName = 'z2D_' + PlotRootName + '_U' + Ustring + '_L' + Lstring + '_N' + Nstring +'.txt'
   openw,nunit,zsaveFileName,/get_lun
   printf,nunit,'Random sea surface realization z(x,y)'
   printf,nunit,format='(2i5,3f10.3,a)',Nx,Ny,Lx,Ly,U10,' = Nx,Ny, Lx,Ly, U10'
   printf,nunit,format='("    i    j      x(i)      y(j)    z(x,y)")'

   for j=Ny,0,-1 do begin
     for i=0,Nx do printf,nunit,format='(2i5,3f10.3)',i,j,xplot(i),yplot(j),zplot(i,j)
   endfor  
   free_lun,nunit
   print,' Surface data written to file ',zsaveFileName
endif


print,'end gen2dsurf'
end


