pro plotsurf_3D

; This routine reads a file with a 2D sea surface z(x,y) created by cgGenerate_2D_SeaSurface
;    and plots the surface in 3D perspective view.

; This routine created Tutorial Fig. and 3.4

; ***************************************************************************************************
; * This code is copyright (c) 2016 by Curtis D. Mobley.                                            *
; * Permission is hereby given to reproduce and use this code for non-commercial academic research, *
; * provided that the user suitably acknowledges Curtis D. Mobley in any presentations, reports,    *
; * publications, or other works that make use of the code or its output.  Depending on the extent  *
; * of use of the code or its outputs, suitable acknowledgement can range from a footnote to offer  *
; * of coauthorship.  Further questions can be directed to curtis.mobley@sequoiasci.com.            *
; ***************************************************************************************************

;-----------------------------------------------------------------------------------------------------------
;| This routine uses Coyote Graphics routines rather than standard IDL for plotting; see www.idlcoyote.com |
;| and "Coyote's Guide to Traditional IDL Graphics: Using Familiar Tools Creatively" by David Fanning      |
;| Any comments below like "CG page xxx" refer to this book.                                               |
;| CG routines are wrappers for standard IDL routines, which make it easier to do certain things like      |
;| create vector eps output files; cgPlot replaces the standard Plot, etc.                                 |
;-----------------------------------------------------------------------------------------------------------

; Written 13 December 2013 by Curtis Mobley.  Copyright (c) by Curtis Mobley, 2013.

;------------------

; Select a file of input surface elevation data.
; Files with this format are created by cgInvFFT2D* routines.
;SurfFileName = 'z2D_U10_L100_1024.txt'
;SurfFileName = 'z2DHydroPol-2D-nok_U10_L100_N2048.txt'

SurfFileName = DIALOG_PICKFILE(FILTER='z2D*',TITLE='Pick a z2D*.txt file' )

; Select the name for the output plot file.
PlotFileName = 'Fig3.4.eps' ;


header = ' '
openr,nunit,SurfFileName,/get_lun
  readf,nunit,format='(a)',header
  readf,nunit,format='(2i5,3f10.3,a)',Nx,Ny,Lx,Ly,U10
  readf,nunit,format='(a)',header   

  ; The surface file already has had an extra row and column added 
  ; with z(Nx,*) = z(0,*) and z(*,Ny) = z(*,0) to give a full periodic tile
  xplot=fltarr(Nx+1)
  yplot=fltarr(Ny+1)
  zplot=fltarr(Nx+1,Ny+1)
  help,zplot
  xread = 0.0 & yread = 0.0 & zread = 0.0
  for j=Ny,0,-1 do begin
    for i=0,Nx do begin
      readf,nunit,format='(2i5,3f10.3)',i,j,xread,yread,zread
      xplot(i) = xread
      yplot(j) = yread
      zplot(i,j) = zread
    endfor  
  endfor  
free_lun,nunit

print,' '
print,' Plot of a 2D sea surface z(x,y)'
print,' '
print, format='(4x,a,2f8.1)','The surface region size Lx, Ly [m] =',Lx,Ly
print, format='(4x,a,2i5)', 'The numbers of spatial samples = Nx, Ny =',Nx,Ny
print, format='(4x,a,2f7.3)', 'Sample spacings Deltax, Deltay [m] =', Lx/Nx,Ly/Ny
print,' '

; set up output for a vector eps file

!x.margin = [2,4]
!y.margin = [3,0]
;!x.Omargin = [3,5]
;!y.Omargin = [0,3]

aspectratio = 0.7
xsize= 6.5
ysize=xsize*aspectratio
charsize = 1.5
!p.charsize = charsize

PS_Start,filename=PlotFileName, /encapsulated, font=1, tt_font='Helvetica', $
         charsize=charsize, default_thickness=2, /nomatch, xsize=xsize, ysize=ysize
 
; pick color table below, since may want different colors or ranges for different plots 

;------------------------ 

xlabel = 'x [m]'
ylabel = 'y [m]'
zlabel = 'z [m]'

minz = min(zplot,max=maxz)
print,'minz, maxz = ',minz, maxz
;maxz = max(abs(zsurf))

; reset z axis range to nice values
minz = -1.5
maxz = 1.5

; can see color tables with Help --> Color Tables --> samples
cgLoadCT,1
;       cgSurf, data
;       cgSurf, data, Shades=BytScl(data)
;       cgSurf, data, /Shaded
;       cgSurf, data, /Shaded, Shades=BytScl(data) 

; Set up plot coordinates for a shaded surface seen in 3D perspective

; axes are plotted below for best appearance

cgSurf,zplot,xplot,yplot, /save, $ 'skirt=minz,$
  /shaded, /elevation_shading, shades=BytScl(zplot), $ ;,/nodata
  xrange=[0,Lx], xtitle=xlabel, xstyle=5, $  ; xstyle=5 forces no x axis
  yrange=[0,Ly], ytitle=ylabel, ystyle=5,$
  zrange=[minz,maxz], ztitle=zlabel, rotx=75,rotz=-55, zstyle=5, $
  charsize=1.2*!p.charsize

; add axes in desired locations

xaxmin = !x.crange[0]
xaxmax = !x.crange[1]
yaxmin = !y.crange[0]
yaxmax = !y.crange[1]
zaxmin = !z.crange[0]
zaxmax = !z.crange[1]

; xaxis
axis,xaxmax,yaxmin,zaxmin,xaxis=0,xtitle=xlabel,charsize=1.2*!p.charsize,/T3D,ticklen=-0.02

; yaxis:
axis,xaxmax,yaxmin,zaxmin,yaxis=1,ytitle=ylabel,charsize=1.2*!p.charsize,/T3D,ticklen=-0.02

; zaxis:
axis,xaxmin,yaxmin,zaxmin,zaxis=1,ztitle=zlabel,charsize=3.5*!p.charsize,/T3D,ticklen=-0.02


; plot header

; add a label at the top
  toplabel = 'U!d10!n =' +  string(U10,format='(f5.1)') + ' m s!u-1!n;' +  $
   '  (N!dx!n,N!dy!n) = (' + string(Nx,format='(i4)') + ',' + string(Ny,format='(i4)') + ')'
     
cgText,0.5,0.94,toplabel,align=0.5,charsize=1.0*!p.charsize,/normal

Sparam = 20
; optional sublabel at the top  
 ; toplabel2 = 'ECKV; ' + Greek('Phi',/PS) + '= C!dS!ncos!u2S!n(' + Greek('phi',/PS) + '/2), S =' + string(Sparam,format='(i3)')

 toplabel2 = 'ECKV; symmetric ' + Greek('Phi',/PS) + '(k,' + Greek('phi',/PS) + ')'

cgText,0.5,0.87,toplabel2,align=0.5,charsize=1.0*!p.charsize,/normal
     
; add statistics labels

zsurf = zplot
deltax = Lx/Nx
deltay = Ly/Ny
; compute the mean sea surface, which should be zero, and mean squared surface elevation
sumz = 0.0
sumzsq = 0.0
for ix = 0,Nx-1 do begin
  for iy = 0,Ny-1 do begin
    sumz = sumz + zsurf(ix,iy)
    sumzsq = sumzsq + zsurf(ix,iy)*zsurf(ix,iy)
  endfor
endfor

zavg = sumz/(float(Nx)*Float(Ny))
;print,'   avg z(x,y) (should = 0) = ',zavg

; the significant wave height from avg of z^2
avgzsq = sumzsq/(float(Nx)*Float(Ny))
H13 = 4.0*sqrt(avgzsq) ; approximate significant wave height
;print,'   significant wave height H_1/3 = 4 SQRT(avg{z^2}) =',H13

; compute the mean square slopes and mean slope angle in x and y directions
dzdx2 = 0.0
thetax = 0.0
for ix = 1,Nx-1 do begin
  for iy = 0,Ny-1 do begin
    dzdx2 = dzdx2 + (zsurf(ix,iy) - zsurf(ix-1,iy))^2
    thetax = thetax + abs(atan(zsurf(ix,iy) - zsurf(ix-1,iy),Deltax)*!radeg)
  endfor
endfor
dzdx2 = dzdx2/(Deltax*Deltax*float(Nx-1)*float(Ny))
thetax = thetax/(float(Nx-1)*float(Ny))

dzdy2 = 0.0
thetay = 0.0
for iy = 1,Ny-1 do begin
  for ix = 0,Nx-1 do begin
    dzdy2 = dzdy2 + (zsurf(ix,iy) - zsurf(ix,iy-1))^2
    thetay = thetay + abs(atan(zsurf(ix,iy) - zsurf(ix,iy-1),Deltay)*!radeg)
  endfor
endfor
dzdy2 = dzdy2/(Deltay*Deltay*float(Nx)*float(Ny-1))
thetay = thetay/(float(Nx)*float(Ny-1))

  swhlabel  = 'H!d1/3!n =' + string(H13,format='(f7.2)') + ' m'
  mssxlabel = 'mss!dx!n =' + string(dzdx2,format='(f7.3)')
  mssylabel = 'mss!dy!n =' + string(dzdy2,format='(f7.3)')
  msstotlabel = 'mss!d !n =' + string(dzdx2 + dzdy2,format='(f7.3)')
  thetaxlabel =  '!9' + string("341B) + '!x' + Greek('theta',/PS) + '!dx!n !9' $
                      + string("361B) + '!x =' + string(thetax,format='(f7.3)') + ' deg'
  thetaylabel =  '!9' + string("341B) + '!x' + Greek('theta',/PS) + '!dy!n !9' $
                      + string("361B) + '!x =' + string(thetay,format='(f7.3)') + ' deg'                      
  xlab = 0.75
  ytop = 0.85
  dylab = 0.05
  cgText,xlab,ytop,swhlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red') 
  cgText,xlab,ytop-dylab,mssxlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red') 
  cgText,xlab,ytop-2*dylab,mssylabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
  cgText,xlab,ytop-3*dylab,msstotlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
  cgText,xlab,ytop-4*dylab,thetaxlabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')
  cgText,xlab,ytop-5*dylab,thetaylabel,charsize=1.0*!p.charsize,/normal;,color=cgColor('red')




PS_End,Resize=50,/PNG ;close and fix the eps file and create a png version

print,'image file ', PlotFileName,' closed OK'

!p.multi = 0 ; reset to normal layout

print,'plotsurf_3D'
end


